#!/usr/local/bin/python

import sys, os, types
import xmlrpclib, httplib
from base64 import encodestring

class BasicAuthTransport(xmlrpclib.Transport): #{{{
	def __init__(self, username=None, password=None,verbose=0):
		self.username=username
		self.password=password
		self.verbose=verbose

	def getparser(self):
		# get parser and unmarshaller
		target = MyUnmarshaller()
		return xmlrpclib.SlowParser(target), target

	def request(self, host, handler,request_body, verbose=0):
		# issue XML-RPC request

		h = httplib.HTTP(host)
		h.putrequest("POST", handler)

		# required by HTTP/1.1
		h.putheader("Host", host)

		# required by XML-RPC
		h.putheader("User-Agent", self.user_agent)
		h.putheader("Content-Type", "text/xml")
		h.putheader("Content-Length", str(len(request_body)))

		# basic auth
		if self.username is not None and self.password is not None:
			h.putheader("AUTHORIZATION", "Basic %s" % encodestring(
			"%s:%s" % (self.username, self.password)).replace("\012", ""))
		h.endheaders()

		if request_body:
			h.send(request_body)

		errcode, errmsg, headers = h.getreply()

		if errcode != 200:
			raise xmlrpclib.ProtocolError(
				host + handler,
				errcode, errmsg,
				headers
				)
		return self.parse_response(h.getfile())

	def getparser(self):
		# get parser and unmarshaller
		target = MyUnmarshaller()
		return xmlrpclib.SlowParser(target), target

class MyUnmarshaller(xmlrpclib.Unmarshaller):
	def xml(self, encoding, standalone):
		self._encoding = 'latin-1'

#}}}

def dbprint(*args,**y): #{{{
	return
	try:
		raise 'dummy'
	except:
		frame = sys.exc_traceback.tb_frame.f_back
		locals = frame.f_locals
		globals = frame.f_globals
		functionname = frame.f_code.co_name
		filename = os.path.basename(frame.f_code.co_filename)
		lineno = frame.f_lineno
	print '>>> in file: %s; function: %s; line: %d' % (filename,functionname, lineno)
	for i in range(len(args)):
		print 'arg #%d =' % i, args[i]
	for i in y.keys():
		print "'%s' =" % i, y[i]

#}}}

# XXX Replace with some other writer...
#from timetable import readCSV

def processdata(data,marshalfn):
	records = []
	skeys = {}
	for r in data:
		rec = {}
		for k in r.keys():
			skeys[k] = None
			try:
				v = r[k].encode('latin-1')
			except:
				v = r[k]
			rec[k] = v
		records.append(rec)
	columns = [	'day',
			   	'start',
			   	'duration',
			   	'track',
			   	'title',
			   	'description',
			   	'author',
			   	'email',
			   	'company',
			   	'url',
			   	'biography',
			   	'room',
			   	'status',
				'gender',
				'pyzo',
				'talktype',
				'targetaudience',
				'year',
				'paper',
				'slides',
			   ]
	# Check
	scolumns = columns[:]
	scolumns.sort()
	skeys = skeys.keys()
	skeys.sort()
	if scolumns != skeys:
		print '*** Columns have changed!'
		K = []
		for k in skeys:
			if k not in columns: K.append(k)
		if K:
			print 'Added   Keys:',
			for k in K: print k,
			print
			K = []
		for k in columns:
			if k not in skeys: K.append(k)
		if K:
			print 'Deleted Keys:',
			for k in K: print k,
			print

	# Feed data
	d = []
	i = 0
	for r in records:
		row = []
		i += 1
		for col in columns:
			try:
				value = r[col]
			except KeyError, why:
				print '*** failed to fetch field %s from item %d: %s' % (col, i, why)
				value = ''

			# Convert Unicode to strings
			if type(value) is types.UnicodeType:
				value = value.encode('latin-1')

			# Strip strings
			if type(value) is types.StringType:
				value = value.strip()

			# Convert dates
			if col in ('paydate', 'regdate'):
				from mx.DateTime import DateTime, DateTimeFrom
				value = DateTimeFrom(value)
				if value == DateTime(1970,1,1):
					value = ''
				else:
					value = value.date

			# Handle None entries
			if value == 'None':
				value = ''

			# Handle weird amount entries
			if col == 'amount':
				if type(value) == types.ListType:
					value = value[0]
				if value:
					try:
						value = float(value)
					except ValueError:
						print '*** failed to parse amount field: %s' % repr(value)
				else:
					value = 0.0

			# Handle weird payment type entries
			if col == 'paytype':
				if value not in ('VISA', 'EUROCARD/MasterCard', 'swift', 'free'):
					value = ''

			# Reformat payresult for free entries
			if col == 'payresult' and value in ('ok', 'Ok', 'OK'):
				value = ''

			# Reformat T-Shirt entry
			if col == 'tshirt' and value == 'NO':
				value = ''

			# Add missing re_type
			if col == 'reg_type' and not value:
				amount = R['amount']['value']
				if amount and amount > 0:
					value = 'attendee'
				else:
					value = 'staff'

			row.append(value)
		d.append(row)
	d.sort()
	d.insert(0,columns)
	import marshal
	marshal.dump(d, open(marshalfn, 'wb'))
	print "dumped", marshalfn

if __name__ == "__main__":
	server=xmlrpclib.Server("http://www.europython.org/", BasicAuthTransport(username='andy',password='robinson'))
	try:
		# get all Registration since today
		#r = server.Registration.rpcExport(0)
		# get all Registration since yesterday
		#r = server.Registration.rpcExport(1)
		#get all Talks of this year
		r = server.Talks.rpcExport(90)
	except xmlrpclib.Fault,s:
		for l in str(s).split('\\n'):
			print l
		sys.exit(-1)
	processdata(r, 'talks.marshal')
