from reportlab import rl_config
from reportlab.lib.colors import Color, black, white
import os, sys

class TimeTableItem:
	names = ('day', 'start', 'duration', 'room', 'track', 'title', 'author',
			   	'email', 'company', 'url', 'biography', 'status', 'gender',
				'pyzo', 'talktype', 'targetaudience', 'year', 'description',
				)
	def __init__(self,names=[],data=[],**kw):
		map(lambda n,d,self=self: setattr(self,n,str(d)),names,data)
		self.__dict__.update(kw)

	def __repr__(self):
		L = []
		A = L.append
		for a in self.names:
			v = getattr(self,a,None)
			if v is not None: A('%s=%s'%(a,repr(v)))
		return 'TimeTableItem(%s)' % ','.join(L)

	def __str__(self):
		L = []
		A = L.append
		for a in self.names:
			v = getattr(self,a,None)
			if v is not None: A('%s=%s'%(a,str(v)))
		return 'TimeTableItem(\n\t%s,\n\t)' % ',\n\t'.join(L)

class TimeTable:
	days = ('1','2','3')
	daynames = ('Wednesday', 'Thursday', 'Friday')
	rooms = ('Room A','Room B','Room C')
	special_times = {'Coffee': (('08:30','30'),('10:30','30'),('15:00','30')),
					'Lunch': (('12:30','60'),),
					'MDFL': (('17:00','15'),),
					}
	def __init__(self,filename='talks',**kw):
		self.data = None
		self.filename = filename
		self.__dict__.update(kw)

	def addSpecial(self):
		A = self.data.append
		for day in self.days:
			for t, sd in self.special_times.items():
				for s,d in sd:
					A(TimeTableItem(day = str(day),track=t,start=s,duration=d,
									author = '',title = '',room = '',
									)
						)

	def readDATA(self):
		if self.data is None:
			if os.path.isfile('%s.marshal' % self.filename):
				import marshal
				data = marshal.load(open('%s.marshal'%self.filename,'rb'))
			elif os.path.isfile('%s.csv' % self.filename):
				import DSV
				data = open('%s.csv'%self.filename,'rb').read()
				qualifier = DSV.guessTextQualifier(_data)
				data = DSV.organizeIntoLines(_data, textQualifier = qualifier)
				delimiter = DSV.guessDelimiter(_data)
				data = DSV.importDSV(_data, delimiter = delimiter, textQualifier = qualifier)
				hasHeader = DSV.guessHeaders(data)
				if not hasHeader: raise ValueError,'talks.csv has no headers'
			else:
				print >> sys.stderr, 'No timetable data'
				data = [[]]
			names = data[0]
			self.data = []
			A = self.data.append
			for d in data[1:]:
				A(TimeTableItem(names,d))
			self.data = filter(lambda tti: tti.status=='accepted' and tti.start!='00:00',self.data)
			self.addSpecial()
			self.sort()
		return self.data

	def sort(self,data=None):
		def f(a,b):
			return cmp((a.day,a.start,a.duration,a.track),(b.day,b.start,b.duration,b.track))
		if data is None: data = self.data
		data.sort(f)

	if sys.hexversion>=0x2020000:
		def __iter__(self):
			self.readDATA()
			return iter(self.data)
	else:
		def __getitem__(self,i):
			self.readDATA()
			return self.data[i]

	def getInfoForDay(self,day,am=1):
		data = self.readDATA()[:]
		if am:
			start = '00:00'
			stop='13:00'
		else:
			start = '13:30'
			stop='18:00'

		def f(x,start=start, stop=stop, day = str(day)):
			if x.day!=day: return 0
			s = x.start
			return s>=start and s<stop

		data = filter(f,data)
		self.sort(data)
		return data

	def layoutData(self,day,am=1):
		rooms = self.rooms
		if am:
			heights = [None,0.18*72,1.8*72,0.18*72,1.8*72,0.2*72]
			startlabels = [None,"08:30","09:00","10:30","11:00","12:30"]
			stoplabels = [None,"09:00","10:30","11:00","12:30","13:30"]
			styles = (	('LINEABOVE',(0,1),(-1,-1),1,black),
						('LINEBELOW',(0,1),(-1,-1),1,black),
						('LINEBEFORE',(0,2),(-1,2),1,black),
						('LINEAFTER',(0,2),(-1,2),1,black),
						('LINEBEFORE',(0,4),(-1,4),1,black),
						('LINEAFTER',(0,4),(-1,4),1,black),
						('BACKGROUND',(0,0),(-1,-1), white),
					)
		else:
			heights = [None,1.8*72,0.18*72,1.8*72,0.1*72,0.3*72]
			startlabels = [None,"13:30","15:00","15:30","17:00","17:15"]
			stoplabels = [None,"15:00","15:30","17:00","17:15","18:00"]
			styles = (
						('LINEABOVE',(0,0),(-1,3),1,black),
						('LINEBELOW',(0,0),(-1,3),1,black),
						('LINEBEFORE',(0,1),(-1,1),1,black),
						('LINEAFTER',(0,1),(-1,1),1,black),
						('LINEBEFORE',(0,3),(-1,3),1,black),
						('LINEAFTER',(0,3),(-1,3),1,black),
						('LINEABOVE',(0,0),(-1,4),0.5,black),
						('LINEBELOW',(0,0),(-1,4),1,black),
						('BACKGROUND',(0,0),(-1,-1), white),
					)
		return rooms, heights, startlabels, stoplabels, styles

class TrackDef:
	def __init__(self,text,special=0,bg=None,fontName=None,fontSize=None, density=0.2):
		self.text = text
		self.special = special
		self.bg = Color(1+density*(bg[0]/255.-1),1+density*(bg[1]/255.-1),1+density*(bg[2]/255.-1))
		self.fontName = fontName
		self.fontSize= fontSize

TrackMap={'Coffee': TrackDef('Coffee',special=1,bg=(6,255,131)),
		'Tutorial': TrackDef('Python Tutorials',bg=(137,255,255)),
		'ZopeTutorials': TrackDef('Zope Tutorials',bg=(135,135,255)),
		'PythonInScienceAndIndustry': TrackDef('Python in Science\n& Industry',bg=(255,132,192)),
		'Lunch': TrackDef('Lunch',special=1,bg=(6,255,131)),
		'MDFL': TrackDef('',special=1,bg=(6,255,131)),
		'Zope': TrackDef('Zope',bg=(135,135,255)),
		'PythonHackers': TrackDef('Python Language',bg=(137,255,255)),
		'Keynotes':TrackDef('Keynotes',bg=(255,255,0), special=2),
		'PythonInBusiness':TrackDef('Python Business',bg=(132,132,70)),
		'PythonApplications':TrackDef('Python Applications',bg=(255,132,70)),
		'PythonFrameworks':TrackDef('Python Frameworks',bg=(200,132,70)),
		'WebServices':TrackDef('Web Services',bg=(194,194,194)),
		'ZopeBofs':TrackDef('Zope BoFs',bg=(255,255,255)),
		'BusinessBofs':TrackDef('Business BoFs',bg=(255,255,255)),
		'PythonBofs':TrackDef('Python BoFs',bg=(255,255,255)),
		'LightningTalks':TrackDef('Lightning Talks',bg=(255,255,255)),
		}

def trackFix(track):
	"simple routine to make track something suitable for use by titleAbstractBiogs"
	track = TrackMap[track].text.replace('\n',' ')
	return	track.replace('&','&amp;')

def trackDict():
	D = {}
	for k in TrackMap.keys():
		if k not in ('Coffee', 'Lunch', 'MDFL'): D[k] = trackFix(k)
	return D

def stopTime(start,duration):
	h, m = map(int, start.split(":"))
	d = int(duration)
	m += d
	while m>=60:
		h += 1
		m -= 60
	h, m = str(h), str(m)
	if len(h)<2: h = "0"+h
	if len(m)<2: m = "0"+m
	return h+":"+m

def findWidth(C,width):
	if type(C) is type([]):
		W = 0
		for P in C:
			W = max(findWidth(P,width))
	else:
		C.wrap(width,9999999)
		W = 0
		if len(C.frags)==1:
			for s,l in C.blPara.lines:
				W = max(W,width-s)
		elif len(C.frags):
			for l in C.blPara.lines:
				W = max(W,width-l.extraSpace)
		delattr(C,'blPara')
	return W

def findHeight(C,width):
	if type(C) is type([]):
		H = 0
		for P in C:
			w,h = P.wrap(width,9999999)
			delattr(P,'blPara')
			H += h
		return H
	else:
		w,h = C.wrap(width,9999999)
		delattr(C,'blPara')
		return h

def shrinkToFit(width,height,C,TS):
	if findHeight(C,width)<=height: return C
	import copy
	NTS = copy.deepcopy(TS)
	fs0 = TS.fontSize
	for i in xrange(99,0,-1):
		f = i/100.
		NTS.fontSize = TS.fontSize*f
		NTS.leading = TS.leading*f
		NTS.bulletFontSize = TS.bulletFontSize*f
		NTS.leftIndent = TS.leftIndent*f
		NTS.rightIndent = TS.rightIndent*f
		NTS.firstLineIndent = TS.firstLineIndent*f
		NTS.spaceBefore = TS.spaceBefore*f
		NTS.spaceAfter = TS.spaceAfter*f
		if type(C) is type([]):
			NC = []
			for P in C:
				NC.append(P.__class__(P.text,style=NTS))
		else:
			NC = C.__class__(C.text,style=NTS)
		if findHeight(NC,width)<=height: return NC
	return C

def subTable(width,height,P):
	from reportlab.platypus.tables import Table
	widths = [width]
	heights = [height]
	w = findWidth(P,0.8*width)
	h = findHeight(P,w)
	if h>0.95*height:
		P = shrinkToFit(w,0.95*height,P,P.style)
		h = findHeight(P,w)
	topPadding = bottomPadding = (height - h)*0.5 - .1
	leftPadding = rightPadding = 0.5*(width-w)
	tstyle = [
			('VALIGN',(0,0),(-1,-1),'TOP'),
			('BOTTOMPADDING',(0,0),(-1,-1),bottomPadding),
			('TOPPADDING',(0,0),(-1,-1),topPadding),
			('LEFTPADDING',(0,0),(-1,-1),leftPadding),
			('RIGHTPADDING',(0,0),(-1,-1),rightPadding),
			]
	return Table([[P]],colWidths=widths,rowHeights=heights,style=tstyle)

from reportlab.graphics.shapes import Drawing
from reportlab.graphics.widgets.grids import ShadedRect
class BGShadedRect(Drawing):
	def __init__(self,width=0,height=0,color=None):
		Drawing.__init__(self,width,height)
		self.add(ShadedRect(),name='sr')
		self._color = color
	def wrap(self,availHeight,availWidth):
		return 0, 0
	def __call__(self,tbl,canv,x,y,w,h):
		old = self.width, self.height
		self.width, self.height = w, h
		self.sr.width, self.sr.height = self.width, self.height
		self.sr.strokeColor, self.sr.fillColorStart, self.sr.fillColorEnd = None, white, self._color
		self.drawOn(canv,x,y)
		self.width, self.height = old

def _getTimeTable(TT,desiredDay,am,noRoomLabels=None):
	from reportlab.platypus.tables import Table, Paragraph, PageBreak, KeepTogether
	from reportlab.lib.styles import ParagraphStyle

	TS = ParagraphStyle('tt_title',fontSize=10,leading=10*1.2)
	data = TT.getInfoForDay(desiredDay,am)
	leftPadding = rightPadding = topPadding = bottomPadding = 3
	tstyle = [('FONT',(0,0),(-1,-1),TS.fontName,TS.fontSize,TS.leading),
			('VALIGN',(0,0),(-1,-1),'TOP'),
			('ALIGN',(0,0),(-1,0),'CENTER'),
			('BOTTOMPADDING',(0,0),(-1,-1),bottomPadding),
			('TOPPADDING',(0,0),(-1,-1),topPadding),
			('LEFTPADDING',(0,0),(-1,-1),leftPadding),
			('RIGHTPADDING',(0,0),(-1,-1),rightPadding),
			('BOX',(0,0),(-1,-1),2,black),
			]
	row = 0
	rooms, heights, starts, stops, styles = TT.layoutData(desiredDay,am)
	map(tstyle.append,styles)
	nCols = len(rooms)
	widths = nCols*[451/float(nCols)]
	nRows = len(heights)
	tdata = [noRoomLabels and (['']*nCols) or rooms]
	for i in heights:
		if i is None: continue
		tdata.append((nCols*[''])[:])
	for d in data:
		start,duration,track,author,title,room = d.start,d.duration,d.track,d.author,d.title,d.room
		row  = None
		for i in xrange(nRows):
			if starts[i] is not None and start>=starts[i] and start<=stops[i]: 
				row = i
		if row is None:
			print >>sys.stderr, "No row for "+title
			continue
		track = TrackMap[track]
		stop = stopTime(start,duration)
		if track.special:
			if track.special==1:
				text = "<b>%s-%s</b>: <font size=+1>%s</font>"%(start,stop,track.text)
			else:
				text = "<font size=-1><b>%s-%s</b></font>: \"%s\", <i>%s</i>."%(start,stop,title, author)
			if track.text!='': tdata[row] = [subTable(451,heights[row],Paragraph(text,TS))]+(nCols-1)*['']
			for s in (('BACKGROUND',(0,row),(-1,row), BGShadedRect(color=track.bg)),
						('TOPPADDING',(0,row),(-1,row),0),
						('LEFTPADDING',(0,0),(-1,-1),leftPadding),
						):
				tstyle.append(s)
		else:
			col = ord(room.lower())-ord('a')
			cell = tdata[row][col]
			if cell=='':
				cell = []
				tstyle.append(('BACKGROUND',(col,row),(col,row),BGShadedRect(color=track.bg)))
			cell.append(Paragraph("<font size=-1><b>%s-%s</b></font>: \"%s\", <i>%s</i>."%(start,stop,title, author),style=TS))
			tdata[row][col] = cell
	for row in xrange(nRows):
		for col in xrange(len(widths)):
			cell = tdata[row][col]
			if type(cell) is type([]):
				tdata[row][col] = shrinkToFit(widths[col]-leftPadding-rightPadding,heights[row]-topPadding-bottomPadding,cell,TS)
	return Table(tdata,style=tstyle,colWidths=widths,rowHeights=heights)

def getTimeTable(TT):
	from reportlab.platypus import Spacer, PageBreak
	L = []
	for day in TT.days:
		for am in 1,0:
			map(L.append,(Spacer(0,0.5*72),_getTimeTable(TT,day,am),PageBreak()))
	return L

def getURLImage(url,width=None,height=None):
	from reportlab.platypus import Image
	from reportlab.lib.utils import open_for_read
	import timetable, os, urlparse
	URL = urlparse.urlparse(url)
	cache_dir = os.path.join(os.path.dirname(timetable.__file__),'url_cache')
	if not os.path.isdir(cache_dir): os.mkdir(cache_dir)
	cache_fn = os.path.join(cache_dir,URL[0]+'_'+URL[1]+URL[2].replace('/','_'))
	if int(os.environ.get('RL_GETIMAGES','0')) or not os.path.isfile(cache_fn):
		open(cache_fn,'wb').write(open_for_read(url).read())
	return Image(cache_fn,width=width,height=height)

def getTTImage(day,width=None,height=None):
	return getURLImage("http://europython.zope.nl/images/schedule_day"+`day`,width=width,height=height)

def titleAbstractBiogs(TT,styles,T):
	if T is None: T = trackDict().keys()
	from reportlab.platypus import Paragraph, Spacer, CondPageBreak
	D = []
	for d in TT:
		t = d.track
		if t in T:
			D.append((trackFix(t), d.day, d.start, d.title,d.author,d.description, d.biography))
	D.sort()

	story=[]
	biogs={}
	cTrack = None
	for d in D:
		track, day, start, title, author, abstract, biog = d
		if title.find('Part II')>=0: continue
		title = title.replace(': Part I','')
		if title and author and abstract:
			story.append(CondPageBreak(36))
			if cTrack!=track:
				cTrack = track
				story.append(Paragraph(track,styles['h3']))
			story.append(Paragraph("<b>%s</b>, %s" % (title, author),styles['Normal']))
			story.append(Paragraph("day: %s, time: %s" % (day, start),styles['Normal']))
			story.append(Spacer(height=3, width=10))
			story.append(Paragraph(abstract,styles['2col']))
			if biog:
				if not biogs.has_key(author):
					story.append(Spacer(0,12))
					story.append(Paragraph(biog,styles['2col']))
					biogs[author] = 1
			story.append(Spacer(0,18))
	return story

def makeTT(TT,fn='postertimetable.pdf',pagesize='A0'):
	from reportlab.pdfgen import canvas
	from reportlab.lib import pagesizes
	from reportlab.lib.units import inch

	A4 = pagesizes.A4
	AX = getattr(pagesizes,pagesize)
	sx = AX[0]/float(A4[0])
	sy = AX[1]/float(A4[1])
	c = canvas.Canvas(fn,pagesize=AX)

	i = 0
	top = A4[1] - inch
	h = top-inch
	h2 = 0.5*h
	y = [top-h,top-h2]
	w = A4[0]-inch
	for day in TT.days:
		c.saveState()
		c.setFont('Helvetica',14)
		c.scale(sx,sy)		#we'll draw to a nominal A4
		c.drawString(inch,A4[1]-inch,TT.daynames[i])
		for am in (1,0):
			T = _getTimeTable(TT,day,am,noRoomLabels=not am)
			wt,ht = T.wrap(w,h2)
			if not am:
				dy = max(h2-ht,0)
			else:
				dy = 0
			T.drawOn(c,inch,y[am]+dy)
		i += 1
		c.restoreState()
		c.showPage()
	c.save()

def main():
	TT = TimeTable()
	if '--print' in sys.argv:
		S = []
		for tti in TT:
			S.append(str(tti))
		print 'Current Data:'
		print '\n\n'.join(S)
	elif '--poster' in sys.argv:
		makeTT(TT)
	else:
		from reportlab.platypus import SimpleDocTemplate
		lst = getTimeTable(TT)
		from brochure import makeNewStyleSheet
		titleAbstractBiogs(TT,makeNewStyleSheet(),lst)
		SimpleDocTemplate('timetable.pdf',showBoundary=1).build(lst)

if __name__=='__main__':	#noruntests
	main()
