from reportlab.lib import colors
from reportlab.lib.validators import *
from reportlab.lib.attrmap import *
from reportlab.graphics.shapes import Group, Line
from reportlab.graphics.widgetbase import Widget
from reportlab.graphics.charts.textlabels import Label

def _getTime(t):
	T = map(int,t.split(':'))
	return (T[0]+float(T[1])/60.0)*100

class TTWidget(Widget):
	_nodoc = 1
	_attrMap = AttrMap(
		x = AttrMapValue(isNumber,desc='x coordinate'),
		y = AttrMapValue(isNumber,desc='y coordinate'),
		width = AttrMapValue(isNumber,desc='width'),
		height = AttrMapValue(isNumber,desc='height'),
		fillColor = AttrMapValue(isColorOrNone),
		strokeColor = AttrMapValue(isColorOrNone),
		strokeWidth = AttrMapValue(isNumber),
		mainTimes = AttrMapValue(isListOfNoneOrString,desc="String times in 24 hour format"),
		mainTimesFontName = AttrMapValue(isString,desc="Name of the font to use in side bar"),
		mainTimesFontSize = AttrMapValue(isNumber,desc="Size of the font to use in side bar"),
		title = AttrMapValue(None,desc="mainTitle"),
		)
	def __init__(self):
		self.x = 0
		self.y = 0*.2*72
		self.width = 6.26*72
		self.height = 9.6*72
		self.mainTimes=("9:00","10:00","11:00","12:00","13:00","14:00","15:00","16:00","17:00","18:00")
		self.strokeColor = colors.black
		self.strokeWidth = 0.5
		self._times={}
		self._allSlots=[]
		self.mainTimesFontName = 'Times-Roman'
		self.mainTimesFontSize = 7

	def addToSlot(self,slot,start,stop,text,bg=None):
		T = self._times.get(slot,[])
		T.append((t,text,bg))
		self._times[slot] = T

	def addAllSlots(self,start,stop,text,bg=None, fontName=None, fontSize=None):
		self._allSlots.append((start,stop,text,bg, fontName, fontSize))

	def draw(self):
		xLoc = (0.26*72, 2.26*72, 4.26*72)
		minT = map(_getTime,self.mainTimes)
		minT, maxT = min(minT), max(minT)
		y0, y1 = 0.2*72, self.height-0.4*72
		yScale = -(y1-y0)/(maxT-minT)
		g = Group()
		for x in 0,0.26,2.26,4.26,6.26:
			g.add(Line(x*72,y0,x*72,y1,strokeColor=self.strokeColor,strokeWidth=self.strokeWidth))

		for t in self.mainTimes:
			l = Label()
			l.x = 0.26*72
			l.y = y1+(_getTime(t)-minT)*yScale
			l.boxAnchor = 'e'
			l.fontSize = self.mainTimesFontSize
			l.fontName = self.mainTimesFontName
			l.setText(t)
			l.dx = -1
			g.add(l)
			g.add(Line(l.x-1,l.y,l.x+1,l.y,strokeColor=self.strokeColor,strokeWidth=self.strokeWidth))

		for start,stop,text,bg, fontName, fontSize in self._allSlots:
			l=Label()
			l.x = xLoc[0]
			l.y = y1+(_getTime(start)-minT)*yScale
			y = y1+(_getTime(stop)-minT)*yScale
			l.height = max(l.y,y) - min(l.y,y)
			l.boxAnchor = 'nw'
			l.width = self.width - l.x
			l.boxFillColor = bg
			l.boxStrokeColor = self.strokeColor
			l.boxStrokeWidth = self.strokeWidth
			l.textAnchor = 'middle'
			l.fontSize = fontSize or self.mainTimesFontSize+2
			l.fontName = fontName or self.mainTimesFontName
			l.setText(text)
			g.add(l)

		return g
