#!/usr/bin/python
"""Creates a brochure for
European Python and Zope Conference 2003 June 25-27, 2003.

Uses getURLImage from timetable.py. To force this to update all
images, set the environment variable RL_GETIMAGES = 1.
"""

from reportlab.pdfgen import canvas
from reportlab.lib.pagesizes import A4, letter
from reportlab.lib.units import inch, cm
from reportlab.lib.colors import red, white
from rlextra.pageCatcher.pageCatcher import restoreForms
from reportlab.platypus.frames import Frame
from reportlab.lib.styles import ParagraphStyle, StyleSheet1
from reportlab.lib.enums import TA_LEFT, TA_CENTER, TA_JUSTIFY, TA_RIGHT
from reportlab.platypus import *
from reportlab.platypus import tableofcontents
from reportlab.rl_config import verbose
import os.path, sys, time
import timetable

_DAYS = ('1','2','3')
_ROOMS = ('Room A','Room B','Room C')
_DAYNAMELIST = ("Wednesday, June 25", "Thursday, June 26", "Friday, June 27")
PAUSE=None
adnum=None
TT=None
started = time.time()
adnum=-1

def log(msg):
    """used for profiling"""
    global elapsed
    elapsed = time.time() - started
    print '%0.2f sec: %s' % (elapsed, msg)
    if PAUSE:
        raw_input('...any key to continue')

def makeNewStyleSheet():
    #if verbose: log ("\tstarted making stylesheet")
    """Returns a stylesheet object"""
    stylesheet = StyleSheet1()

    #normal styles

    stylesheet.add(ParagraphStyle(name='Normal',
                                  fontName='Helvetica',
                                  fontSize=8,
                                  leading=9.6)
                   )

    stylesheet.add(ParagraphStyle(name='BodyText',
                                  parent=stylesheet['Normal'],
                                  spaceBefore=6),
                   alias='body')

    stylesheet.add(ParagraphStyle(name='Heading1',
                                  parent=stylesheet['Normal'],
                                  fontName = 'Helvetica-Bold',
                                  fontSize=16,
                                  leading=20,
                                  spaceAfter=6),
                   alias='h1')

    stylesheet.add(ParagraphStyle(name='Title',
                                  parent=stylesheet['Normal'],
                                  fontName = 'Helvetica-Bold',
                                  fontSize=16,
                                  leading=20,
                                  alignment=TA_CENTER,
                                  spaceAfter=6),
                   alias='title')

    stylesheet.add(ParagraphStyle(name='Heading2',
                                  parent=stylesheet['Normal'],
                                  fontName = 'Helvetica-Bold',
                                  alignment = TA_CENTER,
                                  fontSize=14,
                                  leading=18,
                                  spaceAfter=6),
                   alias='h2')

    stylesheet.add(ParagraphStyle(name='Heading3',
                                  parent=stylesheet['Normal'],
                                  fontName = 'Helvetica-BoldOblique',
                                  fontSize=10,
                                  leading=12,
                                  spaceBefore=12,
                                  spaceAfter=6),
                   alias='h3')

    stylesheet.add(ParagraphStyle(name='Heading4',
                                  parent=stylesheet['Normal'],
                                  fontName = 'Helvetica-Bold',
                                  spaceBefore=12,
                                  spaceAfter=6),
                   alias='h4')

    stylesheet.add(ParagraphStyle(name='Bullet',
                                  parent=stylesheet['Normal'],
                                  firstLineIndent=0,
                                  spaceBefore=3,
                                  leftIndent = 8,
                                  bulletFontName='ZapfDingbats',
                                  bulletFontSize=6),
                   alias='bu')

    stylesheet.add(ParagraphStyle(name='Definition',
                                  parent=stylesheet['Normal'],
                                  firstLineIndent=0,
                                  leftIndent=36,
                                  bulletIndent=0,
                                  spaceBefore=6,
                                  bulletFontName='Times-BoldItalic'),
                   alias='df')

    stylesheet.add(ParagraphStyle(name='Code',
                                  parent=stylesheet['Normal'],
                                  fontName='Courier',
                                  fontSize=8,
                                  leading=8.8,
                                  firstLineIndent=0,
                                  leftIndent=36))

    # 'Big' styles - varients based on a font size of 10 rather than 8
    stylesheet.add(ParagraphStyle(name='NormalBig',
                                  parent=stylesheet['Normal'],
                                  fontSize=10,
                                  leading=12),
                   alias='bNormal')

    stylesheet.add(ParagraphStyle(name='BodyBig',
                                  parent=stylesheet['BodyText'],
                                  fontSize=10,
                                  leading=12),
                   alias='bbody')
                   
    stylesheet.add(ParagraphStyle(name='BodyBigRight',
                                  parent=stylesheet['BodyBig'],
                                  alignment=TA_RIGHT),
                   alias='bbodyr')
                   
    stylesheet.add(ParagraphStyle(name='BodyBigRightSmallerFont',
                                  parent=stylesheet['BodyBigRight'],
                                  fontSize=8),
                   alias='bbodyrs')      
                   
    stylesheet.add(ParagraphStyle(name='Heading3Big',
                                  parent=stylesheet['Heading3'],
                                  fontSize=12,
                                  leading=14),
                   alias='h3b')

    stylesheet.add(ParagraphStyle(name='Heading4Big',
                                  parent=stylesheet['Normal'],
                                  fontName = 'Helvetica-Bold',
                                  fontSize=10,
                                  leading=12,
                                  spaceBefore=12,
                                  spaceAfter=6),
                   alias='h4b')

    stylesheet.add(ParagraphStyle(name='BigBullet',
                                  parent=stylesheet['bNormal'],
                                  firstLineIndent=0,
                                  spaceBefore=3,
                                  bulletFontName='ZapfDingbats',
                                  bulletFontSize=6),
                   alias='bbu')
    # big indented styles - indented versions of above

    stylesheet.add(ParagraphStyle(name='NormalBigIndented',
                                  parent=stylesheet['NormalBig'],
                                  leftIndent=24),
                   alias='biNormal')

    stylesheet.add(ParagraphStyle(name='BodyBigIndented',
                                  parent=stylesheet['BodyBig'],
                                  leftIndent=24),
                   alias='bibody')

    stylesheet.add(ParagraphStyle(name='BigBulletIndented',
                                  parent=stylesheet['BigBullet'],
                                  leftIndent=36,
                                  bulletIndent=24),
                   alias='bibu')


    # styles specifically for two column layouts
    stylesheet.add(ParagraphStyle(name='TwoColumnJustified',
                                  parent=stylesheet['Normal'],
                                  alignment=TA_JUSTIFY),
                   alias='2col')

    stylesheet.add(ParagraphStyle(name='TwoColumnJustifiedVariant',
                                  parent=stylesheet['BodyText'],
                                  alignment=TA_JUSTIFY),
                   alias='2cola')

    stylesheet.add(ParagraphStyle(name='TwoColumnJustifiedVariantRight',
                                  parent=stylesheet['TwoColumnJustifiedVariant'],
                                  alignment=TA_RIGHT),
                   alias='2colar')
                   
    # varients that won't show up in TOCs
    stylesheet.add(ParagraphStyle(name='Heading2a',
                                  parent=stylesheet['Heading2']),
                   alias='h2a')

    stylesheet.add(ParagraphStyle(name='Heading3a',
                                  parent=stylesheet['Heading3']),
                   alias='h3a')

    #if verbose: log ("\tstylesheet complete\n")
    return stylesheet

styles = makeNewStyleSheet()


def myTitlePage(c, doc):
    global adnum
    if verbose: log ("\tinside function 'myTitlePage'")
    adnum=-1
    c.saveState()
    c.bookmarkPage("cover")

    #do all restoreForms here so they don't get repeated anywhere else...
    #restoreForms('ep_logo_alternative_1.data', c, verbose=0) #no longer required?
    if verbose: log ("\tstarted restoreForms in 'myTitlePage' ('brochure_cover.data')")
    restoreForms('brochure_cover.data', c, verbose=0)
    if verbose: log ("\trestoreForms complete ('brochure_cover.data')\n")
    if verbose: log ("\tstarted restoreForms in 'myTitlePage' ('first_floor_ceme.data')")
    restoreForms('first_floor_ceme.data', c, verbose=0)
    if verbose: log ("\trestoreForms complete ('first_floor_ceme.data')\n")
    if os.path.isfile('brochure_template.data'):
        if verbose: log ("\tstarted restoreForms in 'myTitlePage' ('brochure_template.data')")
        restoreForms('brochure_template.data', c, verbose=0)
    if verbose: log ("\trestoreForms ('brochure_template.data') complete\n")
##    for ad in ['OrbiTeam.data','egenix.data','Aragne.data','Heracles.data', 'asdu.data', 'logilab.data', 'infrae.data']:
    for ad in ['Aragne.data', 'asdu.data', 'zeomega.data']:
        if verbose: log ("\tstarted restoreForms in 'myTitlePage' ('%s')" % ad)
        restoreForms(os.path.join('adverts',ad), c, verbose=0)
        if verbose: log ("\trestoreForms ('%s') complete\n" % ad)

    xtrans = -(A4[0]/inch)+8
    ytrans = -(A4[1]/inch)+15
    c.translate(xtrans,ytrans)
    if verbose: log ("\tstarted doform in 'myTitlePage' ('front_PF0')")
    c.doForm('front_PF0')
    if verbose: log ("\tdoform completed ('front_PF0') completed\n")
    c.restoreState()
    if verbose: log ("\tfunction 'myTitlePage' completed\n")

    c.addOutlineEntry("European Python and Zope Conference 2003", "cover", 0, None)

def myLaterPages(c, doc):
    if os.path.isfile('brochure_template.data'):
        if not c.hasForm('PF0'): restoreForms('brochure_template.data', c, verbose=0)
        xtrans = -(A4[0]/inch)+8
        ytrans = -(A4[1]/inch)+15
        c.saveState()
        c.translate(xtrans,ytrans)
        c.doForm('PF0')
        c.restoreState()
    #page decoration - footer, pagenumbers etc
    c.saveState()
    c.setFont("Helvetica-BoldOblique", 10)
    #c.drawString(A4[0]/2.0, 790, str(c.getPageNumber()))#centered
    c.drawString(550, 790, str(c.getPageNumber()))#right hand edge
    c.drawString(38, 34, "Charleroi - Belgium")
    c.drawRightString(553, 34, "European Python and Zope Conference 2003")
    c.setLineWidth(1)
    c.line(39,779,556,779)
    c.line(39,51,556,51)
    c.restoreState()

def getAdvert(c,adnum):
    if verbose: log('\tinside function getAdvert')

    ads = ['ad0_PF0','ad1_PF0', 'ad2_PF0']
    if adnum >= len(ads):
        #none to draw
        return
    c.saveState()
    if adnum==0:
        # Aragne
        w = 553 # size - for background rect
        h = 379
        x = 451.0/553
        y = 348.0/379
        bx=x; by=y
        dx = (1.00*inch)*x
        dy = 0.40*inch

    elif adnum==1:
        #asdu
        w = 462 # size - for background rect
        h = 353
        x = 451.0/515 #scale
        x=x*1.01
        y = 348.0/402
        bx= inch/2.0
        by= 402
        dx = ((0.6*inch)*x) # translation
        dx=dx*0.97
        dy = -((A4[1]-inch-412-23)*y)

    elif adnum==2:
        #zeomega
        w = 462 # size - for background rect
        h = 353
        x = 451.0/515 #scale
        x=x*1.01
        y = 348.0/402
        bx= inch/2.0
        by= 402
        dx = ((0.6*inch)*x) # translation
        dx=dx*0.97
        dy = -1.40*inch

    if adnum<7:
        c.translate(dx, dy)
        c.scale(x, y)
        c.setFillColor(white)
        c.setStrokeColor(red)
        c.rect(bx, by, w, h, stroke=0, fill=1)
    if verbose: log("\tstarted doform for advert #%s ('%s')" % (adnum, ads[adnum]))
    c.doForm(ads[adnum])
    c.drawRightString(553, 34, "European Python and Zope Conference 2003")
    if verbose: log("\tfinished doform #%s('%s')" % (adnum, ads[adnum]))
    c.restoreState()
    if verbose: log('\tfunction getAdvert completed\n')

def myAdsPage(c, doc):
    myLaterPages(c,doc)
    global adnum
    adnum=adnum+1
    getAdvert(c,adnum)

def myFloorPlanPage(c, doc):
    myLaterPages(c,doc)
    c.saveState()
    xtrans = -(A4[0]/inch)+inch
    ytrans = -(A4[1]/inch)+(4.5*inch)
    c.translate(xtrans,ytrans)
##    if verbose: log ("\tstarted doForm for FloorPlanPage ('ff_PF0')")
##    c.doForm('ff_PF0')
##    if verbose: log ("\tdoForm complete ('ff_PF0')\n")
    c.restoreState()
##    getAdvert(c, 6)


def getCoverPage(genby, genfor):
    if genby or genfor:
        story.append(Spacer(height=24, width=10))
        if genby:
            story.append(Paragraph(("""<para align="CENTER" fontSize="24" leading="30"><b>Generated by:</b>\t%s</para>""" % genby), style=styles["bbody"]))
        if genfor:
            story.append(Paragraph(("""<para align="CENTER" fontSize="24" leading="30"><b>Generated for:</b>\t%s</para>""" % genfor), style=styles["bbody"]))
        #story.append(FrameBreak())
    story.append(NextPageTemplate('OneCol'))

def getWelcomeLetter():
    if verbose: log ("\tstarted function getWelcomeLetter")

    newPage()
    story.append(Paragraph('Welcome to EuroPython Conference 2003 (EPC2003)<onDraw name="myOnDrawCB" label="Welcome letter"/>', style=styles["h2"]))

    story.append(Spacer(height=12, width=10))

    story.append(Paragraph("""This
is the second edition of our volunteer-organized event, focused on
bringing together Python and Zope developers across Europe and beyond.
""", style=styles["bbody"]))

    story.append(Paragraph("""EPC2003 is also an opportunity for those that are new to
these technologies to learn more, about both the software and the
people behind it.  The conference remains the only major European
event dedicated solely to the Python programming language and its
applications.""", style=styles["bbody"]))

    story.append(Paragraph("""This second EuroPython conference is now a fact! The entire
conference team worked hard to make EPC2003 as good as the original
and we hope that we have succeeded.  The conference has more
talks/tutorials (over 70) than last year with three parallel tracks.
Not to mention the luminaries and friends from across the world of Python,
with whom you can chat and share stories.""", style=styles["bbody"]))

    story.append(Paragraph("""Now that the second edition of this event is underway, we
can say that EuroPython is an event to stay. An event to which
every interested person can look forward to for years to come.
And this was the goal to achieve: creating a solid, new Python event, at
a low price, built by Python folks for other Python folks, with a
promising future ahead.""", style=styles["bbody"]))

    story.append(Paragraph("""Though we faced long and scary days in preparation, we
deeply hope that you'll enjoy this second event.  And we deeply thank
all of you for coming to Charleroi to help move Python forward,
together.""", style=styles["bbody"]))

    story.append(Spacer(height=12, width=10))

    story.append(Paragraph("""Have fun,""", style=styles["bbody"]))

    story.append(Paragraph("""The EuroPython Team""", style=styles["bbody"]))

    story.append(Spacer(height=24, width=10))
    if verbose: log ("\tfunction getWelcomeLetter complete\n")


def getTOC():
    """Make an automatically generated Table of Contents"""
    if verbose: log ("\tstarted function getTOC")
    newPage()
    story.append(Paragraph('Table Of Contents<onDraw name="myOnDrawCB" label="Contents"/>', style=styles["h2a"]))
    #make styles for Table of Contents
    tocLevelStyles = []
    d, e = tableofcontents.delta, tableofcontents.epsilon
    for i in range(0,5):
        tocLevelStyles.append(makeTocHeaderStyle(i, d, e))
    toc = tableofcontents.TableOfContents()
    toc.levelStyles = tocLevelStyles
    story.append(toc)
    if verbose: log ("\tfunction getTOC complete\n")


def _init_TT():
    return timetable.TimeTable(days=_DAYS, rooms=_ROOMS, daynames=_DAYNAMELIST)

def init_TT():
    global TT
    if TT is None: TT=_init_TT()

def getTimetable():
    """Uses timetable.py to automatically generate a timetable from the CSV file."""
    init_TT()
    if verbose: log ("\tstarted function getTimetable")
    story.append(NextPageTemplate('AdPage'))
    for day in ("1", "2", "3"):
        for AM in 1,0:
            tday = _DAYNAMELIST[int(day)-1]
            #if day=="3" and not AM: continue
            newPage()
            if day=="1" and AM:
                story.append(Paragraph('Timetable<onDraw name="myOnDrawCB" label="Timetable"/>', style=styles["h2"]))
            else:
                pass
                #story.append(Paragraph('Timetable', style=styles["h2a"]))
            story.append(Paragraph('%s %s<onDraw name="myOnDrawCB" label="Day %s - %s"/>'% (tday, AM and "(morning)" or "(afternoon)", day, AM and "AM" or "PM"), style=styles["h2a"]))
            story.append(timetable._getTimeTable(TT,day,AM))
    story.append(Spacer(height=6, width=10))
##    story.append(Paragraph(("""<b>NB</b> The times given here are not
##binding -- the entries are merely to give a rough outline of which
##topics are planned. It is the nature of a BoF/LT day to have
##short-notice changes."""), style=styles["body"]))

    story.append(NextPageTemplate('OneCol'))
    if verbose: log ("\tfunction getTimetable complete\n")


def getTalkDetails(tracks):
    """Uses timetable.py to automatically generate the talk details section from the CSV file."""
    if verbose: log ("\tstarted function getTalkDetails")
    init_TT()
    L=timetable.titleAbstractBiogs(TT,styles,tracks)
    if L:
        story.append(NextPageTemplate('TwoColPlus'))
        newPage()
        #story.append(Paragraph('Talks and Speakers Details', style=styles["h2"]))
        story.append(Paragraph('Talks and Speakers Details<onDraw name="myOnDrawCB" label="Talks and Speakers"/>', style=styles["h2"]))
        story.append(NextPageTemplate('TwoCol'))
        map(story.append,L)
    if verbose: log ("\tfunction getTalkDetails complete\n")

def getTravelInfo():
    """Function to add the Travel Information page to the brochure.
    Info taken directly from the web site on 19 May 2003."""

    if verbose: log ("\tstarted function getTravelInfo")

    story.append(NextPageTemplate('OneCol'))
    newPage()
    #This bit from the Travel Information page
    story.append(Paragraph('Travel Information<onDraw name="myOnDrawCB" label="Travel Information"/>', style=styles["h2"]))

    URL = 'http://europython.zope.nl/information/travel/images/europe'
    story.append(Spacer(height=6, width=10))
    story.append(timetable.getURLImage(URL, width=200.0*(100.0/137), height=100))
    story.append(Spacer(height=6, width=10))

    story.append(Paragraph(("""With one airport closeby and another just reachable within an hour, a
Railway station with a connection to the High Speed Train and several
motorways, Charleroi is easily reachable."""), style=styles["bbody"]))

    story.append(Paragraph(("""Once in Charlerloi, public transport can bring you to almost
everywhere."""), style=styles["bbody"]))

    story.append(Paragraph(("""For more information on the location of the conference, please have a
look at the location webpage."""), style=styles["bbody"]))

    story.append(Paragraph(("Getting to Charleroi:"), style=styles["h3a"]))

    story.append(Paragraph(("By road"), style=styles["h4b"]))
    story.append(Paragraph(("E19 from Brussels, Antwerp and Amsterdam, in the North"), style=styles["bbody"]))
    story.append(Paragraph(("E 42 from Lille, Paris and the Euro Tunnel, in the West"), style=styles["bbody"]))
    story.append(Paragraph(("E 42 from Namur, Lie`ge and Cologne, in the East"), style=styles["bbody"]))

    story.append(Paragraph(("By train"), style=styles["h4b"]))
    story.append(Paragraph(("""Arrival at Charleroi Station, right at the heart of the city centre.
Train links from all big European cities to Charleroi. For more
information, visit SNCB"""), style=styles["bbody"]))

    story.append(Paragraph(("""Charlerloi Station has also a connection with the TGV (high-speed
train) network"""), style=styles["bbody"]))

    story.append(Paragraph(("By air"), style=styles["h4b"]))
    story.append(Paragraph(("""Arrival at Brussels Airport in Zaventem. Charleroi is easily
accessible by road, after an hours drive on the E 19 or by train,
from the airport via Brussels."""), style=styles["bbody"]))

    story.append(Paragraph(("""The train trip will take aprox. 1 hour 25 minutes, the ticket
(Zaventem Airport - Charleroi Sud) costs around 8,30 EUR. The times
below may vary by 5 minutes, so be sure to be 10 minutes earlier at the
train station."""), style=styles["bbody"]))


    story.append(Paragraph(("Time Schedules from Zaventem Airport to Charleroi Sud "), style=styles["h4b"]))
    story.append(Table(
        [
            [Paragraph(("Zaventem Airport"), style=styles["bbody"]),
             Paragraph(("Every 05' or 30' past the hour "), style=styles["bbody"])],
            [Paragraph(("Arrival Brussels North (Bruxelles Nord)"), style=styles["bbody"]),
             Paragraph(("Every 20' or 52' past the hour "), style=styles["bbody"])],
            [Paragraph(("Departure to Charleroi"), style=styles["bbody"]),
             Paragraph(("Every 32' or 01' past the hour "), style=styles["bbody"])],
            [Paragraph(("Arrival at Charleroi"), style=styles["bbody"]),
             Paragraph(("Every 26' or 53' past the hour "), style=styles["bbody"])]
            ],
                 colWidths=[3.5*inch,2*inch],
                 rowHeights=[12,12,12,12],
                 style=None,
                 splitByRow=1,
                 repeatRows=0,
                 repeatCols=0)
                 )

    story.append(Paragraph(("""Regional Airport "Brussels South Charleroi", Building S11B, 6041
Charleroi (Gosselies) phone 32(0)71 25.12.11"""), style=styles["bbody"]))

    story.append(Paragraph(("""If flying from other parts of Europe on a budget, strongly consider
taking one of the budget airlines (www.ryanair.com, www.easyjet.com,
www.go-fly.com) to Stansted and out to Charleroi. This would cover
about 30 locations in Europe..."""), style=styles["bbody"]))
    story.append(Paragraph(("From the USA, consider staging through London... "), style=styles["bbody"]))

    newPage()
    #This bit from the Location of the Conference Building page
    story.append(Paragraph('Location of the Conference Building<onDraw name="myOnDrawCB" label="Building Location"/>', style=styles["h2"]))

    #story.append(Paragraph(("If you are the owner of a Palm OS 3.5 or compatible, then the following mappy files could be very usefull (avaliable from the
    story.append(Paragraph(("""The conference will take place in
    Belgium, Charleroi to be more precise. The building is called
    "CEME" (Charleroi Espace Meeting Europen). You can find more
    information about it on their website. You may see a huge
    image of the surroundings by clicking here."""), style=styles["bbody"]))

    story.append(Paragraph(("""You can reach the conference building
    by car or by bus:"""), style=styles["bbody"]))

    story.append(Paragraph(("By bus (TEC):"), style=styles["h4b"]))
    story.append(Paragraph(("""Starting at the 'South' railway
    station of Charlerloi ('gare du Sud'), you have two
    possibilities (A bus ticket costs around 1 Euro):"""), style=styles["bbody"]))
    story.append(Paragraph(("""Take any bus, indicating the
    direction 'Chausse de Bruxelles'. Exit the bus at bus stop
    'La Planche', traverse the road. On the other side of the
    road, follow the street 'rue des Franais'. You will find the
    CEME at the end of this street (approximately 5 minutes on
    foot)."""), style=styles["bbody"]))

    story.append(Paragraph(("""Take the bus with line-number 41
    heading for 'Courcelles' (less frequent than the above
    possibility). The trip will take about 10 minutes. Exit the
    bus at bus stop 'Ravin' and take the street 'Gohyssart' (rue de
    Gohyssart) which leads to the CEME (approximately 3 minutes on
    foot)."""), style=styles["bbody"]))

    story.append(Paragraph(("By car"), style=styles["h4b"]))
    story.append(Paragraph(("Please, have a look at the map below."), style=styles["bbody"]))
    URL = 'http://europython.zope.nl/images/map_ceme'
    story.append(timetable.getURLImage(URL, width=891.0*(400.0/1181), height=400))
    if verbose: log ("\tfunction getTimetable complete\n")

def getTransportationInfo():
    """Function to add the Transportation Information page to the brochure.
    Info taken directly from the web site on 13 June 2003."""
    from reportlab.lib.colors import black, red, green, blue, gray
    if verbose: log ("\tstarted function getTransportationInfo")

    story.append(NextPageTemplate('OneCol'))
    newPage()
    #This bit from the Transportation page
    story.append(Paragraph('Transportation Information<onDraw name="myOnDrawCB" label="Transportation Information"/>', style=styles["h2"]))
    story.append(Paragraph('Transport in Charleroi', style=styles["h3"]))
    story.append(Paragraph('Public transport (Bus)', style=styles["h3"]))

    story.append(Paragraph(("""With public transport in Charleroi you can go almost everywhere (68 lines).
                                Just take the time to read carefully the long timetables. If you know where
                                your hotel is, have a look now on the website (unfortunately only in French...):
                                http://www.tec-charleroi.be"""), style=styles["bbody"]))

    story.append(Paragraph(("""They also have a phone number if you get lost among all their lines."""), style=styles["bbody"]))

    story.append(Paragraph(("""<b>Phone</b>: 32(0)71 23.41.15"""), style=styles["bbody"]))

    story.append(Paragraph(("""<b>Open:</b> 6h30 - 18h30"""), style=styles["bbody"]))

    story.append(Paragraph(("""Unfortunately they don't speak english but force them a bit and you will succeed to know what you need."""), style=styles["bbody"]))

    story.append(Paragraph(("""<b>Payment:</b> Ready cash (most of the drivers won't accept your big notes). The price depends of how many zones you will have to travel:"""), style=styles["bbody"]))

    story.append(Spacer(height=6, width=10))

    story.append(Table(
        [
            [Paragraph((""), style=styles["h1"]), Paragraph(("1 TICKET"), style=styles["h2"]),Paragraph(("-"), style=styles["h2"])],
            [Paragraph(("Number of ZONE"), style=styles["h1"]), Paragraph(("Normal price"), style=styles["h1"]),Paragraph(("For 1 Day"), style=styles["h1"])],
            [Paragraph(("1 or 2"), style=styles["h2"]), Paragraph(("1,10"), style=styles["h2"]),Paragraph(("-"), style=styles["h2"])],
            [Paragraph(("3"), style=styles["h2"]), Paragraph(("1,65"), style=styles["h2"]),Paragraph(("-"), style=styles["h2"])],
            [Paragraph(("4"), style=styles["h2"]), Paragraph(("2,20"), style=styles["h2"]),Paragraph(("-"), style=styles["h2"])],
            [Paragraph(("5"), style=styles["h2"]), Paragraph(("2,75"), style=styles["h2"]),Paragraph(("-"), style=styles["h2"])],
            [Paragraph(("6 and more"), style=styles["h2"]), Paragraph(("3,30"), style=styles["h2"]),Paragraph(("-"), style=styles["h2"])],
            [Paragraph(("All zones"), style=styles["h2"]), Paragraph(("3,30"), style=styles["h2"]),Paragraph(("5,50"), style=styles["h2"])]
            ],
                 colWidths=[2*inch,2*inch,2*inch],
                 rowHeights=[22,22,22,22,22,22,22,22],
                 style=TableStyle([
                    ('SPAN', (1,0), (-1,0)),
                    ('GRID', (0,1), (-1,-1), 2, gray),
                    ('OUTLINE', (1,0), (2,0), 2, gray),
                    ]),
                 splitByRow=1,
                 repeatRows=0,
                 repeatCols=0)
                 )

    story.append(Spacer(height=6, width=10))

    story.append(Paragraph(("""<b>Signs:</b> To understand the bus signs, here is what you will see most of the time:"""), style=styles["bbody"]))
    story.append(Spacer(height=6, width=10))
    story.append(timetable.getURLImage('http://www.europython.org/information/transportation/images/bus_signal', width=350*0.6, height=379*0.6))
    story.append(Spacer(height=6, width=10))

    story.append(Paragraph(("""<b>Interesting lines for CEME:</b>"""), style=styles["bbody"]))
    story.append(Spacer(height=6, width=10))
    story.append(Paragraph(("""<b>LINE 41</b> Take the bus with linenumber 41 heading for 'Courcelles'
                            (less frequent than the above possibility). The trip will take about 10 minutes.
                            Exit the bus at busstop 'Ravin' and take the street 'Gohyssart' (rue de Gohyssart)
                            which leads to the CEME (approximately 3 minutes on foot). (Here is the timetable).
                            Be careful that this bus is going from 5h40 until 20:50 !! So if you want to go to
                            restaurant or something you will have to find another solution. """), style=styles["bbody"]))
    story.append(Spacer(height=6, width=10))
    story.append(Paragraph(("""<b>Second solution:</b> Take any bus, indicating the direction '<b>Chausse de Bruxelles</b>'.
                            Exit the bus at busstop '<b>La Planche</b>', traverse the road. On the other side of the road,
                            follow the street 'rue des Franais'. You will find the CEME at the end of this street
                            (approximately 5 minutes on foot)."""), style=styles["bbody"]))
    story.append(Spacer(height=12, width=10))

    story.append(Paragraph(("""Taxi"""), style=styles["h3"]))

    story.append(Paragraph(("""Here are the 3 main taxi companies of Charleroi. It's the same price for all of them
                            so just make your choice (about 1 euro/km)."""), style=styles["bbody"]))

    story.append(Table(
        [
            [Paragraph(("<b>TAXIS CAROLO</b>"), style=styles["bbody"]), "Phone 1: 32(0)71 32.32.32\nPhone 2: 32(0)71 20.83.76\nPhone 3: 32(0)71 20.83.83"],
            [Paragraph(("<b>IDEALTAX</b>"), style=styles["bbody"]), "Phone 1: 32(0)71 36.52.42\nPhone 2: 32(0)71 43.05.05"],
            [Paragraph(("<b>Confort Tax</b>"), style=styles["bbody"]), "Phone 1: 32(0)71 41.31.31\nPhone 2: 32(0)71 48.82.00\nPhone 3: 32(0)71 21.99.99"],
            ],
                 colWidths=[1.3*inch,1*inch],
                 rowHeights=[50,50,50],
                 style=None,
                 splitByRow=1,
                 repeatRows=0,
                 repeatCols=0)
                 )
    story.append(Spacer(height=12, width=10))

    story.append(Paragraph(("""Rent a bike"""), style=styles["h3"]))
    story.append(Paragraph(("""Bicycle is a way to feel more free in Charleroi. Here are the places:"""), style=styles["bbody"]))
    story.append(Paragraph(("""<b>La Maison des Cyclistes</b> Bicycle only:"""), style=styles["bbody"]))
    story.append(Paragraph(("""Address: Rue du Grand Central 57, 6000 Charleroi"""), style=styles["bbody"]))
    story.append(Paragraph(("""Phone:  32(0)71 53.58.50"""), style=styles["bbody"]))
    story.append(Paragraph(("""Open Time:  12h30 - 17h"""), style=styles["bbody"]))
    story.append(Spacer(height=12, width=10))

    story.append(Paragraph(("""Hitching"""), style=styles["h3"]))
    story.append(Paragraph(("""I am sure you will be able to find a friendly Belgian ready to give you a lift!"""), style=styles["bbody"]))
    story.append(Spacer(height=12, width=10))

    story.append(Paragraph(("""Share cars"""), style=styles["h3"]))
    story.append(Paragraph(("""But best way is maybe to share cars among you..."""), style=styles["bbody"]))


    if verbose: log ("\tfunction getTransportationInfo complete\n")

    
    
    
def getPlacesToMeet():
    """Function to add the Places To Meet page to the brochure.
    Info taken directly from the web site on 19 June 2003."""
    from reportlab.lib.colors import black, red, green, blue, gray
    if verbose: log ("\tstarted function getPlacesToMeet")

    story.append(NextPageTemplate('OneCol'))
    newPage()
    #This bit from the Places To Meet page
    story.append(Paragraph('Places To Meet<onDraw name="myOnDrawCB" label="Places To Meet"/>', style=styles["h2"]))
    story.append(Paragraph('Cafe', style=styles["h3"]))

    story.append(Paragraph("""Best place to meet and best moments you expect, I am sure: meetings around belgian beers... But where to drink them? There are about 150 cafes in Charleroi and the surroundings, so you will have 'some' choice. Here is a small list of our favourite ones:""", style=styles["bbody"]))
    story.append(Spacer(height=12, width=10))
    story.append(Table(
        [
            [Paragraph(("<b>Brasserie de l'Eden</b>"), style=styles["bbody"]), Paragraph(("""A big brasserie where you can drink a lot of belgian beer and also eat. Last year, this
                                                                                  brasserie was the meeting place after each conference day. This year again they are ready
                                                                                  to welcome all Europython participants. A free drink is offered there by them. As we go
                                                                                  there quite often we even took a MENU (Vegetarian food available!). We told them to be
                                                                                  ready with lot's of Chimay."""), style=styles["bbody"])]],
                 colWidths=[1.5*inch,4.5*inch],
                 rowHeights=[80],
                 style=TableStyle([('VALIGN', (0,0), (-1,-1), 'TOP'),('FONT', (0,0), (-1,-1), 'Helvetica')]),
                 splitByRow=1,
                 repeatRows=0,
                 repeatCols=0)
                 )
    story.append(Table(
        [
            ["Address:", "Boulevard Jacques Bertrand 1"],
            ["", "6000 Charleroi"],
            ["Phone:", "32(0)71 30.64.96"],
            ["Open Time:", "9h - Open"]            
            ],
                 colWidths=[1*inch,2*inch],
                 rowHeights=[16,16,16,16],
                 style=TableStyle([('VALIGN', (0,0), (-1,-1), 'TOP'),('FONT', (0,0), (-1,-1), 'Helvetica')]),
                 splitByRow=1,
                 repeatRows=0,
                 repeatCols=0)
                 )
    story.append(Spacer(height=6, width=10))
    story.append(Table(
        [
            [Paragraph(("<b>Le Luxembourg</b>"), style=styles["bbody"]), Paragraph(("""A really good cafe/restaurant
                                                                                        near to the Business HOTEL. 
                                                                                        Very calm and nice."""), style=styles["bbody"])]],
                 colWidths=[1.5*inch,4.5*inch],
                 rowHeights=[30],
                 style=TableStyle([('VALIGN', (0,0), (-1,-1), 'TOP'),('FONT', (0,0), (-1,-1), 'Helvetica')]),
                 splitByRow=1,
                 repeatRows=0,
                 repeatCols=0)
                 )
    story.append(Table(
        [
            ["Address:", "Rue du Pont Neuf, 41"],
            ["", "6000 Charleroi"],
            ["Phone:", "32(0)71 31.63.59"],
            ["Open Time:", "10h - 02h"],
            ],
                 colWidths=[1*inch,2*inch],
                 rowHeights=[16,16,16,16],
                 style=TableStyle([('VALIGN', (0,0), (-1,-1), 'TOP'),('FONT', (0,0), (-1,-1), 'Helvetica')]),
                 splitByRow=1,
                 repeatRows=0,
                 repeatCols=0)
                 )
    story.append(Spacer(height=6, width=10))
    story.append(Table(
        [
            [Paragraph(("<b>Star Rock Cafe</b>"), style=styles["bbody"]), Paragraph(("""Well known cafe in the center of Charleroi."""), style=styles["bbody"])]],
                 colWidths=[1.5*inch,4.5*inch],
                 rowHeights=[20],
                 style=TableStyle([('VALIGN', (0,0), (-1,-1), 'TOP'),('FONT', (0,0), (-1,-1), 'Helvetica')]),
                 splitByRow=1,
                 repeatRows=0,
                 repeatCols=0)
                 )
    story.append(Table(
        [
            ["Address:", "Boulevard Joseph Tirou 122"],
            ["", "6000 Charleroi"],
            ["Phone:", "32(0)71 30.28.77"]],
                 colWidths=[1*inch,2*inch],
                 rowHeights=[16,16,16],
                 style=TableStyle([('VALIGN', (0,0), (-1,-1), 'TOP'),('FONT', (0,0), (-1,-1), 'Helvetica')]),
                 splitByRow=1,
                 repeatRows=0,
                 repeatCols=0)
                 )
    story.append(Spacer(height=6, width=10))
    story.append(Table(
        [
            [Paragraph(("<b>Le Village Africain</b>"), style=styles["bbody"]), Paragraph(("""Great african atmosphere there, really simple but nice."""), style=styles["bbody"])]],
                 colWidths=[1.5*inch,4.5*inch],
                 rowHeights=[20],
                 style=TableStyle([('VALIGN', (0,0), (-1,-1), 'TOP'),('FONT', (0,0), (-1,-1), 'Helvetica')]),
                 splitByRow=1,
                 repeatRows=0,
                 repeatCols=0)
                 )
    story.append(Table(
        [
            ["Address:", "..."],
            ["", "6000 Charleroi"],
            ["Phone:", "32(0)...."]],
                 colWidths=[1*inch,2*inch],
                 rowHeights=[16,16,16],
                 style=TableStyle([('VALIGN', (0,0), (-1,-1), 'TOP'),('FONT', (0,0), (-1,-1), 'Helvetica')]),
                 splitByRow=1,
                 repeatRows=0,
                 repeatCols=0)
                 )
    story.append(Spacer(height=6, width=10))
    story.append(Table(
        [
            [Paragraph(("<b>The Irish Times Pub</b>"), style=styles["bbody"]), Paragraph(("""If you miss the Irish Guiness and the Irish
                                                                                atmosphere (they also have Belgian beer [fortunately...]).
                                                                                They speak more English than french over there ;)"""), style=styles["bbody"])]],
                 colWidths=[1.5*inch,4.5*inch],
                 rowHeights=[40],
                 style=TableStyle([('VALIGN', (0,0), (-1,-1), 'TOP'),('FONT', (0,0), (-1,-1), 'Helvetica')]),
                 splitByRow=1,
                 repeatRows=0,
                 repeatCols=0)
                 )
    story.append(Table(
        [
            ["Address:", "Boulevard Joseph Tirou 86"],
            ["", "6000 Charleroi"],
            ["Phone:", "32(0)71 31.61.52"],
            ["Open Time:", "12h - 02h"]],
                 colWidths=[1*inch,2*inch],
                 rowHeights=[16,16,16,16],
                 style=TableStyle([('VALIGN', (0,0), (-1,-1), 'TOP'),('FONT', (0,0), (-1,-1), 'Helvetica')]),
                 splitByRow=1,
                 repeatRows=0,
                 repeatCols=0)
                 )
    
    story.append(Paragraph('Restaurant', style=styles["h3"]))
    
    story.append(Paragraph("""There are about 70 restaurants in Charleroi. Here too you
                            will have many choices. Of course it's impossible to list them
                            all here so we advise you to make your choice on this website:
                            http://www.resto.be Or just see when you will be there.
                            Charleroi isn't big so you find different kind of restaurant
                            near to each other.""", style=styles["bbody"]))
    
    story.append(Paragraph('Green meeting place', style=styles["h3"]))
    
    story.append(Paragraph("""Charleroi isn't a so green town but there one main park:
                            <b>the Reine Astrid Park</b>. It's near to the center of Charleroi.
                            Here it is on the map:""", style=styles["bbody"]))
    story.append(Spacer(height=6, width=10))
    story.append(timetable.getURLImage('http://www.europython.org/information/images/map_astrid_park.gif', width=360*0.6, height=360*0.6))
    
    story.append(Paragraph('CyberCafe', style=styles["h3"]))
    
    story.append(Paragraph("""For the most addicted among you, you will be able to access
                            the net after Europython (don't forget that you will be able
                            to access the net during the conference!!) We didn't tried
                            this shop so it's at your own risk...""", style=styles["bbody"]))
    story.append(Spacer(height=6, width=10))
    story.append(Table(
        [
            [Paragraph(("<b>Mont Cameroun</b>"), style=styles["bbody"]), Paragraph(("""Sounds african internet there :)"""), style=styles["bbody"])]],
                 colWidths=[1.5*inch,4.5*inch],
                 rowHeights=[20],
                 style=TableStyle([('VALIGN', (0,0), (-1,-1), 'TOP'),('FONT', (0,0), (-1,-1), 'Helvetica')]),
                 splitByRow=1,
                 repeatRows=0,
                 repeatCols=0)
                 )
    story.append(Table(
        [
            ["Address:", "Rue de l'Aigle Noir 7"],
            ["", "6000 Charleroi"],
            ["Phone:", "32(0)71 50.92.40"],
            ["Open Time:", "09h - 22h"]],
                 colWidths=[1*inch,2*inch],
                 rowHeights=[16,16,16,16],
                 style=TableStyle([('VALIGN', (0,0), (-1,-1), 'TOP'),('FONT', (0,0), (-1,-1), 'Helvetica')]),
                 splitByRow=1,
                 repeatRows=0,
                 repeatCols=0)
                 )
    story.append(Spacer(height=12, width=10))
    story.append(Paragraph("""We sincerely hope you will enjoy your stay in Charleroi.""", style=styles["bbody"]))     
    
    if verbose: log ("\tfunction getPlacesToMeet complete\n")
    
    
    
    
def getFloorPlan():
    if verbose: log ("\tstarted function getFloorPlan")

    story.append(NextPageTemplate('FloorPlanPage'))
    newPage()
    story.append(Paragraph('Floorplan<onDraw name="myOnDrawCB" label="Floorplan"/>', style=styles["h2"]))
    story.append(Spacer(height=12, width=10))

    story.append(Paragraph(("""We've rented the CEME congress building at Charleroi. On this page
    you'll find the floorplans of the ground and first floor. """), style=styles["bbody"]))

    story.append(Paragraph(("""<b>Conference Rooms</b>"""), style=styles["bbody"]))

    story.append(Paragraph(("""The conference rooms are located on the first
    floor of the building: """), style=styles["bbody"]))

    story.append(Spacer(height=5, width=10))

    URL = 'http://www.europython.org/information/floorplan/images/ceme_floorplan'
    fp = timetable.getURLImage(URL, width=471*0.50, height=437*0.50)
    fp.hAlign = 'CENTER'
    story.append(fp)
    if verbose: log ("\tfunction getFloorplans image 1 complete\n")

    story.append(Spacer(height=8, width=10))

    story.append(Paragraph(("""<b>Internet access and battery charging</b>"""), style=styles["bbody"]))

    story.append(Paragraph(("""These rooms are located on the ground floor. We will give
    you all the technical details at the conference itself. Just take your laptop and an
    Ethernet cable (RJ45) with you. """), style=styles["bbody"]))

    story.append(Spacer(height=5, width=10))

    URL = 'http://www.europython.org/information/floorplan/images/map_ceme_level0.gif'
    fp = timetable.getURLImage(URL, width=747*0.45, height=633*0.45)
    fp.hAlign = 'CENTER'
    story.append(fp)
    if verbose: log ("\tfunction getFloorplan image 2 complete\n")

def getListOfOrganizers():

    story.append(NextPageTemplate('OneCol'))
    newPage()
    story.append(Paragraph('The EuroPython Organization Team<onDraw name="myOnDrawCB" label="The EuroPython Organizers"/>', style=styles["h2"]))

    story.append(Paragraph('EuroPython Executive Committee', style=styles["h3b"]))

    story.append(Paragraph('<b>Tom Deprez</b>', style=styles["bNormal"]))

    story.append(Paragraph("""Tom Deprez is employed as a radiation physicist and an information
technology project engineer in the Department of Radiology at the
University Hospital of Leuven, Belgium.""", style=styles["biNormal"]))

    story.append(Paragraph("""As a freelancer, Tom is one of Aragne.com's employees for development
of Zope (and Python) applications.""", style=styles["bibody"]))

    story.append(Paragraph("""A user of Zope since the separate products merged into one, Tom has
used Zope in several intranet and extranet applications and has
performed technical reviews of Zope Books for several publishers. He
further uses Delphi and lately Python for the development of other
applications.""", style=styles["bibody"]))

    story.append(Paragraph("""One of Tom's biggest hobbies, besides programming, is Role-Playing games.""", style=styles["bibody"]))

    story.append(Paragraph("""Tom is an electronical engineer with post-graduate diplomas in
biomedical and clinical engineering techniques, as well as a masters
in networking and multimedia.""", style=styles["bibody"]))

    story.append(Spacer(height=6, width=10))

    story.append(Paragraph('<b>Denis Frere</b>', style=styles["bNormal"]))
    story.append(Paragraph("Denis Frre is a Free Software defender and a Python lover.", style=styles["biNormal"]))
    story.append(Paragraph("He is co-founder and ceo/president of", style=styles["bibody"]))
    story.append(Paragraph("Aragne (a Python-Zope business company),", style=styles["bibu"], bulletText="l"))
    story.append(Paragraph("P3B (a Belgian non-profit organization caring about Python and and its different incarnations)", style=styles["bibu"], bulletText="l"))
    story.append(Paragraph("OS3B (a Belgian GNU/Linux User Group)", style=styles["bibu"], bulletText="l"))
    story.append(Paragraph("He translated one of the first Python books in French and ... is your host today. :-)", style=styles["bibody"]))

    story.append(Spacer(height=6, width=10))

    story.append(Paragraph('<b>Marc-Andr Lemburg</b>', style=styles["bNormal"]))
    story.append(Paragraph("""Marc-Andr Lemburg is the CEO and founder of eGenix.com Software
GmbH, Langenfeld, Germany. He has a degree in mathematics from the
University of Dsseldorf.""", style=styles["biNormal"]))
    story.append(Paragraph("""His work with and for Python started in Winter 1993/4. In 1997 he
became Python Core Developer and since February 2002, he is working
for the Python Software Foundation (PSF) as board member.""", style=styles["bibody"]))
    story.append(Paragraph("""Marc-Andr is the author of the well-known eGenix.com mx Extensions,
e.g. mxDateTime and mxODBC.""", style=styles["bibody"]))

    story.append(Spacer(height=12, width=10))
    story.append(Paragraph('EuroPython Support Team', style=styles["h3b"]))
    story.append(Paragraph('Joachim Schmitz (Zope database, payment system and general logistics)', style=styles["bNormal"]))
    story.append(Paragraph('Vincent Maton (Logo, web-site and brochure design)', style=styles["bNormal"]))
    story.append(Paragraph('Martijn Faassen (Initial idea and supporter from the first hour)', style=styles["bNormal"]))
    story.append(Paragraph('Nicolas Chauvat (Initial idea and supporter from the first hour)', style=styles["bNormal"]))

    story.append(Spacer(height=12, width=10))
    story.append(Paragraph('EuroPython Track Chairmen', style=styles["h3b"]))
    story.append(Paragraph("""Andy Robinson, Martijn Faassen, Shae Erisson, Paul Everitt, Tom Deprez,
Tim Couper, Marc-Andre Lemburg, Marc Poinot, Nicolas Chauvat.""", style=styles["bbody"]))


interviewDict={
    'Zadka':'Moshe Zadka',
    'Hudson':'Michael Hudson',
    'Lemburg':'Marc-Andre Lemburg',
    'Grisby':'Duncan Grisby',
    'Thompson':'Phil Thompson',
    'Couper':'Tim Couper',
    'Faassen':'Martijn Faassen',
    'Chauvat':'Nicolas Chauvat',
    'Everitt':'Paul Everitt'
   }


def getInterviews(I):
    if verbose: log ("\tstarted function getInterviews")

    if I is None: I = interviewDict.keys()
    if I:
        story.append(NextPageTemplate('OneCol'))
        newPage()
        story.append(Paragraph('Interviews<onDraw name="myOnDrawCB" label="Interviews"/>', style=styles["h2"]))

        story.append(Spacer(height=24, width=10))
        story.append(Paragraph(("""This section contains all the pre-interviews related to EuroPython2003.
    We hope these will slake your thirst, or stir up your appetite for
    EuroPython 2003."""), style=styles["bbody"]))
        story.append(Spacer(height=24, width=10))

        N=interviewDict.keys()
        N.sort()
        for n in N:
            if n in I:
                story.append(Paragraph('Interview with '+interviewDict[n], style=styles["bbu"], bulletText="l"))

        for n in N:
            if n in I:
                exec "get"+n+"Interview()"
    if verbose: log ("\tfunction getInterviews complete\n")


def getZadkaInterview():
    ## Start of interview
    #Interview with Moshe Zadka

    story.append(NextPageTemplate('TwoColPlus'))
    newPage()
    story.append(Paragraph('Interview of Moshe Zadka<onDraw name="myOnDrawCB" label="Moshe Zadka"/>', style=styles["h3"]))
  
    story.append(FrameBreak())
    story.append(NextPageTemplate('TwoCol'))
    
    story.append(Paragraph(("""Tom Deprez"""), style=styles["2colar"]))

    story.append(Paragraph(("""<b>EuroPython: I've you search the net for Moshe Zadka,
    you easely end up in: http://www.moshez.org/. So, it's easy to learn a lot about 'Moshe',
    but anyway, can you introduce yourself to the readers in a condensed way?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""Moshe Zadka: I'm a mathematician (currently working on my MSc.
    in differential topology) from Israel. I have had the UNIX bug since 1995, and the free
    software bug since 1997."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: How is your name pronounced? Do you experience that
    people have some difficulty with your name?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: I'll start with the second part: "Moshe" is a fairly popular Jewish
    name, so Jews usually know how to pronounce it. English speakers tend to pronounce "she"
    like the 2nd person female pronoun, instead of as the beginning of "shell". As long as that
    mistake is avoided, I am pretty flexible with how my name is pronounced."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Have you heard about EuroPython before?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: Yes. I have thought about going to the first EP, but life got in
    the way. I am very happy to be able to go to this one!"""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: What's your relationship with Python?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: I'm a (fairly non-active, currently) core Python developer. I did
    all kinds of small things in the Python CVS, including documenting a few of the more obscure modules
    and implementing _contains_."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Did you manage to combine Python with your work, or
    do you have to use other programming languages?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: Currently I don't do any programming professionally. However,
    I've managed to use Python in several of my previous jobs -- as infrastructure for exploratory
    and algorithmic statistical research, build infrastructure and dynamic web sites
    programming."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: You're the person with the highest talk proposals.
    Ten, of which seven are accepted. For being 'fairly non-active', you've do admit that it is a bit
    contradictory. Where is the catch? Do you like to talk or is there another driving force?
    Python Advocay?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: Well, I'm not as "non-active" as it seems. I have stopped being
    active on the main Python mailing lists, but that's mostly because my energies have been devoted
    to Twisted -- the new up and comer. Also, yes, I do enjoy public speaking, as curious as it may
    be in a geek."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: On your website, I've read about your own project
    'Python Mail System'and your involvement in the 'Bicycle Repair Man' project. Can you tell
    us more about these projects?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: PMS is the mailer I use daily, but I do not maintain it anymore.
    The biggest lesson I have learned from PMS is that in Python, it is often easier to just rewrite
    an application from the ground up then to use someone else's -- nowadays, I'm recommending to people
    who want to write a Python MUA to use the "email" module and write it themselves."""), style=styles["2cola"]))

    story.append(Paragraph(("""I'm no longer involved in BRM. I helped Shae Erisson a bit, back when
    he was the lead developer. It is a very exciting project -- the only refactoring browser for Python
    which is free. A refactoring browser is a tool to help do menial tasks when refactoring code such as
    abstracting repeated code into functions, moving functions between modules, etc."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: So you actually encourange people not to use PMS? And write
    one of their own?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: Yes. I realize it sounds weird, but I realized at some point PMS is
    an extension of what I wanted in e-mail software. Everyone has differnt needs and priorities. Many
    of the things in PMS started out as being generic, but I only used the genericity in one point. It
    really is so easy to write software with Python, often frameworks aren't needed."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Why the name 'Bycicle Repair Man'?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: I wasn't involved in choosing it. It is my understanding it's based
    on a Monty Python skit."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Is the BRM project still active? Would you recommend
    people to have a look at it and join it?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: I have no idea. Like I said, I am no longer involved in it.
    """), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: On which Python projects are you working at the
    moment?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: I'm most active on "Twisted", the asynchronous networking framework.
    I'm involved in design and documentation, but what interests me most from a research point of view
    is how to deploy Twisted-based applications in a flexible manner. That ties in with my biggest role
    in the Twisted project as the Debian maintainer. I make sure Twisted integrates with the Debian
    tools, so that I can give users of the Debian operating system a system which is powerful, easy
    and flexible."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: We've heard about Twisted before :-) (see also interview
    with Itamar last year). To what can it be compared?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: Well, it's hard to compare it honestly to anything else. The
    first thing that comes to mind is "asyncore" and "medusa", but Twisted is based on a much
    stronger pattern (complete decoupling between protocol and transport), and has much higher-level
    parts (like a web server, NNTP server and IRC clients), all of which can be easily integrated.
    It can be compared to Zope, but it does not depart that much from standard Pythonic sense
    (for example, it is easy to avoid inheriting from multiple classes in Twisted) and does not
    force a complete methodology."""), style=styles["2cola"]))

    story.append(Paragraph(("""But really, perhaps the best comparison is to the Python standard
    library, but for networking: you don't have to use it, but if you don't, you'll waste your time
    and reinvent the wheel."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: I'm a little bit confused. You compare Twisted with
    a library and an application. Is it both? Ie does it contain a general layer (the library) and
    a layer on top which makes Twisted as an application? Are all layers independent from each
    other?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: In Python, there is no clear difference between applications and
    libraries -- which is why I prefer the word "framework". Here is an example: if you are writing a
    Gtk+ program, it is possible, with Twisted, to embed a production-grade web server in it with one
    function call. Another way of saying that same thing is that it is easy to write a GUI program
    which will manage, in-process, a Twisted web server. It is hard to say which of these depictions
    is closer to the truth, if such a truth exists."""), style=styles["2cola"]))

    story.append(Paragraph(("""I gave the example of a web server intentionally: that same
    line to add a web server would work on the command line, as a way of writing a web sitt force a
    complete methodology."""), style=styles["2cola"]))

    story.append(Paragraph(("""But really, perhaps the best comparison is to the Python standard
    library, but for networking: you don't have to use it, but if you don't, you'll waste your time
    and reinvent the wheel.(this is how most GUI programs inside Twisted work)."""), style=styles["2cola"]))

    story.append(Paragraph(("""Note that the Python standard library contains such things as
    Tkinter's mainloop or SocketServer's serve_forever, both intended for taking over an application's
    main loop."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: What is Twisted for yourself? Why would you recommend
    people to Twisted and how would you make a person 'warm' for Twisted?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: As I said above, I really think that every Python programmer
    writing networking code should know about Twisted."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: You've mentioned Zope. EuroPython has a complete track,
    three days long devoted on Zope. Can this be devoted on different ways of marketing? Or is
    Twisted younger and needs a bigger group of followers?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: Well, Twisted is younger. Zope has been around, if you count
    in Principia and Bobo, for about a decade, while Twisted has only been around for three years.
    I'm pretty sure, though, you could fill in three days of Twisted track if you wanted --
    so you'll have to ask the EP organizers to give Twisted a track next year."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Have you've used Zope and Twisted in such a way that
    you could give us a sort of comparision (your view)? What are the Pro's/Con's of
    both?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: I have to first say that I hadn't had occasion to use Zope 3,
    so all my remarks will pertain to Zope 2. Zope 2 was a pleasure to work with, including
    the TTW management. I even wrote a Hero(TM) GM's helper with Zope, which automatically
    rolled dice and calculated statistics, and integrated with the prose for an adventure in
    a transparent way. However, Zope is hard to extend with pure Python. As soon as I tried to
    do non-trivial programming, I had to use Products and learn about arcane things like
    "the Folder interface". I feel that Zope is very well suited to a mainly-content site,
    especially when the TTW editing capabilities are important to the clients. Twisted is good
    in programming-heavy contexts, especially when the "site" has to integrate with other
    protocols."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: If I'm right, it is one of the intentions of Zope3
    to make it easier to extend it with pure Python."""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: So I have heard too.?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Are you following the development of
    Zope3?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: No."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Is someone else of Twisted following Zope3? Or the other
    way around is someone of the Zope developers also involved in Twisted?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: I am not sure. I think itamar follows Zope, but perhaps not too
    closely. As for the Zope developers, you'll have to ask them."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Do you think Zope and Twisted could benefit from each
    other (cross-breed) or are they competitors? Is there any direction in coperation between the
    developers of both products?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: There has been some work on basing Zope 3's network layer on
    Twisted, and there was an ugly hack for making this happen with Zope 2. It is my belief that
    Zope, and ZEO, could benefit from using Twisted rather than medusa. However, currently nobody
    is working on making the official Zope 3 network layer be Twisted, as far as
    I know."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Any idea why there is less 'official' interest?
    Is it a simple priority rule? It seems to me, what I have heard during this interview, both
    could indeed benefit a lot from each other.?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: I am not sure. Again, this is something where I expect Zope developers
    interest to lead the way, and I am not involved in Zope."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Can you direct us to some applications which are able
    to show the full power of Twisted?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: No :-)"""), style=styles["2cola"]))

    story.append(Paragraph(("""The simple reason is that Twisted is very powerful, so it's hard to show
                            the full power of Twisted. However, there are quite a few nice Twisted-based
                            programs: HEP is an integrated message server, mc-foo is a client/server media player,
                            conch is a Twisted ssh client/server and Twisted Web, which I will be talking about in
                            the conference."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: What do you like best about Python?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: I don't know. I want to clarify that -- Python has a certain quality
                            that makes it "easy to use". I don't know what it is -- if it is the namespace
                            model, the indentation-for-structure or something else. But it is this quality
                            that makes me choose Python for each task, or to be sorry I didn't whenever I don't."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: What do you not like about Python?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: I am somewhat apprehensive about the "new style classes". It might
                            be just an old-timer's reluctance to change, but I feel they lack the
                            simplicity that Python has."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Can you elaborate more on the 'new style classes'?"""),
    style=styles["2cola"]))

    story.append(Paragraph(("""MZ: Not really. I haven't used them enough in a production environment
                            (Twisted only recently lifted the compatibility with Python 2.1 requirement) so
                            I don't have any real data yet."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Is there something that Python doesn't have yet, of which
    Twisted would be able to benefit if someone implemented it?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: I'm going to be unoriginal here: a cross-platform high-quality version
    of psyco integrated in the standard library."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Why is it that Python doesn't play along with the bigger
                            development languages, like Java, C++, Delphi, ... ? Is Python too immature? Or
                            is it something else?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: I beg to differ."""), style=styles["2cola"]))

    story.append(Paragraph(("""Python certainly "plays along" with the "bigger" development languages.
    Many projects are written in Python (for example, large parts of the Debian
    infrastructure are) and Python is used for
    mission critical roles everywhere. Just look at where posters to c.l.py work :-)
    Python may not have Java's marketing hype or C++'s "industry standard" stamp,
    but it is certainly alive and well."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Correct. But isn't it a fact that lot's of companies from
    the closed-source tend to use Java, C++ or the newcommer C# more quickly than
    Python?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: I think that nowadays nobody is seriously considering Java
    as an end-user delivery language, except for Sun and Oracle. C++ has the "industry standard"
    stamp -- people feel it is a "safe" choice. I don't know of any major applications written
    in C# either."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: I lately hear a lot from people in my neighbourhood
    that in the future most development (mostly applications which need a very strong connection to
    the user. Very high User Interface or internet/intranet related) will be done in either Java or .Net.
    What do you think about such statements. Is/will Python able to deliver the same
    things?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: I feel such statements are usually made by people buying Sun's
    or MS's hype. Python is already able to deliver those things: with Twisted, writing graphical
    clients for network applications is a breeze, and they are easier to program since no threads
    and race conditions have to be taken into account."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Can you direct us to some examples of such clients
    written in Twisted?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: tkconch is a nice one, in Twisted: it is a graphical ssh client
    (much like PuTTY)."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Browsing through your webpages, I saw something
    named 'Advogato'. What is it and why does it interest you?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: http://advogato.org is a blog devoted to free software and the
    free software community. Many interesting people use it as their main blog and there are
    occasionally interesting articles related to free software there."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: You are a Debian developer. Is there a special reason
    why you choose Debian?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: Debian is a portable, comprehensive and free operating system.
    This means I can trust Debian to work on all the machines I have, to have the software I need
    and to not limit me in how I use it. It is an experiment in integrating a lot of software, and
    as far as I am concerned -- a successful one."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Are Debian and Python the only things you are interested
    for in the computer world?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: I am not much of a theory guy, when it comes to computer. Thus,
    I enjoy implementing ideas I have. Python is (almost) always the right technology to use, and
    Debian is (almost) always the correct way to deploy it. I'm not sure if it qualifies as a "yes"
    or a "no", though."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: If the readers didn't knew yet, together with Anna
    Ravenscroft, you volunteerd to host the Lightning Talks and Open Space. To you like these kind of
    talks/discussions?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: I always have a simple recommendation of lightning talks: even if
    they are dead-boring, the boredom is over in five minutes. Most often, the speaker has to finish
    before you would get bored."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: To whom need the interested people address to give an
    LT at EuroPython2003?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: They should contact either me, at eplt @ moshez.org, or Anna, at
    rev_anna_r @ yahoo.com. It's recommended to CC: us both."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: If EuroPython would give an explecit track to Twisted
    next year. Could we then count on you for a full day of Moshe's Twisted talks? ;-)</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: Probably :-)"""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Finally, what do you expect from EuroPython? Anything you
    look forward to?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MZ: I am looking forward to meeting people whom I've met before
    (like Alex Martelli and Jack Jansen) and to meeting people with whom I have had quite a bit of
    online dealings but have never met (like Michael Hudson and Anna Ravenscroft). For me, conferences
    are mostly about meeting people, not about giving or listening to talks, even if the amount of my
    talk proposals would seem to indicate otherwise."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>Interested readers can find more info at:</b>"""), style=styles["2cola"]))


    story.append(Paragraph(("""http://www.advogato.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.debian.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.europython.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.moshez.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.twistedmatrix.com/"""), style=styles["2cola"]))


    ## End of interview

def getLemburgInterview():
    ## Start of interview
    #Interview with Marc-Andre Lemburg

    story.append(NextPageTemplate('TwoColPlus'))
    newPage()
    story.append(Paragraph('Interview of Marc-Andre Lemburg<onDraw name="myOnDrawCB" label="Marc-Andre Lemburg"/>', style=styles["h3"]))
    story.append(FrameBreak())
    story.append(NextPageTemplate('TwoCol'))

    story.append(Paragraph(("""Tom Deprez"""), style=styles["2colar"]))

    story.append(Paragraph(("""<b>EuroPython: Marc-Andre, can you introduce yourself to our readers?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MAL: Most people probably know me because they have used one of the
                                well-known mx Extensions for Python, of which mxDateTime and mxODBC
                                are probably the most used of these out there."""), style=styles["2cola"]))

    story.append(Paragraph(("""I started working with Python in the Winter of 1993 and immediately fell in love with Guido's design decisions. Later on, when Python's development moved to SourceForge, I was invited as Python Developer and granted CVS write access. Since early 2001, I am also member of the Python Software Foundation (PSF) board and busy working on getting the legal and financial aspects of Python contributions in place."""), style=styles["2cola"]))

    story.append(Paragraph(("""Ever since I left university, I have been working professionally with and for Python. I first started working as consultant (which I still enjoy) and founded eGenix.com early in 2000, a German company aimed at providing high quality professional skills, products and tools on the basis of the Python programming language."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Can you tell more about the Python Software Foundation (PSF) board?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MAL: The board manages the everyday business of the PSF and decides on all major activities related to the PSF's mission, i.e. furthering the use and development of Python and related technologies/activities."""), style=styles["2cola"]))

    story.append(Paragraph(("""The latest big undertaking of the PSF was to help organize and financially backup the PyCon conference in the US (which happens to be a low-cost conference similar to EuroPython)."""), style=styles["2cola"]))

    story.append(Paragraph(("""Apart from these outstanding events, we are currently looking into getting legal backup for securing Python's IP with the ultimate goal of obtaining the IP for the whole interpreter. In short: try to reduce the size of the LICENSE file."""), style=styles["2cola"]))

    story.append(Paragraph(("""Most of the tasks are managed offline, but we do have monthly IRC meetings to discuss important issues and to pass motions which guide future developments."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: How is your progress in the work you do for the PSF and are/were there caveats?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MAL: As with all open source or voluntary work, available time is the single major caveat in working for the PSF. Things are not always moving as fast as they could, but we're still making steady and very healthy progress."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: On which parts of Python are you working as Python developer? Which parts interest you most?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MAL: Since I wrote much of Python's Unicode implementation building on an initial prototype written by Fredrik Lundh a few years ago, I still maintain most of it. These days I tend not to have much time to actually do coding work, but I try to overlook the general design and make sure that it stays in line with what the original idea behind the Unicode integration."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Can you explain the original idea around the Unicode integration? It might be handy for some of the developers to keep it in mind when developing.</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MAL: The original idea was to make 8-bit strings and Unicode behave well when they meet each other in the interpreter workings. We had long discussions about how to implement this properly and I think the current decision to use the ASCII encoding as basis for string/Unicode inter-operation was a very practical one. Everybody dealing with XML or similar Unicode aware technology will have come across the infamous "UnicodeError: ordinal not in range(128)" -- that's "explicit is better than implicit" in real life :-) """), style=styles["2cola"]))

    story.append(Paragraph(("""My suggestion to developer dealing with Unicode is to always work with Unicode in the core of an application and to explicitly apply conversions to the data at the application interface boundaries, e.g. GUI interface, file I/O, etc."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Last year, you were one of the co-organizers and one of the chairmen of the Python Business track. What can we expect this year from the Business track and what did you learn from last year?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MAL: I learned a lot in the process of co-organizing EuroPython 2002."""), style=styles["2cola"]))

    story.append(Paragraph(("""Trying to organize such an event in the same way as you would an open source project simply didn't work out. The difference between working on e.g. Python and conference is that there are hard deadlines, a tight budget, real money and lots of small tasks involved which nobody really likes or cares about, but which still have to be done to make such an event a success."""), style=styles["2cola"]))

    story.append(Paragraph(("""My feeling is that a bazar style management is not the right approach here. The cathedral works much better, since it gives more control and also assures that unpleasant work is done right. People shouldn't be afraid to get paid for this."""), style=styles["2cola"]))

    story.append(Paragraph(("""Anyway, it was interesting to work on a conference and I might do it again in the future.
    """), style=styles["2cola"]))

    story.append(Paragraph(("""Regarding the business track this year, I think we have a set of very interesting talks covering everything from Python awareness marketing to proven business solutions. The Python market is growing and now is a great time to participate in it."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Aren't you afraid that paying people for organizing an event like EuroPython would just raise the budget too high? You could fill the gap by attracting sponsors, but at the moment most possible sponsorships seem to be already too high (i.e. not many companies support EuroPython financially).</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MAL: EuroPython is a very valuable event for Python users and companies investing into Python. If you compare the pricing structure to other similar events such as e.g. the ACCU conferences in the UK, you'll find that it would well be possible to pay the organizers for the work they put into the event."""), style=styles["2cola"]))

    story.append(Paragraph(("""As side effect it would also make the conference more attractive to larger companies because it adds more "value" to the event (the more you pay, the more important it is). These companies could then have a big impact on the sponsoring budget item."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Isn't the Python community still too small? Is the amount of large capital companies big enough to provide visitors which can pay high entrance fees? Don't you think you would lose another very interesting group of Python developers, e.g. the scientists, small companies, etc?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MAL: The Python community is not as small as you might think. The fact that only few attendees at EuroPython come from large companies suggests that there is not enough awareness at these companies."""), style=styles["2cola"]))

    story.append(Paragraph(("""What Python really needs is to grow up in this respect and tap right into these structures. The potential is certainly there, but people seem to be generally afraid of losing the warm fluffy community feeling when opening themselves to IT professionals like IBM, HP, Microsoft, etc."""), style=styles["2cola"]))

    story.append(Paragraph(("""Other events such as Linux World have shown that this needn't be the case and we could well benefit from their marketing experience which would then open more doors and generate more corporate awareness."""), style=styles["2cola"]))

    story.append(Paragraph(("""That said, I do think that there's a pricing structure which would benefit all parties, e.g. by implementing ideas like the DevDay we had at the IPC conferences or tutorial days which are not part of the standard entry fee."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: You speak of low awareness of EuroPython to certain companies. Since EuroPython is organized in an open-source-spirit way, we hoped that everybody would put in some effort and the word would have been spread around. Is it your opinion that this didn't happen? This would indeed imply that the present method of organising doesn't work. Or is it just that these companies automatically think that low-budget conferences are not worth it?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MAL: I don't think that a voluntary organization is a less perfect way of approaching a conference and you all did a great job in managing it and getting the word out to the open source community."""), style=styles["2cola"]))

    story.append(Paragraph(("""However, in my experience, getting the word out to companies not active in the open source world needs a different approach, e.g. press releases have to be sent out to news sources and press release services, direct mailings help reach those companies which you are particularly interested in, ads in popular magazines can also raise the awareness and credibility, etc. It may even be worthwhile to invite press to the conference. All this adds to the budget, of course."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Anyway, I don't think the organizers of EuroPython are afraid to expect money for the organization and the time they've put in ;-).</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MAL: Perhaps you should set up an online donation system like the PSF is currently underway in implementing ?!"""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: That's a good idea ;-)</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: When did you heard for the first time about Python?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MAL: I found it on an OS/2 Hobbes CR-ROM, whizzed through the tutorial and was immediately convinced that I had found what I had always been looking for (leaving BASIC, Assembler, various Pascals and FORTRAN behind)."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: What's your relationship to Python?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MAL: I use Python and C for just about everything I work on. In general, I use Python for the OO-style works and C for performance. The eGenix mx Extensions are an example of the way I work: performance relevant bits are coded in C and then glued together using Python. Higher level APIs are all coded in Python.

"""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Is there a general rule when you start using C? Do you first work everything out in Python and then start looking for the bottlenecks, or do you think ahead on these issues, before starting to program?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MAL: I know what Python is slow at and generally write these bits in C for performance. While the other way around is probably more common, there have only been few instances where I took the approach to implement something in Python and then rewrite it in C. However, I do prototype C implementations in Python."""), style=styles["2cola"]))

    story.append(Paragraph(("""There are also cases where a C interface is simply more convenient, e.g. in the case of mxDateTime which was basically written so that I could interface date/time values back and forth between mxODBC and Python."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Can you tell us something about eGenix? When did you decide to start your own company? And why did you choose Python, while other more well-known languages might be more profitable.</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MAL: eGenix is managed as virtual company. I use my contacts in the Python world to find the right people for each project and also enjoy working on them myself."""), style=styles["2cola"]))

    story.append(Paragraph(("""The company has been in business since early in 2000 and has been active doing mostly project business. Last year we have been moving more towards a product based company and we expect the product revenue share to grow over the next couple of years. Our new Zope product, the mxODBC Zope DA is a good success and we intend to offer more products in that area."""), style=styles["2cola"]))

    story.append(Paragraph(("""Why Python ? I think that Python offers a much more productive approach to business solutions and so do our customers. It is sometimes hard to convince new customers of going the Python way, but once you talk them into it (mentioning Jython for Java-buzzword compliance usually helps a lot), they find out that they get more for less by deploying this technology."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: How do you convince new customers? Do you use the same arguments the most of the time? Which arguments have the most success?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MAL: Ok, here's a recipe:"""), style=styles["2cola"]))

    story.append(Paragraph(("""The most successful approach has been to talk in terms of solutions rather than using highly detailed technology presentations."""), style=styles["2cola"]))

    story.append(Paragraph(("""If you get a feeling that the other knows a bit more than just the usual set of buzzwords, then you can test the waters and try to find out whether they are aware of Python. If they are, then there's a chance that bringing up the Python topic can gain you something. Python is still not well-known enough to not frighten management."""), style=styles["2cola"]))

    story.append(Paragraph(("""Support, finding programmers, etc. is a big issue."""), style=styles["2cola"]))

    story.append(Paragraph(("""If using Python as argument for your solution does not fit the situation, it's better to say you're programming C with a smart toolkit on top of it :-)"""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: How do you handle the 'support' and 'Python programmers' issue? Would a Python Job-market be a good idea?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MAL: We already have a Python job market. I don't know whether it actually works or how many contacts are made this way."""),
    style=styles["2cola"]))

    story.append(Paragraph(("""For virtual companies, support usually means email support, plus second level phone support for resellers. Email is great for delegation and works around the world. Phone support is expensive, so we restrict that to the absolute minimum."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: By "virtual company" you mean that it is rather small as fixed personnel, but that it is big with consultants? Is a project manageable with people all over the world? What's your experience with this?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MAL: Whether a project is manageable without direct contact depends on the project itself and on how much trust the customer is willing to put into the quality of the project management and lead. I've found that software projects can very well be organized in this way -- using the same tools that we build upon in the open source world."""), style=styles["2cola"]))

    story.append(Paragraph(("""The advantage of the approach is obvious: you can access a much greater skill set and have people work on the project around the clock if needed."""), style=styles["2cola"]))

    story.append(Paragraph(("""Not employing programmers also reduces financial risks for the company itself e.g. in case of economic decline."""), style=styles["2cola"]))

    story.append(Paragraph(("""TheKompany is a well-known example of a company where this works very well."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: You offer commercial products written in Python, an open source programming language. What's your experience in this field? Aren't people in the open source world more used to paying for consultancy?"""), style=styles["2cola"]))

    story.append(Paragraph(("""MAL: That is an interesting question. I have found that many people in the Python and Zope world are not very keen on getting paid support (they don't mind getting free support ;-). Fortunately, the situation is a little different for paid products."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: That's an interesting experience as well, I didn't expect this.</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Aren't you afraid by making commercial products that you lose some of the benefits of open source, such as accessibility to a big audience and a wider group can provide improvements to the code?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MAL: No. I generally receive very little feedback for the open source parts of the eGenix mx Extensions. Looks like everybody is mostly happy with them as they are :-)"""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: How do you handle the license schema? I.e. the mixture of commercial and open source projects in a same application</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MAL: Yes. With the commercial bits protected by a licensing scheme."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: On which Python projects are you working at the moment?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MAL: The Zope DA is pretty much done and works great. Next to come are new releases of egenix-mx-base and mxODBC 2.1 (which already happens to be part of the Zope DA). I'm also working on mxLicenseManager which provides companies like eGenix a better way to ship commercial products to customers. The Zope DA already uses it, for example. """), style=styles["2cola"]))

    story.append(Paragraph(("""There are also three other projects I'm currently involved with, one building on Mailman and the two others in the application server space. All use Python as primary working horse."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: You've written a Zope product. What is your experiences on Zope and why did you decide to write an ODBC driver product for Zope?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MAL: Because I found that the state of Zope's SQL connectivity wasn't all that brilliant at the time (even Paul Everitt agreed here). Developing the Zope product was more work than expected, due to the missing documentation of so many important aspects in the Zope machinery. It turned out that reading the Zope source was the only way to get at all the needed details."""), style=styles["2cola"]))

    story.append(Paragraph(("""Now that we've completed that work, our customers don't have to bother with this any more and getting Zope connected to the world of ODBC compatible databases has become really easy."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Have you already looked at the Zope3 development concerning SQL connectivity?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MAL: Last I looked everything looked pretty much the same as in the Zope2 tree. eGenix will watch this development, of course."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Can you elaborate more on the Mailman project?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""Only this much: it deals with mailing lists that have a huge number of subscribers and is built around an SQL database."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: About which application servers are we talking here?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MAL: The eGenix Application Server (our in-house product, which is also part of the Siena Web Services Architecture) and custom Python-based application server platforms in general."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: How does the eGenix Application Server fit next to Zope and Twisted? Are the three systems comparable?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MAL: Zope is comparable to the eGenix Application Server (eAS). In fact, both were developed around the same time."""), style=styles["2cola"]))


    story.append(Paragraph(("""eAS is programmed using a component model of loosely coupled objects, much like Zope3 will be in the years to come. However, it doesn't restrict the programmer to a specific working model, so it is basically up to the implementation whether it wants to use an object based approach or rather go with the traditional separation of data and logic."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""The eGenix Application Server is also written with application writing in mind rather than mixing content management with web server operations. Adding new features is easy and that's the main benefit I'd expect from an application server."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Can't say much about Twisted, but I'll definitely have a go at it if the right project comes along."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Why did the need arise to create another application server?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Because there weren't any usable Python based application servers at the time (1995)."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Sorry, bad investigation on my part, the company Siena Technology was founded in 2002, so I thought it was developed around that time as well. What's the reason that Zope is more in the picture at the moment than other Python web application servers? Or is this just a wrong perception of the market?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MAL: Zope is very popular and has a large following. However, it's not the answer to all problems. In cases where you are more focussed on applications than presenting content or you have machine-machine interaction, different solutions are worth considering and provide more robustness, e.g. you don't want to a web service server to fall over just because someone added a service which causes all resources of the machine to be locked."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Do you think both products can learn from each other? Would a combination of both give birth to a new application server closer to a perfect ideal?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MAL: I thought about that, yes. It would be possible to run Zope as a service within the eGenix Application Server, for example. """), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Why did you start in developing the mx-base library? When did the need arise? And did you at that moment expected it would become so vast and popular?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MAL: Not really :-) I started out in November 1997 with mxDateTime and mxODBC. In 1998, I added mxTools, mxStack, mxTextTools and mxProxy."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""All of these were originally written for what now is the eGenix Application Server. I made them open source to be sure they run on as many platforms as possible and because I think that basic building blocks of solutions are always good candidates for open source projects. """), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: A lot of people use the egenix-mx-base products, wouldn't it be good to include them in the standard Python bundle?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MAL: I often get this question and from a licensing point of view there wouldn't be any obstacle in doing so. The reason why it is not done is that eGenix wouldn't maintain the product inside the Python development space. Our method of doing development is different from the Python one and we also care about things that PythonLabs is not all that interested in, e.g. Python 1.5.2 compatibility."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Not much of a deal, though, since Linux distributions are already starting to ship egenix-mx-base as part of the OS."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: In which way is your development method different from the PythonLabs?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MAL: Very simple: I'm not Dutch ;-)"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Interesting, do you want to imply that Dutch people have another way of development than the world? Or do you use an exotic way of development? :-)</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MAL: Ask Tim Peters, he knows the answer."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Seriously, software development is a creative process and sometimes the mindsets of different groups of people don't match. We'd rather listen to our users than having to discuss e.g. backwards compatibility or coding style with PythonLabs."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What do you like best about Python?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MAL: The elegance and swiftness of the language."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What do you not like about Python?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MAL: The deprecation process (because it's implemented too fast)."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What is the next 'big' feature Python should have, according to you?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MAL: Nothing. The last few versions have introduced enough features already."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""In my opinion, the next releases should put even more effort into optimizing the interpreter (Python 2.3 is already a lot faster than 2.2 and 2.1). I think there's a lot which can be done by giving the Python byte code compiler more hints as to which optimizations are possible, e.g. about constantness of certain object values, etc.

"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Finally, what do you expect from EuroPython?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MAL: Lot's of fun and good contacts. I really enjoyed last year and am looking forward to this year's event in sunny Charleroi. Hope to see you all :-) """), style=styles["2cola"]))
    
    story.append(Paragraph(("""http://www.europython.org"""), style=styles["2colar"]))
    
    story.append(Paragraph(("""People who would like to find out more about the mentioned topics, have a look at:"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""http://www.egenix.com/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.europython.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.gnu.org/software/mailman/mailman.html/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.python.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.python.org/psf/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.siena-tech.com/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.thekompany.com/home/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.zope.org/"""), style=styles["2cola"]))


    ## End of interview
    
def getHudsonInterview():
    ## Start of interview
    #Interview with Hudson

    story.append(NextPageTemplate('TwoColPlus'))
    newPage()
    story.append(Paragraph('Interview of Michael Hudson<onDraw name="myOnDrawCB" label="Michael Hudson"/>', style=styles["h3"]))
    #story.append(Paragraph(("""Interview of Paul F. Dubois of Numeric Python Fame"""), style=styles["Normal"]))
    story.append(FrameBreak())
    story.append(NextPageTemplate('TwoCol'))

    story.append(Paragraph(("""Tom Deprez"""), style=styles["2colar"]))
    
    story.append(Paragraph(("""<b>EuroPython: Hi Michael, can you introduce yourself to our readers?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""Michael Hudson: I'm Michael Hudson, a 24 year old mathematics PhD student at
    the University of Bristol in the UK."""), style=styles["2cola"]))

    story.append(Paragraph(("""I'm chairing the Python Language Track at EuroPython 2003."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: What can we expect from the Python Language Track? Can you whet
    our appetite?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MH: I had a difficult time trying to guess what people would find interesting
    as talk subjects, so I've organized a series of talks that I'm expecting to find very interesting, and hoping
    everyone else does too. We have talks on extending Python, on bridging Python to other language runtimes, on
    integrating Python into an operating system, explaining some of the deep and powerful parts of the language,
    and many others."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: When did you hear about EuroPython?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MH: Martijn Faassen emailed me in 2001 when he was kicking off the process that led
    to EuroPython 2002, although I only attended EuroPython last year and this year I'm helping to run it."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Any peculiar things you still remember from EuroPython2002?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MH: Not really... Google can find you my diary from last year's conference, which I'd
    have to read to refresh my memory anyway."""), style=styles["2cola"]))

    story.append(Paragraph(("""[editor note: For the curious readers: http://starship.python.net/crew/mwh/europython.html]"""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Ouch, not a good point for EuroPython... If it would have been a great
    congress, you clearly would have known something.</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MH: Well, you did ask if there was anything peculiar I remembered . I mostly remember
    a sense of having fun and being interested."""), style=styles["2cola"]))

    story.append(Paragraph(("""Actually, probably the best bit of EuroPython 2002 for me was Armin's
    talk on psyco."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: What's your relationship with Python?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MH: If I remember right, I first encountered Python back in 1997 in an article in Byte,
    but it wasn't until I went to university later in the year -- and got a fast internet connection right in my room --
    that I really learned the language. I submitted a few patches here and there over the next few years, joined python-dev
    late spring 2000, was given commit rights to Python CVS in 2001 and somewhat accidentally ended up doing the bulk of
    the work for the 2.2.1 release in April 2002."""), style=styles["2cola"]))

    story.append(Paragraph(("""Just another student who uses up his free time hacking Open Source software..."""), style=styles["2cola"]))

    story.append(Paragraph(("""[editor note: this article was by jeffrey shell, an original developer of Zope and
    employee at DC/ZC]"""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: How can someone accidentally do the bulk of work for a certain version?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MH: Well, it was a maintenance version, so the bulk of the work was deciding whether a
                            checkin to the trunk of Python's CVS repository qualified as bug fixes, applying them to
                            the release22-maint branch and so on. I was just monitoring the python-checkins list and
                            porting the occasional fix when the idea was suggested that 2.2.1 should be released fairly
                            soon. That was when a week suddenly disappeared into searching CVS logs for fixes, the
                            trackers on SF for bugs, fixing last minute crashes, editing web pages and so
                            on."""), style=styles["2cola"]))

    story.append(Paragraph(("""I helped a bit for 2.2.2 as well, but that was a more distributed effort."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: You attend several PyPy sprints. Can you tell us something about this project?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MH: "Several" == "Two" == all the sprints we've had so far"""), style=styles["2cola"]))

    story.append(Paragraph(("""PyPy is an attempt to implement Python in Python, which is an idea that appeared from
    time to time. The current project apparently emerged from a discussion on the German Python list, particularly
    between Christian Tismer and Holger Krekel -- both of whom I met at EuroPython last year."""), style=styles["2cola"]))

    story.append(Paragraph(("""Holger set up a mailing list and a subversion archive and so on and organised the first
    sprint in Hildescheim back in February. The second sprint (which I am actually at now as I type this has been
    organised by Laura Creighton and Jacob Halln in G?thenburg and there's a third sprint planned for just before EuroPython."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: What do you expect from this project?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MH: Firstly, another implementation of Python. We hope that this new implementation
    will be more flexible -- enabling tweaks to algorithms and structures used by the interpreter to be prototyped
    more easily, for example -- and, maybe, hopefully, faster using techniques somewhat like psyco (although it
    currently runs something like a thousand times slower...)."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: With another implementation, you mean a new implementation built from
    the ground up?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MH: More or less, yes. Currently we depend on the extant CPython implementation both
    as an implementation host and for borrowing bits of functionality that we don't yet implement. But the goal is
    to achieve a self-hosting system independent from any existing implementation, yes."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: At the end, if the PyPy project will succeed, will it mean that
    specialized compilers as Psyco become unnecessary? Are people like the developers of Psyco involved in PyPy?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MH: The developer of psyco is Armin Rigo and he is heavily involved in PyPy, yes.
    He's giving a talk on PyPy as part of my track."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Interesting, so his talk will be really fresh, since the next PyPy
    sprint is right before EuroPython at Louvain-la-Neuve!</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MH: Yes! Actually I'm expecting quite a few speakers from my track at
    Louvain-la-Neuve: Alex Martelli, Anna Ravenscroft and Christian Tismer will all be there and are all
    giving talks in the Python Language track. I will be really unimpressed if they are still writing their
    talks and not hacking PyPy."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: How will PyPy handle platform differences? Or isn't this an i
    mportance for PyPy?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MH: This is not yet much of an issue. People are using Linux, Windows and
    Mac OS X to develop PyPy, so a certain amount of portability is likely to maintained."""), style=styles["2cola"]))

    story.append(Paragraph(("""For now, we basically inherit portability from CPython. We will have to worry
    more when we attempt to make the implementation free-standing."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: How does the project get along?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MH: Reasonably well, I think. Currently, pretty much all the work that has been
    done has been done at the two sprints, at least partly because the whole system architecture has been in a
    state of flux and it's easier to have the courage to break everyone else's code when you can explain to them
    in person what they have to do to fix."""), style=styles["2cola"]))

    story.append(Paragraph(("""Hopefully by the end of the current sprint, or maybe the next one, which is
    only a few weeks off, we will have reached a point where there are multiple, largely independent areas
    to work on which suits the usual mode of open source projects much better."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: So, after these initial sprints, the PyPy initiaters hope to
    attrack more people in the project? Which independent areas does PyPy have in mind? Does the PyPy project
    has already an official website?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MH: That's the idea, yes. As to which areas need work, I fear this margin is
    to small to contain an explanation"""), style=styles["2cola"]))

    story.append(Paragraph(("""There is a website: http://codespeak.net/pypy/"""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: On which other Python projects are you working at the
    moment?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MH: Well, I do some work on CPython itself. We're coming up to the release of
    version 2.3 and so are in bugfixing mode for that."""), style=styles["2cola"]))

    story.append(Paragraph(("""I also have written a readline-a-like, pyrepl, that occupies a certain amount
    of my time, but this project has reached the point where it is good enough for the purposes I originally
    wrote it for, so I don't do so much work on it any more."""), style=styles["2cola"]))

    story.append(Paragraph(("""Other than that, I use Python for a bunch of unreleased and unreleasable tasks,
    from analyzing and creating plugins for Escape Velocity Nova to things other people probably use their
    desktop calculator for."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Why do all these different Python implementations exist, e.g.
    Jython, CPython? Is there a big need for them? I can imagine that it confuses some beginners in the Python
    language.</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MH: Well, the ultimate goal of PyPy is world domination Eventually we want to
    have the best, fastest, most flexible implementation of the Python language around."""), style=styles["2cola"]))

    story.append(Paragraph(("""OTOH, that's certainly not what we have today, so any newcomer to the language
    who thinks that PyPy is the implementation she should be using is clearly getting her information from
    the wrong sources."""), style=styles["2cola"]))

    story.append(Paragraph(("""I don't see much Jython/CPython confusion out on comp.lang.python, so I'm not
    too worried about this aspect. We need to maintain very good compatibility with CPython, and that's
    a goal."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Can you tell us more on pyrepl?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MH: Well, I describe it as a "library for building command-line interfaces".
    It's similar to the GNU readline library, but with support for multiline editing and a more flexible
    interface (this latter largely comes for free with being implemented in Python, not C."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Will pyrepl one day be implemented in the python core? Or is
    there a reason why it shouldn't? Looks a nice improvement to me.</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MH: pyrepl probably isn't sufficiently mature or portable to be part of the
    Python core as yet, but one day it would be nice."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Erm, "Escape Velocity Nova"?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MH: It's a game by Ambrosia Software:"""), style=styles["2cola"]))

    story.append(Paragraph(("""http://www.ambrosiasw.com/games/evn """), style=styles["2cola"]))

    story.append(Paragraph(("""One of the problems of buying an Apple laptop was putting me back in range
    of these guys."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: What do you like best about Python?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MH: I think, probably, the single biggest plus of Python is that it blurs the
    distinction between using and programming a computer."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Can you elaborate on that?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MH: Well, the other day on IRC I was trying to help someone with some file
    format problems. It turned out that all he needed to do was remove the first 128 bytes of the file, and he
    didn't know how to do that. If he'd been familiar with Python, this would have been trivial."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: What do you not like about Python?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MH: Occasionally, the lack of syntactic flexibility means that you have to type
    more boilerplate than you really want to. But the cultural tendency against magical source-compressing
    techniques overall counts as a plus."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: What the next 'big' issue Python should have, according to
    you?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MH: I don't know. Like my pyrepl package I mentioned earlier, Python works
    "well enough". If there were no major changes to Python between now and the end of eternity, this would
    hardly be a major disaster."""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: What's your experience with clearing rhododendrons?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MH: They burn really well."""), style=styles["2cola"]))

    story.append(Paragraph(("""[editor note: For the confused readers: Have a look on the personal
    website of Michael]"""), style=styles["2cola"]))

    story.append(Paragraph(("""<b>EuroPython: Finally, what do you expect from EuroPython?</b>"""), style=styles["2cola"]))

    story.append(Paragraph(("""MH: I expect a lot of really interesting talks, and to meet a lot of
    really interesting people!"""), style=styles["2cola"]))

    story.append(Paragraph(("""http://www.europython.org"""), style=styles["2colar"]))
    
    story.append(Paragraph(("""People who would like to find out more about the mentioned topics, have a
    look at:"""), style=styles["2cola"]))

    story.append(Paragraph(("""http://codespeak.net/pypy/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://pyrepl.codespeak.net/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://starship.python.net/crew/mwh/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.ambrosiasw.com/games/evn/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.europython.org/"""), style=styles["2cola"]))


    ## End of interview
    
def getFaassenInterview():
    ## Start of interview
    #Interview with Martijn Faassen

    story.append(NextPageTemplate('TwoColPlus'))
    newPage()
    story.append(Paragraph('Interview of Martijn Faassen<onDraw name="myOnDrawCB" label="Martijn Faassen"/>', style=styles["h3"]))
    story.append(FrameBreak())
    story.append(NextPageTemplate('TwoCol'))
    
    story.append(Paragraph(("""Tom Deprez"""), style=styles["2colar"]))
    
    story.append(Paragraph(("""<b>EuroPython: Martijn, please introduce yourself to our readers.</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Martijn Faassen: I'm from the Netherlands, and co-run (with Kit Blake) a small software
    development company here, called Infrae (http://www.infrae.com). My
    interests are fairly diverse, in a geeky way. I like computer
    programming, history, philosophy, strange humor and games, for
    instance. Since I've been fairly active in the Python and Zope
    communities for some years quite a few people in those communites know
    about me."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: You are the chairman of the Python Frameworks track. What
    can we expect this year and did you learned something from last year?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MF: Last year I ran both the Python Language and Python Applications
    track, besides helping out to organize the conference itself. This was
    too much for me to juggle, so this year I've taken it easier. Though I
    still have helped some in the general organization, my main task is to
    run the Python Frameworks track. There are many frameworks from small
    to large in Python to do a wide range of things. One important reason
    we have so many even with the comparatively small resources of the
    Python community is because Python makes it so easy to develop
    frameworks."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""The track has talks ranging from discussions of GUI toolkits to XML
    processing frameworks. And of course also a number of talks about the
    famous Twisted. Two of these latter talks are part of the special
    Moshe Zadka track we seem have at the conference. If they are canceled
    that would be as he will have collapsed with exhaustion by day 3. The
    track being about frameworks means many of these talks will feature
    some tutorial style content -- frameworks are meant to make your life
    easier, and the track's here to help you learn how."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Can you tell us more about Infrae? Where is it situated?
    The philosophy, ...?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MF: We're a small company based in the middle of Rotterdam, the
    Netherlands. For the last half year we've been hovering between 4 to 6
    people working here in the office, not including Torvald, a big
    norwegian forest cat (Kit's cat). Before that we were smaller. We also
    have a wide network of freelancers within the Python and Zope worlds
    that we draw on occasionally."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Our philosophy is evolving, but it definitely involves us being active
    in the Python and Zope communities. We like the community, and it's a
    very useful network to draw on for business reasons as well. We
    produce open source software, in a customer-driven fashion. If a
    customer wants something and pays for it, we create or extend a system
    to do it. Of course we try to keep the new development general enough
    to be useful for others as well. So far that has worked quite well for
    us, probably partially because many of our customers are from the
    academic sector and may be more in tune with open source. The whole
    open source business strategy is a permanent experiment, though --
    we'll see how it develops over time."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Since we like developing new things and would like to improve the
    foundation, we've also put quite some time in projects like Zope 3 and
    Forest, so we're not completely customer-driven."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What's your relationship with Python and Zope?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MF: My main contributions to the Python community are me helping to
    organize EuroPython and my very wacky postings about the PSU that I
    deny any knowledge of to the comp.lang.python newsgroup. I haven't
    been contributing a lot codewise, though there are a few little
    modules here and there now."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""I've contributed significant amounts of code to the Zope community
    over the years though. Well known in the community is Formulator. I
    also maintain a few other Zope products, such as ParsedXML. And in the
    last year and a half I've been the lead developer for Silva, Infrae's
    structured content management system based on Zope
    (http://www.infrae.com/products/silva)."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""I make my living using Python and Zope in the context of
    Infrae. Infrae is an open source company and we like the community
    aspects a lot."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Can you tell more about Silva, the product behind Infrae?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MF: Silva is a content management system, even though Kit would prefer to
    use another term as 'content management system' is so vague. So we say
    Silva is an 'authoring and publication system'. Silva is focused on
    managing large quantities of structured content. This content can be
    presented within a web site, but can also be exported as XML and
    transformed (for instance to MS Word). Since we are aggressive about
    keeping the content entered into Silva clean and structured it is
    future-proof and media-independent."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Silva can be used by just a single user, but it shines in large
    organization, as its tree based structure is designed to scale. Silva
    can be hooked up to user authentication systems like LDAP and Silva's
    user interface makes it easy even large quantities of users. Silva
    also has a multi-version workflow, an extensible metadata system, and
    much more."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""By the way, while Silva is very important for Infrae we're definitely
    also interested in doing other Python or Zope related development
    (hint hint). We might find a way to integrate it into Silva though. :)"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What made you decide to build your own Zope framework system, 
    except starting from existing ones, like CMF, CPS, Kontentor, Plone?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MF: We started building Silva innocently and unambitiously back in late
    2001. While CMF had been around, it had been in vaporware stage
    (portal toolkit, etc) for so long I was wary of going with it then. Of
    course it matured a lot since then and gained tremendous momentum, so
    it became more attractive. Then again, going your own way has its
    advantages too, as you can determine your priorities (and fulfill
    those of your customers) yourself."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""I don't think we'd heard about either Plone or CPS yet; I'm not sure
    whether either was around at the time. An early version of Kontentor
    was actually around I think. Joachim Werner (main developer of
    Kontentor) told us that it is actually inspired by some of the same
    ideas Silva has, as both derive conceptually from a very early
    implementation of XMLWidgets I did in 1999/2000 or so. But still our
    ambitions weren't so high, and we had our own ideas..."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Over time of course Silva quickly developed and became more ambitious,
    driven by customer requests. We generalized a number of the components
    more and more, and the user interface has received endless polishing
    over time."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Which customers are using Silva? Can you point us to some
    websites as examples of use?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MF: The biggest 'sponsor' of initial Silva development was the Erasmus
    University here in Rotterdam. They use Silva for parts of the website,
    but also and especially use Docma to export that same information to
    study guides that are printed on paper. This is an example of a
    Silva site at Erasmus:"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""http://web.eur.nl/master"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Another large contributor to Silva development is ETH in Zurich,
    Switzerland (Swiss Federal Institute of Technology). They run quite a
    number of Silva sites at various departments, among others:"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""http://www.mat.ethz.ch/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.vc.ethz.ch/"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""The Materials site has a page on Silva there as well:"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""http://www.mat.ethz.ch/silva"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""And I just found out from Kit they have a Silva course, which looks
    pretty neat!"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""http://www.mat.ethz.ch/silva/course"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Another customer that uses Silva is the Bijvoet Center of Biomolecular
    Research at the University of Utrecht."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""http://www.bijvoet-center.nl/glycotrain"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Besides these and other customers there are also a quantity of Silva
    sites that we may not even know about (though we know about some as
    the developers post on our mailing lists). Because it is so easy to
    plug in completely different layout templates in Silva, it is hard to
    recognize a Silva site even if you run into one!"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What is Formulator?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MF: Developing web applications means you have to write a lot of web
    forms. Being a conscientious programmer, I kept worrying about
    validation of the user input into the form, and kept finding myself
    very similar validation functionality over and over again (is this
    field required? not too long? etc). So being also a lazy programmer I
    started writing a Zope system to make the validation and also
    generation of web forms easier. I also tried very hard to make it fit
    inside of Zope's through the web user interface."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Apparently I did it well enough for people to decide to put Formulator
    in their own Zope toolchest. I regularly get mail from all over the
    world from people who use Formulator. We at Infrae use Formulator in
    many places ourselves, and currently the requirements of Silva tend to
    drive Formulator development the most."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""In Zope 3 the Formulator concepts evolved further and this turned into
    Zope 3's Schema and Forms. The core of this code was written by me,
    though now it passed through many hands and underwent many
    changes. Zope 3's schemas and forms are used throughout the Zope 3
    codebase."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: When did you hear for the first time about Python and Zope?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MF: I discovered Python somewhere in '98, though I heard about it first
    some years before that, I think. At the time I was programming a lot
    in C++ and Delphi, and I was looking for a scripting language to
    supplement those. I messed around with Perl for a week or two and
    while I could recognize its power I couldn't get it into my head."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""So one night I ended up on www.python.org, and downloaded and printed
    Guido's tutorial. I read it before going to sleep, and the next day I
    could program in Python. Inexpertly, but I soon fixed that, and I fell
    in love with the language. Quickly forgot about Perl, and didn't do
    much anymore with C++ or Delphi either."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""I went to work for the University of Utrecht only a few weeks after
    that and had the great fortune that I could introduce and use Python
    there right away. Since then I've been making my living programming in
    Python!"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Then when Zope was open sourced somewhere around the end of 1998 I
    started using it for work soon after. In 2001 I and Kit Blake founded
    Infrae, a Python and Zope development company."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: On which projects are you working at the moment?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MF: Silva and associated projects take most of my time at Infrae right
    now. My task is to guide the other developers on the project in their
    activities when it comes to the code. Recently the Silva user
    interface got a big overhaul both codewise and in its look and
    feel. We've also added an extensive metadata system and improved the
    performance of various parts of the user interface and public pages
    significantly. Silva 0.9.2 is released in beta form now and it offers
    a vast amount of new features compared to the 0.9.1 release -- please
    check it out! We also have a system called Docma which allows for
    integration of Silva with Word; it can both export Silva content to
    Word as well as import Word content into Silva."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""My biggest new project is Forest, the XML database I will be talking
    about at the conference. It can store and index arbitrary XML data and
    has an implementation of the XPath query language. My dream is to use
    it as a storage backend for Silva's XML data at some point in the
    future, when we have ported Silva over to Zope 3."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Another new project I'm working on is an OAI-PMH client for Python,
    along with Zope integration and Silva integration. OAI-PMH stands for
    "Open Archives Initiative Protocol for Metadata Harvesting", an
    enormous mouthful I love to roll off my tongue like it is star trek
    technology doubletalk. It's an XML over HTTP protocol for the
    harvesting of metadata about resources, such as publications."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""A university for instance has a lot of people publishing scientific
    papers. If they have a server serving out OAI-PMH they can expose
    metadata about these papers to external harversters (title, author,
    etc). Other people can then build applications using the harvested
    information. We've built a basic harvester (a Python module), and
    we're also building applications on top of it -- Zope integration
    using the ZCatalog as well as Silva integration."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""I'm not working on these projects alone -- Silva especially has had
    many different people contributing to it, both within Infrae and
    through the open source process. Working in a team is a lot of fun, as
    everybody has their own strengths. I for instance suck at user
    interfaces, but Kit and Jan-Wijbrand are great at them."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""I'm also very interested in Zope 3 and have contributed some things to
    it; I kicked off the development of Zope 3 schema and have recently
    been active in the integration of XML facilities into Zope. And in
    december 2002 Infrae organized the Zope 3 'Sprintathon' here in
    Rotterdam, which was generally thought to be very successful."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Can you tell us more about Forest? How far are you in the 
    development of it? Will it be a stand-alone product?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MF: Forest is a stand-alone system, but it does currently require ZODB4
    (which hasn't been released yet but is developed along with Zope
    3). It should be easy enough to port it over to ZODB3 and it shouldn't
    be too hard to port it over to layer over other data storage
    technologies like Berkeley DB or Metakit."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""I intend to provide tight integration with Zope 3 when the time comes,
    however."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Forest is far enough along to be able to store large XML datasets and
    it support a large part of XPath. It's not far enough along yet to be
    ready for production use, and I intend to do a lot of performance work
    on it still as well. It *is* far enough along for other people to drop
    in and join in with the fun, however. If you are interested in XML,
    query language implementation and databases then do drop me a mail. :)"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""For more about Forest please come to my talk!"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Only one question on this subject, you talk about an XML
    database, however, you use the ZODB or other data storage technologies
    as backend. Can you then still talk about an XML Database?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MF: Forest uses an underlying data storage technology such as the ZODB and
    its BTree facilities. It however creates its own indexes and
    datastructures optimized for efficient queries of XML data in
    particular. This is similar to the way the ZODB and use Berkeley DB as
    an underlying storage, or to the way the Zope Catalog adds indexing
    facilities and querying facilities to the ZODB. Another example is
    MySQL, which can use Berkeley DB as well to offer a transactional
    storage. So I guess "XML Database" is still the best word I have for
    this. :)"""), style=styles["2cola"]))
     
    story.append(Paragraph(("""<b>EuroPython: Is Silva already preparing the port to Zope3? What do you
    think of Zope3? Will people still be able to create the same projects
    as they do now or is the new version more heading to Content
    Management Systems?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MF: Zope 3 is definitely general enough for any web application, not just
    content management systems. It pushes the Zope 2 concept of combining
    building blocks into applications much further still with its
    component architecture. As a Python programmer Zope 3 is very
    exciting. The Zope 2 gods require a large amount of concessions and
    sacrifices in Python code. Zope 3 has been designed to create a
    minimal burden to the Python programmer instead -- I can just write
    Python and *then* add all the stuff separately to make it work in the
    framework, such as a user interface. It also should become easier to
    combine different components into one application."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Infrae has not started porting Silva to Zope 3 yet, but we are
    actively planning for it. Since Silva content can all be exported as
    clean XML, it will be relatively easy for us to port over content from
    Zope 2 Silva into Zope 3 Silva. Silva is also 'prepared conceptually'
    to bridge the gap more easily -- we split views from content for
    instance, and use services throughout. That said porting to Zope 3
    will still be a significant effort. Customer contributions are
    welcome. :)"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: How do you handle the view-content split in Silva?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MF: It isn't too dissimilar from the way the CMF Skins mechanism does it,
    though we have different advantages of disadvantages. Silva since the
    fall of last year actually uses the same technology as CMF does to
    store Page Templates and Python Scripts on the filesystem instead of
    in the ZODB, which makes for much easier management of the code in a
    version control system like CVS. This functionaly has been extracted
    from the CMF core by Kapil Thangavelu into a product called
    'FileSystemSite', which we use."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Silva's content objects are implemented as classes in Python, in the
    normal way you write a Python product for Zope. Silva's views are in a
    separate 'view' hierarchy, which is full of page templates and python
    scripts, along with some XML serializations of Formulator Forms.  An
    individual view is actually a Zope folder in the view hierarchy (but
    this can be treated as a directory on the filesystem thanks to
    FileSystemSite). As a view is a Zope folder is acquires the content of
    Folders higher up, faking a simple kind of single inheritance in
    views."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Each content type normally has three types of view associated with it
    -- an adding view showing an add form, an edit view showing an edit
    form for the author (including tabs and the like) and a public view
    that determines how the content is rendered to the end user of the
    site (though this not include layout or navigation, which we add later
    outside the view system). Views are associated to content objects in a
    view registry. The Silva UI is composed mostly out of edit views with
    some add views, and the public website is composed from a few layout
    templates placed in content space along with the public view of the
    objects."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""This model/view architecture allows us to easily vary the two
    independently from each other. We rewrote large parts of the view code
    in the recent months, but the content code did not change that much,
    for instance."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""The CMF skin mechanism is quite similar. The difference is that CMF
    Skins are a stack while Silva views are a tree. A disadvantage of CMF
    skins in my mind is that a single skin can contain views for any
    content object, which can make things rather messy if this isn't
    organized very strictly. Also there doesn't seem to be as easy a way
    to make separate public views, but I may be mistaken there.  An
    advantage of the CMF skins compared to Silva views is that the
    extension mechanism of CMF skins is more flexible. It is easier to
    insert a skin in a stack than it is to graft a different sub-tree onto
    the main view tree in Silva, though we do manage to do it. :)"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What do you like best about Python?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MF: It doesn't constrain me as a programmer. I can generally express what
    I need without having to jump through hoops to make it work. It's
    concise but not at the expense of readability. It has just enough
    syntax; not too little and not too much. Python is a very *balanced*
    language, its main strength is that it does not have a main strength
    at the expense of other things."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What do you not like about Python?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MF: Aside from a few very minor nits here and there the main thing I would
    like to see improved is the relatively slow performance of the current
    batch of interpreters.  While for the majority of tasks Python is fast
    enough, and while one can always escape to extensions in C or some
    other speedier language, it would be nice if my Python code was
    blazingly fast as well. This is why I'm very excited about projects
    like Psyco and Pypy, which aim to improve performance. But Python's
    plenty nice enough right now for me to wait patiently for these to
    mature."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""One area of Python that I'm particularly interested in is the XML
    infrastructure so naturally I have suggestions on how to improve it
    too. :)"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What is the next 'big' issue Python should have, according
    to you?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MF: Increased performance would be my best candidate for the next 'big'
    thing for the language implementation itself. I don't have any
    suggestions for feature additions to the language itself; I'd prefer
    it if that remains relatively stable. On the other hand, it's always
    fun to experiment with new approaches..."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""One thing that will continue to happen is the creation of intriguing
    new libraries and frameworks in Python. Just witness the rise of
    Twisted in recent times, for instance. It's hard to predict what piece
    of Python software will be next making it big, though clearly I hope
    Zope 3 will be. Overall there seems to be a focus on the construction
    of larger application frameworks in Python recently. I suspect one
    enabling big change for that is the growth of unit testing in the last
    three years or so within the community. A good test harnass allows
    frameworks to become more solid and more ambitious."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: You've mentioned Zope and Twisted. How do you compare
    these two? Do you know of the Siena Web Services Architecture,
    mentioned in a previous interview here at EuroPython?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MF: I haven't had the opportunity to play with Twisted yet, though we do
    use it in our Docma server application. Zope has a focus on content
    and meta-content building blocks, all wrapped up in a user interface.
    Twisted focuses more on the lower level network architecture
    issues. Twisted is more for Python programmers while Zope can also be
    used to good effect by other people such as designers and people who
    program incidentally as part of another task."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""It is my hope that eventually someone will step up and integrate
    Twisted and Zope 3, so that we can gain the best of both worlds. There
    have been some smaller steps taken in this direction already, and the
    good intent is there, now it only needs to happen."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""I hadn't heard about the Siena Web Services Architecture until I read
    about it in the interview with Marc-Andre. :)"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Do you have a general view on how Twisted and Zope3 could be
    integrated in each other?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MF: Just a general view only. Zope 3 like Zope 2 currently contains its
    own networking code to handle things like HTTP and FTP. Twisted
    thrives on implementing networking protocols like this. If Zope 3
    integrated well with Twisted we'd get support for a *lot* of protocols
    for free, and the Twisted people would maintain them. The Zope 3
    people could then focus on the aspects of Zope 3 that do make it
    special, such as the component architecture and exposing it through
    its user interface."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Then again, Zope 3's HTTP server works just fine, so there is no
    urgent need in the Zope 3 community to start this integration
    activity. We may feel the need more when Zope 3 gets big customer
    deployments; Twisted may help it scale better."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: You like games, computer games? Have you looked at PyGame?
    Do you have an interest in creating one day your own computer game?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MF: I like games a lot, both computer games as well as non-computer
    games. I have been running an RPG campaign for friends since the
    mid-nineties now, but we only find time once every two or three months
    on average to get together to play. I used to play more computer
    games, but I have found less time than I might wish to play them after
    starting Infrae. Infrae's lots of fun too, though. :)"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""One computer game I like a lot is Europa Universalis II. It's a
    strategy game that simulates world history from about 1400 - 1800, and
    is quite detailed."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""I've been tracking PyGame since the project started, and played with
    it once every while. And yes, one of my ambitions since I first
    encountered computers when very young was to create my own games. Over
    time I've created a few half-finished attempts, and it is how I
    learned to program; I imagine I'm not the only programmer who followed
    this route. And it's still one of my hobby-ambitions to create some
    computer game one day. But then I have many such projects, so you may
    have to wait a few decades. :)"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Python and history simulations... You might have a look at
    Civil (http://civil.sourceforge.net/) and start from there. As a start 
    you could add a European theatre :)</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MF: I am aware of Civil, though I haven't tried it yet. Civil seems to be
    more about individual battles, while Europa Universalis is of a really
    grand strategic scope indeed. Battles are entirely abstracted, but it
    does feature intricate alliance diplomacy, new countries forming (the
    Dutch rebellion from the Spanish is in there for instance), and the
    reformation. I like open ended simulations. I also enjoy artificial
    life simulations of evolution for that reason (and I've written a few
    unpublished experimental ones in Python)."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""I have been vaguely considering writing a game of that type in Python
    someday. But that's just a thought, which is many steps away from a
    project. Again, a few decades perhaps... :)"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Can you reveal the secret around 'Python Secret
    Underground'? It might be difficult -since you aren't part of it-
    of course, but you might at least have heard something about it. :)</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MF: I have no idea what the Python Secret Underground even could be. I
    never heard of it. However if they existed they might be an ominous
    all powerful organization, as they would have access to Guido's time
    machine. This would give them the power to roam across time and space
    and branch of alternate timelines with abandon. There would be a rumor
    that in one perverse experiment they branched off a timeline where
    Python did not exist, probably just to see what would happen, at least
    that would've been my reason to do that, they might've been more
    serious."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""In that timeline, Zope ended up being written in some other language
    by Jim Fulton who shortly thereafter went entirely mad. He was locked
    up in a lonely monastery in Belgium where they brewed Chimay. On calm
    dark nights his cries "the braces, oh no, the braces!" could be heard
    from kilometers away. Incidentally Guido ended up there as well, as
    one day in some newsgroup he proposed a language that would do block
    structure by indentation only. Kind people reported him to the police
    and they kindly put him in the same monastery. I myself happily
    discovered Lisp instead of Python, and therefore grew more and more
    eccentric over time. I ended up founding an entity called the Flee
    Software Foundation, and giving speeches at conferences about how we
    are all doomed and we should flee software before it is too late. There
    also is a song I sing about it (don't ask)."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""People just didn't know any better in that timeline. The sad end was
    when they equipped all their missile launching systems with their
    timeline's version of Zope. Unfortunately the missiles implicitly
    acquired the wrong targets and humanity was annihilated. Even worse,
    we were all using other languages and there was no EuroPython."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""This is all entirely rumor though. I disclaim any knowledge of such
    matters, as the PSU does not exist."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Finally, what do you expect from EuroPython?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""MF: Besides learning about all the interesting projects going on I hope to
    meet new people as well as reacquaint myself with those I met
    before. It wouldn't hurt if we could drum up some more business for
    Infrae as well!"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""If you read this far, thanks for reading my incessant babbling! It's
    all Tom's fault, he kept asking more and more questions! Actually
    thanks Tom for all your effort. :)"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Finally, I won't be at any Python Secret Underground meetings at
    EuroPython, as there is no such thing as the PSU and there is
    certainly no plan to hold any PSU meetings on the Open Spaces
    track. Meaningless non-code phrase follows, please ignore: Today,
    Torvald does not like the banana. Fnord. Repeat: Today, Torvald does
    not like the banana. Fnord."""), style=styles["2cola"]))

    story.append(Paragraph(("""http://www.europython.org"""), style=styles["2colar"]))
    
    story.append(Paragraph(("""People who would like to find out more about the mentioned topics, have a look at:"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""http://www.infrae.com/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.zope.org/Members/faassen/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.openarchives.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.eur.nl/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.europython.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.twistedmatrix.com/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.python.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.zope.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.siena-tech.com/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://civil.sourceforge.net/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://pygame.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.europa-universalis.com/"""), style=styles["2cola"]))

    #end of interview
    
def getCouperInterview():
    ## Start of interview
    #Interview with Tim Couper

    story.append(NextPageTemplate('TwoColPlus'))
    newPage()
    story.append(Paragraph('Interview of Tim Couper<onDraw name="myOnDrawCB" label="Tim Couper"/>', style=styles["h3"]))
    story.append(FrameBreak())
    story.append(NextPageTemplate('TwoCol'))

    story.append(Paragraph(("""Tom Deprez"""), style=styles["2colar"]))
    
    story.append(Paragraph(("""<b>EuroPython: Hi Tim, tell us something about yourself.</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Tim Couper: I'm from England ( .. still part of Europe ..), studied
    mathematics at Edinburgh (BSc) & Oxford (PhD)."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Since those days, I've been working in IT for over 20 years (I even
    pre-date Microsoft), and have run a number of companies over this time
    - software, hardware, systems & telecomms integrators, and have been
    on the boards of disparate companies, from those providing compiler
    technology to satellite connectivity. This has given me lots of time
    to develop expertise in most of the major languages and o/s; then
    about 3 years ago, I discovered Python "by accident", and found it to
    be absolutely BRILLIANT for an extremely wide range of
    applications."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Within the Python/Open Source space, I'm currently the co-ordinator
    of the UK Python community conferences, on the Advisory Board of
    LinuxWorld UK, and am also the "Conference Compere" of the EuroPython
    Conference."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: So you are the owner of several companies?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""TC: I have owned, operated and sold several companies in my time;
    now I run TAC Software, a software consulting business, which focusses
    on providing solutions for companies with complex or large
    data-related issues."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: You must have a lot of experience and learned a lot
    over the years. Can you give some advice to just-starting companies or
    people planning to start?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""TC: That's a big subject! Be realistic about you can achieve, and
    about the products and services that you want to deliver. Plan
    pessimistically about cash flow and sales; realistically about
    expenses and costs. Find investors, if necessary, with whom you are
    comfortable, communicate clearly and openly your aspirations and
    expectations of them. And then be prepared to work hard, not only at
    technology, but at accounts, administration, sales, and all the other
    aspects of company life. And have fun while doing it: always laugh at
    least once a day, particularly at yourself."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: How big can TAC Software be seen? Do you mostly work
    with fixed personel or do hire on a freelance basis? </b>"""), style=styles["2cola"]))
     
    story.append(Paragraph(("""TC: Mostly with freelance specialists on a case by case basis."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Is TAC Software Python focussed? Or do you also use
    other languages, depending on the customer requests?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""TC: My preference would be to do Python/Jython work, because it is
    easiest to work in, can deliver great value in a short space of time,
    and the code produced is very maintainable. However, the nature of my
    role as a "technical authority" requires multi-o/s and multi-language
    skills."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Marc-Andre already gave a hint on how you can win a
    customer into the use of Python? Can you share with us your ideas and
    experiences?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""TC: In many companies, suggesting Python as part of a solution can
    only be done from a basis of personal credibility.  That is, someone
    in the client company is prepared to bet on you or your company to
    deliver something of greater value than if they chose an alternative
    route. Finding many of such customers depends largely on the cultural
    value given to risk-taking, so I suspect there are many more in the
    USA and Asia than Switzerland or Germany, with the UK somewhere in
    between."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""With my current client, as I shared at the last EuroPython, it was
    finding a couple of small problems whose rapid solution by Python gave
    it, and me, increased credibility. "Hanging out" with the technical
    teams is a good place to find needs. Listening to project managers and
    their problems informally, then equally informally providing
    suggestions, and solutions, makes a big difference. One of the key
    factors in its success was being able to have others with specialist
    expertise to come in and work on the project.  Having people of the
    calibre of Duncan Grisby & Marc-Andre Lemburg (both past and current
    speakers at EuroPython) able to give consulting time, makes a great
    difference in the eyes of the client. Maybe I should do another talk
    on this subject next year."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: How did you discover Python.  By accident?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""TC: I noticed the book "Python in 24 hours" on a friend's bookshelf
    and borrowed it to read for interest's sake, and in doing so realised
    just what a useful, fully OO language it was. It was clear from the
    book that Python offered much of what I'd been looking for for a
    number of years, but found it difficult to believe that if it was SO
    good, and yet straightforward, why hadn't I heard of it before."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""I subsequently went to the Python 9 conference in the USA hoping to
    find out what the REAL limitations of the language were, by meeting
    experienced developers. But I was amazed by the mixture of people and
    skillsets at the conference, and its use in such diverse applications
    as, for example, particle physics for large multi-dimensional array
    processing, transnational real-time games processing, neural network
    development, as well as its extensive business use - small, medium and
    large. And astounded by the claims of robustness across all the major
    o/s, which experience has shown to be true. And then to discover
    Jython too, providing the same elegance and functionality to the
    closed Java world."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""I'm sure this discovery of unbelievable breadth of scope is
    something that many will experience when they "stumble across"
    Python."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: When did you hear about EuroPython?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""TC: I was involved in its foundation, having attended the
    pre-Europython event in Bordeaux in 2001."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Together, with Marc-Andre Lemburg, you are the
    chairmen of the Python Business track again?  What can we expect this
    year and did you learn something from last year?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""TC: Last year's presentations were really excellent, and provided a
    forum for sharing ideas and learning."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: If I recall correctly, last year you were
    accompanied by your daughter at the conference. Did she enjoy it? Is
    she a Python user as well?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""TC: Yes, Fiona is studying for a PhD in Edinburgh in Speech and
    Language Processing. Her department works in a mix of Python, C++ and
    Perl, but is beginning to rationalise on Python, in which she is now
    quite proficient. We met with Guido at the recent UK Python conference
    who declared us to be a first in the python community: a first "father
    and daughter" pair. In the interests of completeness, I also have a
    son, Alex, studying Computer Science at Manchester University, who
    will be using Python for his final year project work."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What does the lady of the house say to all this
    Python? :-)</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""TC: My wife, Audrey has lived with me, computer technology and
    companies for these years. Our family isn't completely
    software-oriented: we have another daughter who is a primary school
    teacher and another son working in computer networks. So we're only
    half-Python."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Will they join us this year at EuroPython?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""TC: Unfortunately not. Maybe next year."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What is Siena-Tech?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""TC: This is a company that I and Marc-Andre Lemburg have created to
    promote the rather exciting distributed web-services/multi-tier
    architecture that we've deployed globally within a huge transnational
    pharmaceutical company."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Can you tell us something about the Siena Web
    Services Architecture?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""TC: It's a Python/C enterprise, distributed services system that
    enables services be written essentially as Python methods. It provides
    also a client-side portion which is presented as a COM object on MS,
    or as a .so on *nix, which makes the calling of the services as simple
    as .NET. So the development complexity of the client and server side
    software is reduced to application developer level. It also supports
    multiple interacting instances of Siena with auto-synchronisation of
    services. This enables server-side failover and can make use of
    client-side processing power (by having a completely synchronised
    Siena instance on the desktop, or on a departmental server)."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What's your relationship with Python?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""TC: Purely platonic. I'm representing python.org at the LinuxWorld
    UK (Birmingham NEC Sep 3-4 2003), where python.org is having a
    substantial stand, with speaker area, on the main exhibition floor
    (alongside Oracle, Linux, IBM ad Microsoft). I feel that this is a
    unique opportunity to raise the profile of Python to the several
    thousand visitors that attend the conference and have a number of
    volunteers from the UK python community to work on the stand, but we
    still need more! And it'll be an opportunity for individuals to
    promote their own services. (Anyone interested, please contact me
    tim@2wave.net)"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: On which Python projects are you working at the
    moment?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""TC: I'm completing the role as lead developer for the largest R&D
    data project (4 sites, 2 countries) that this pharmaceutical has ever
    done. Python has played a key role in being the language used to
    migrate the terabytes of data from all over the world, provide MQ
    broker processes and other client and server-side elements. As well
    as, of course, the vital Siena system, which is providing a
    middle-tier access to the Oracle database, and synchronising services
    to local instances of Siena in all parts of the world."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What do you like best about Python?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""TC: As a language - its natural, "obvious" format, well-
    engineered, robust, and it's applicable to a wide variety of problem
    domains. And it's free."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""As a community, a nicer bunch of people you couldn't find!"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""And the track record of its careful and thoughtful development
    ensures that software developed in it will be useable for years to
    come."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What do you not like about Python?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""TC: Really only its lack of "market penetration". It deserves
    better. But that's up to us ..."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What is the next 'big' issue Python should have,
    according to you?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""TC: Don't have any burning technical issuesq. It is perhaps a
    weakness of its Open Source roots that appears to restrict Python
    awareness to dissemination largely only by word of mouth."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Any ideas on how to improve this? Do you see some
    attitude changes towards Python, in the last few years?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""TC: There aren't any easy answers, but there are many who share
    this feeling within the Python community. We've seen valuable ongoing
    attempts to raise the profile through the Python Business Forum, for
    example, and in the focus of the Python track that Marc and I are
    leading. The effectiveness of these attempts has yet to be measured in
    the wider computing community."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Also the raising of Linux's profile (via IBM, Sun ..) and, European
    and other national government's interest in Open Source has given some
    validity to Open Source in general in Europe.  So purchasing
    departments in major corporations may consider Open Source Linux
    whereas they wouldn't have 5 years ago; and with that change in
    world-view - from purchasing proprietary o/s to purchasing Linux -
    comes a greater freedom of thought for their development staff, and an
    increased need for cross-platform products, tools and services."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""What makes this difficult is the reality that there are well-funded
    companies with expensive technologies whose existence would be
    threatened if the profile of Open Source alternatives became widely
    known, so they spend large amounts of PR money, e.g. advocating the
    advantages of J2EE, or in the case of another proprietary vendor,
    .NET. Or perhaps the SOAP fiasco is a better example. In all cases,
    the prolific PR and comunications machines generate huige amounts of
    "noise" in the computer industry and, so, like in a busy street
    market, the majority of decision makers miss not only the "quieter" -
    more useable, better supported tools, but also miss the point."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Finally, what do you expect from EuroPython?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""TC: A stimulating time, with pleasant Pythoneers from many
    nations. It would be my hope that this will be a starting point for
    making ever-stronger the national and cross-national relationships,
    and business, as a result of the new ideas found and acquaintances
    formed."""), style=styles["2cola"]))

    story.append(Paragraph(("""http://www.europython.org"""), style=styles["2colar"]))
    
    story.append(Paragraph(("""People who would like to find out more about the mentioned topics, have a look at:"""), style=styles["2cola"]))

    story.append(Paragraph(("""http://www.siena-tech.com/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.europython.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.python.org/"""), style=styles["2cola"]))
    
    #end of interview
    
def getThompsonInterview():
    ## Start of interview
    #Interview with Phil Thompson

    story.append(NextPageTemplate('TwoColPlus'))
    newPage()
    story.append(Paragraph('Interview of Phil Thompson<onDraw name="myOnDrawCB" label="Phil Thompson"/>', style=styles["h3"]))
    story.append(FrameBreak())
    story.append(NextPageTemplate('TwoCol'))
    
    story.append(Paragraph(("""Martijn Faassen"""), style=styles["2colar"]))

    story.append(Paragraph(("""<b>EuroPython: Phil, please introduce yourself to our readers?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Phil Thompson: I run a software development business, Riverbank Computing, that
    provides programming services to anybody who has a need. Over the past
    couple of years this has involved a lot of embedded Linux work -
    kernel porting, device driver development - plus PyQt of course."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""I've been working with UNIX in some form or other for over 20
    years. In that time I've been a university lecturer, managed
    development/support teams of 60+ people (and been out of work from
    time to time)."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""At the moment I'm focusing on redecorating the bedroom which I agreed
    to do in a moment of weakness (ie. after my wife bought me a couple of
    beers)."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: How did you get involved with Python?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""PT: I first looked at Python around 1990 to solve a particular problem,
    but I couldn't believe that a language was trying to tell me how I
    should layout my code."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""I looked at it again about 5 years ago when I wanted an alternative to
    Tcl. I was using Tcl, Tk and incrTcl/Tk but was getting increasingly
    frustrated with how ugly my applications were to the user. The final
    straw was a release of incrWidgets that included a date widget but
    with the US format hardcoded."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""However, I can't explain why I then moved to a language that also had
    Tk as the default GUI - but it all worked out in the end."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: How did you get involved with Qt?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""PT: I installed KDE v1. As I implied before, if I'm looking at a screen
    for hours at a time, it's important that I quite like what I'm looking
    at. Obviously it's all a matter of personal taste, but I like my
    desktop and it works for me. It followed that I wanted my own
    applications to fit in."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What do you like about Python?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""PT: I think the first thing that attracted me (once I'd let the language
    win and decide how I should layout my code) was the extent of the
    standard modules.  Reading the documentation I felt like a kid in a
    toy shop."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""These days I'm more interested in the C API. The API, and the way the
    internals are structured, are so clean and consistent - a joy to work
    with.  With the Python source code you know that a particular function
    should exist, you can guess what it is called, you can guess the name
    of the source file, and you can guess the sub-directory that should
    contain that file."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: In what areas could Python be improved?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""PT: My biggest concern has been resolved with the introduction of new
    style classes. The next major release of SIP (the wrapper generator
    used to create PyQt) will implement new style classes - it always
    feels like a step forward when you are able to rip out lines of code
    rather than add them."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""I've always felt that Python was much weaker than Tcl when it comes to
    version control - particularly of external modules. Tcl allows
    multiple versions of a module to be installed, and an application can
    specify which version it needs to use. About 3 years ago I promised to
    write a PEP on the subject, and one of these days I may actually find
    the time to do it."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What distinguishes Qt from other GUI toolkits?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""PT: (Puts on asbestos suit.) I haven't really looked at other toolkits for
    a couple of years - so (I expect) my views are out of date."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""From a user point of view, as I have said, I think visual appearence
    is important. If I was trying to get money out of a potential customer
    I would never, ever, ever show them a Tk (on UNIX) GUI based
    application. Gtk based applications are not pretty, but acceptable."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""From a developer point of view, taken at it's simplest, Qt is
    appealing because you write fewer lines of code to get the same job
    done. I've been involved with two projects that have allowed a direct
    comparison between a Gtk implementation and a Qt
    implementation. VeePee required about 20% less code, QScintilla (the
    port of Scintilla to Qt) requires about 40% less code."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""PyQt exploits all the advantages of Qt - plus you get the additional
    productivity of using an interpreted language. I have two favorite
    examples..."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Hans-Peter Jansen wrote a demo (included with the PyQt examples) that
    uses Qt's data aware widgets to query and update any SQL database in
    180 lines of Python."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Detlev Offenbach has developed eric (the IDE written entirely in PyQt)
    in a very short period of time, but it is fully featured including a
    debugger, syntax aware editor, links to Qt Designer, UnitTest, code
    coverage and metrics and two source code control systems."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: How did you get started writing PyQt?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""PT: It goes back to my decision to switch to Python from Tcl and wanting
    applications that wouldn't look out of place on a KDE desktop."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""At the time (in 1998) there had been an attempt to create Qt wrappers
    using SWIG. The problem was (at the time - SWIG may have changed
    since) the wrappers were incomplete and required some "unnatural"
    SWIG-imposed conventions. I quickly realised that an alternative to
    SWIG was needed that tightly integrated C++ class libraries with
    Python without imposing any conventions on the programmer. So SIP was
    born as a small SWIG, although it has grown up a bit since then."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: I heard that the Qt developers were actually skeptical
    that this level of integration could be done, could you tell me more
    about that?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""PT: I've seen the comment on their web site, and they also said the same
    about PerlQt. I think the reaction says more about their lack of
    knowledge of Python-like languages and what you can do with
    them. Personally I have no idea why people (particularly commercial
    organisations where time and code re-use is money) write applications
    in C++. Lots of organisations have significant investment in older
    libraries, but they should be turning those into Python modules using
    SIP/SWIG etc."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What was their reaction when you did it?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""PT: Confusion I think. Trolltech were kind enough to invite me to Oslo a
    couple of years ago. They have lots of very bright C++ developers. I'm
    not sure they understood why anybody would want to program using any
    other language."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What's coming up for Qt and PyQt?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""PT: I can't speak for Qt - v3.2 is currently in beta and my guess would be
    that v4 would be out sometime later this year."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""PyQt will continue to closely track Qt and Python. When a new version
    of Qt is released then PyQt is usually updated within a few days."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""The major development that is being done is to SIP. SIP v4 will
    generate wrappers that use new style classes. It will also generate a
    version of PyQt that will work on native MacOS/X. I am also taking the
    opportunity to drop support for older versions of Python and Qt -
    although SIP v3 will still be maintained for those that need it."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Your interviewer has been drooling over specifications of
    the Sharp Zaurus handheld recently, mainly because it runs Linux and
    Python. You ported Python to the Zaurus, could you tell us more about
    it?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""PT: As the Zaurus is just a Linux box, porting Python is just a case of setting
    up an approprate development environment."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""The extra work I did was minimising the memory footprint for such devices by
    looking at which of the standard modules could be removed (given that a PDA is
    more likely to be used as a client rather than a server) and how to split the
    remainder into a number of collections. Users could then choose to install only
    the subset they needed."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""To do this I did an analysis of all the inter-module dependencies. It's
    surprising how many modules are imported by the time the first line of your
    Python code gets executed. The standard site.py imports distutils and pydoc
    which themselves drag in literally dozens more. I simply changed site.py to
    gracefully cope if distutils and pydoc aren't installed."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Porting SIP and PyQt to the Zaurus was only a little bit more difficult. With
    the embedded version of Qt the hardware manufacturer can choose, at a fairly
    low level, which classes to include. I really just had to make the
    configuration process a bit more flexible to deal with non-standard sets of
    classes."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Finally, what do you expect from EuroPython?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""PT: I think the same as most other people - the opportunity to meet people
    who you have been reading for several years."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Don't forget the Belgian beer; the other people I
    interviewed mentioned that. :)</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""PT: Of course. When you are a guest in other people's countries it is only polite
    to immerse yourself fully in their culture and traditions."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""http://www.europython.org"""), style=styles["2colar"]))
    
    story.append(Paragraph(("""People who would like to find out more about the mentioned topics, have a look at:"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""http://www.riverbankcomputing.co.uk/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.riverbankcomputing.co.uk/pyqt/index.php"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.riverbankcomputing.co.uk/sip/index.php"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.die-offenbachs.de/detlev/eric3.html"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.opendocs.org/pyqt/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.kde.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.swig.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.trolltech.com/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.trolltech.com/products/qt/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.riverbankcomputing.co.uk/zaurus/index.php"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.zaurus.com/"""), style=styles["2cola"]))
    
    #end of interview
    
def getGrisbyInterview():
    ## Start of interview
    #Interview with Duncan Grisby

    story.append(NextPageTemplate('TwoColPlus'))
    newPage()
    story.append(Paragraph('Interview of Duncan Grisby<onDraw name="myOnDrawCB" label="Duncan Grisby"/>', style=styles["h3"]))
    story.append(FrameBreak())
    story.append(NextPageTemplate('TwoCol'))
    
    story.append(Paragraph(("""Martijn Faassen"""), style=styles["2colar"]))
    
    story.append(Paragraph(("""<b>EuroPython: Duncan, please introduce yourself to our readers.</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Duncan Grisby: I'm best known in the Python world for my work with CORBA. I'm the
    author of omniORBpy, one of the CORBA implementations for Python; I
    also maintain omniORB, the C++ part."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Professionally, I currently have two main things that occupy my time.
    I own a company called Apasphere which offers consulting, teaching,
    and commercial support for omniORB. I am also the co-founder of a
    startup company called Tideway Systems which is working on software to
    help large companies manage their IT infrastructure by understanding
    dependencies between components. At Tideway, we're making heavy use of
    both Python and CORBA."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Outside of work, I'm a keen photographer, my favourite subject being
    black and white figure studies. I also enjoy kickboxing, and I'm the
    assistant instructor at the local kickboxing club."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What is interesting about distributed systems?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""DG: The main thing is that they are very hard. As soon as more than one
    computer is involved in an application, there are many more things
    that can cause it to go wrong, including network partitioning, data
    loss, and concurrency. Getting it right, and building robust and
    efficient distributed applications is a challenge. I like challenges."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What first attracted you to CORBA?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""DG: In 1995, I started a PhD in Computer Science at Cambridge University,
    sponsored by Olivetti and Oracle Research Ltd (ORL). At that time, a
    group at ORL had just started developing what became VNC. Although VNC
    is great on a local area network, it isn't so good in a wider area,
    where no matter how much bandwidth you have, latency gets you. I
    started thinking about building a whole new windowing system that
    would let you move your desktop from one place to another, without
    suffering the effects of latency so much."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""At that time, CORBA was just coming to maturity as a middleware
    platform that had a classical object model, the possibility of high
    performance, the ability to interwork between programming languages.
    Even better, one of the researchers at ORL, Sai-Lai Lo, was working on
    an ultra fast and compact CORBA 2 ORB in C++, called omniORB. CORBA,
    and omniORB, was therefore the obvious choice of platform upon which
    to base my PhD work."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""I still like and use CORBA because it is still by far the best
    solution to a wide range of distributed systems problems."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What came of the windowing system you were thinking about?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""DG: I never intended to make it a complete system, just enough to
    experiment with, and prove the things I wanted to prove. I did develop
    it enough to build some simple applications, but to make it a complete
    window system would have required a huge amount of work. I thought
    about releasing it as an open source project, but I decided that it
    would cause me to get sucked into dealing with it into the future,
    which I didn't want to do. Apart from anything else, a lot of the code
    was far from robust, since it was never intended to be used for real,
    so the first step would probably have been to throw everything away
    and start again doing it properly. Mainly, I think the world's
    existing window systems are too entrenched, so trying to make a new
    one is almost certainly doomed to failure."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Anyone who is really interested can read my PhD dissertation, linked
    from http://www.grisby.org/research/"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Are you aware of Fresco (used to be called Berlin)? They
    use CORBA in the core of their windowing system. What is your opinion
    on it?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""DG: Yes, I know about Fresco. Berlin, as it was then, was just starting   
    its second complete development team and technical overhaul at the 
    time I started my PhD. I had quite a lot of discussions with the
    people working on that when I was working on my PhD, but they had a
    rather different view of how to structure their system than I did.
    Since that time, as far as I know, there has been another gradual but
    complete change of developers, and another change in direction,
    resurrecting the original Fresco system, which started out as a
    component framework on top of X."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""The other connection to Berlin / Fresco I have is that they are using  
    omniORB as their CORBA implementation, and I have implemented a couple
    of small omniORB features to help them out."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""I think Fresco has two main problems. First, they make rather too many
    CORBA calls, meaning performance is a difficult issue for them.
    Second, they have the same issue I mentioned earlier, that no matter
    how great their software ends up being, they will have a very hard
    time competing with existing window systems, X in particular."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What motivated you to start developing OmniORB?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""DG: Well, I didn't start omniORB -- Sai-Lai did. In fact, Sai-Lai started
    omniORB 2. omniORB 1 predated the GIOP standard, and only interworked
    with Orbix. Anyway, omniORB was written because at the time, ORL had
    some small custom-designed hardware devices and wanted an ORB to run
    on them. It was only once it was written that everyone realised it was
    better than the other ORBs available, and we started using it on
    mainstream platforms."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""I started working on omniORB when I encountered some things I needed
    for my PhD, which weren't yet implemented. When I finished my PhD, I
    was offered a job at ORL. By the time I started, ORL had become AT&T
    Laboratories Cambridge. Because of my previous work on omniORB, the
    natural thing for me to do was to join Sai-Lai working on omniORB. A
    while after that, I became the project leader, and I have continued in
    that role since AT&T closed the Cambridge lab last year."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: How did you first get involved with Python?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""DG: Each month, one of the projects at AT&T gave a review presentation to
    tell everyone about their progress and show some demos. In one of the
    CORBA project reviews, we built a distributed music player that
    streamed CD audio over the network via CORBA. It was controlled with
    various interesting devices we had access to, but we needed a GUI to
    finish it off. I'd been wanting to try out Python for a while, so I
    decided to make the GUI with that, using a CORBA implementation for
    Python called Fnorb."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Within about a week, I had learnt Python and built quite a complex GUI
    using PyGTK, which controlled the rest of our CORBA demo via Fnorb
    (adding threading support to PyGTK along the way). After that, I was
    hooked on Python, but I was disappointed with some aspects of Fnorb so
    I started writing omniORBpy."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: In what areas do you think Python could be improved?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""DG: Nothing significant, really. I'm happy as it is."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: There is a lot of buzz concerning SOAP and web services
    recently.  Could you compare and contrast with CORBA?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""DG: The software world is largely driven by hype. At any one time, there
    are usually one or two "hot" technologies that are hyped enormously,
    and are generally proclaimed to be the solutions to all problems. As
    part of this, the standard practice is to rubbish all other
    technologies that are remotely similar to the one being hyped, in
    order to convince the world that the technology being hyped is indeed
    the solution to all problems."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Web services and SOAP are currently occupying the hype spot. (Before
    that, the hot technology was Java, which was going to render all other
    languages obsolete. Remember that?). There is some overlap between
    SOAP and CORBA (but not as much as some people say), so CORBA is
    getting the rubbishing treatment."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""The main technical difference between SOAP and CORBA is that SOAP is
    procedural or message oriented and CORBA is object oriented. The
    distinction is exactly the same as the difference between procedural
    languages and object oriented ones. For some applications, a
    procedural style is the easiest to use; for others, you really want
    object orientation. Another significant difference is that SOAP
    messages are XML text, while CORBA uses a binary protocol. Text is
    possibly easier to deal with when debugging (although tools like
    Ethereal will decode CORBA's GIOP for you, and have you tried to read
    a SOAP message!?), but text requires much much more network bandwidth
    and processor time to handle."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""The CORBA specification also goes much further than the web services
    specifications, in that it specifies an object model, standard
    language mappings, and a wide range of standard services that
    applications can use. Web services will no doubt grow most of these
    things, but they don't exist yet."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""The thing I find really sad about the whole web services push is that
    when CORBA was being designed, the people writing the specifications
    inevitably made some mistakes. Most of the mistakes were fixed, but a
    few remain, largely to keep backwards compatibility. The web services
    people could learn an enormous amount from the mistakes made with
    CORBA (and the successes of course), but on the whole they are
    ignoring everything that has gone before, and are making all the same
    mistakes (and more) again."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""To close what turned into a rather long answer, I'm a strong believer
    that people should choose the right technology for the job in hand.
    For some problems, the right technology is web services; for others it
    is CORBA; for yet others it is something else."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Recently REST (REpresentational State Transfer) has been
    put out as a contrast against remote procedure call style mechanisms
    like SOAP and CORBA. What is your opinion about a design philosophy
    like this, compared to an RPC style system?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""DG: REST fits nicely in the "something else" category I just mentioned.     
    It is definitely a good solution for some kinds of problem. It is,
    however, most clearly not a very good solution for other kinds of
    problem. The REST articles I have read tend to fall into the trap of
    trying to convince the reader that REST is the solution to all
    distributed computing problems, which clearly it isn't. Having said
    that, I certainly think that some of the things REST proposes are a  
    far more sensible than some of the most complex parts of the SOAP and   
    related specifications."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Finally, what do you expect from EuroPython?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""DG: Mainly, I hope to have a really great time like I did last year. It's
    always fun to meet interesting people from the Python community. The
    Belgian beer is good too..."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""http://www.europython.org"""), style=styles["2colar"]))
    
    story.append(Paragraph(("""People who would like to find out more about the mentioned topics, have a look at:"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""http://omniorb.sourceforge.net/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.grisby.org/research/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.grisby.org/presentations"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.realvnc.com/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.apasphere.com/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.tideway.com/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.fresco.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.w3.org/TR/SOAP/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.w3.org/2002/ws/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://internet.conveyor.com/RESTwiki/moin.cgi/FrontPage"""), style=styles["2cola"]))
    
    #end of interview
    
def getChauvatInterview():
    ## Start of interview
    #Interview with Nicolas Chauvat

    story.append(NextPageTemplate('TwoColPlus'))
    newPage()
    story.append(Paragraph('Interview of Nicolas Chauvat<onDraw name="myOnDrawCB" label="Nicolas Chauvat"/>', style=styles["h3"]))
    story.append(FrameBreak())
    story.append(NextPageTemplate('TwoCol'))
    
    story.append(Paragraph(("""Tom Deprez"""), style=styles["2colar"]))

    story.append(Paragraph(("""<b>EuroPython: Nicolas, can you introduce yourself to our
    readers?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>Nicolas Chauvat</b>: I am one of those kids that started fooling around
    with computers before they turned twelve. It was a lot of fun and a
    great opportunity to learn tons of things, including english. Today I
    regret not having the internet at that time, since that would have
    made learning much faster."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""After completing scientific studies I spent some time in a couple
    AI research labs and R&D companies. I eventually founded Logilab with
    a few friends in 2000. I am also the secretary of the Python Business
    Forum, that Logilab co-founded with AB Strakt and ReportLab last
    year."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Logilab was founded almost 4 years ago. Can you look back
    satisfied that you made the right decision at that time? Which things
    did you encounter that you didn't expect? Both positive and
    negative.</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>NC</b>: Starting your own company is a lot of work, but it's like
    computers: a great opportunity to learn tons of things."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""I'm very happy with the decision I made at that time. We keep
    growing, have a great working environment with talented people around
    and we are all making a living doing things we like. What's better
    than that ? (I know people at AB Strakt whom will agree with me
    here...)"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: As last year, you are the chairman of
    the Python Science & Industry track. What can we expect this year and
    did you learn something from last year?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>NC</b>: Python is very strong in the scientific community and keeps
    gaining momentum. Last year we had several good talks and proved that
    a lot of disciplines are using Python for programming, ranging from
    astrophysics to biology."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""The adoption of Python is not really surprising when you know that
    researchers in those fields often need to develop their own tools to
    process the data they gather in order to model complex phenomenons or
    prove theories. The ease with which Python can wrap existing C/C++ and
    Fortran code is also a big win. When you add simple GUIs building and
    database connections, very few can resist."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""This year the scientific track will last for a full day and present
    twelve different talks, ranging from simulation to large computation
    and including the processing of geographic and astrophysics data.
    Slides are already available online for people to read, comment and
    think about. Authors will welcome feedback."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""I wish this yearly event to be the kind of forum that any scientist
    needing to do some data processing by himself will turn to in order to
    share experiences and ideas."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Can you tell us more about Logilab?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>NC</b>: Logilab is a european company located in Paris, focused on
    advanced computing and knowledge management. We specialize in Python
    and provide services ranging from custom development to high-level training 
    mixing Python, C++ and scientific computing. We also have been developing
    a line of products that is about to reach the market and includes a
    natural language processing server and a generic knowledge management
    and querying server. Our customers so far have been mostly industries
    and research entities, but we started reaching more public administrations."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Logilab is also convinced that "Logiciel Libre" is the way to go
    for many reasons. We are promoting free (as in speach) software and
    publishing a lot of it under the GPL licence on logilab.org. Of
    course, that does not prevent us from providing our customers with
    proprietary developments, nor from selling some of our software under
    proprietary conditions."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Why do you promote free software?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>NC</b>: we promote "Logiciel Libre" because we think that
    IT firms should be suppliers of services, not the owners of
    the IT system of their clients. Nowadays, most companies would
    not be in business without a solid IT infrastructure. If your
    company uses non-free software for part of something vital, it
    means that a problem with that software threatens the existence
    of the company. You would think twice about driving abroad for the holidays
    if you knew that only a single garage in Europe is able to fix your car if it breaks.
    And you would accept not being allowed to changes tires yourself? Or not
    allowed to look under the hood yourself nor ask someone else to do it for
    you... even if the garage that sold you the car closed shop?"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""That's even more true in the case of public administrations that are
    responsible for ensuring the perenity of the information that
    concerns the state and the citizens. Any company can die and disappear. Even one large
    enough to provide power to part of a continent. Free software is
    about putting back the user/client in control of something he needs
    to be able to rely on and build on. We are writing and *selling*
    free software. If our clients are unhappy with us or if we disappear,
    they can ask any other service company to resume development
    or maintenance. They do not belong to us..."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Oh, and I'll take the opportunity to mention that I am opposed to
    software patents (but not necessarily to patents). Patents make sense to encourage
    invention by granting inventors a monopol limited in time. That monopol is supposed to allow
    them to get ahead of the competition and benefit from their research investments.
    The project of law currently discussed
    in Brussels is about restricting everyone's freedom to use a computer
    and forbidding the reuse of *ideas*, not forbidding for a couple years
    the copy of some invention. Moreover, computer programs are already
    protected by the legislation that concerns authors' rights. Please read more on the topic at
    http://www.eurolinux.org/"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Can you elaborate on your soon to be launched line
    of products?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>NC</b>: We have been developing software components that provide
    natural language processing functionnalities. You can enter a sentence
    and have the computer make some sense out of it, then come up with an
    answer. I'm sure you'd want that kind a web guide for the
    europython.org web site."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""That's closely related to the Narval project we created the company
    for. Narval is an agent framework. It's available under the GPL
    licence on our website. Unfortunately, we had to put development on
    hold in order to finish the other projects first."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""The knowledge management software I was mentionning makes it easy
    to maintain a knowledge base. I'm using knowledge in a broad sense.
    For example, we're using it to manage our business contacts, computer
    hardware, etc. You enter a schema and both the web interface and the
    storage are handled for you. You can then browse through your data,
    edit it and tie entities with relationships, even if they are
    aggregated from different sources. A big plus is that you can query
    the information you stored using a high level language, much closer to
    your model than SQL. I'd say it's an example of what the semantic web
    portals are supposed to provide. Last but not least, you can query
    your knowledge base using natural language, so that typing "who works
    for Logilab?"  will get you a list of persons."""), style=styles["2cola"]))
    
    
    story.append(Paragraph(("""<b>EuroPython: What do you use Python for?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>NC</b>: We do everything in Python. If the result is too slow, we move
    part of the code to C/C++."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: This means that you develop everything first in
    Python and only later decide to move some code parts to C/C++? You
    never try to find the bottlenecks in advance?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>NC</b>: You can *not* find bottlenecks in advance. "Early optimisation
    is the root of all evils", etc."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What are your criteria to decide that a part needs
    to be optimized? Do you follow a certain canvas?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>NC</b>: I suppose we're doing it the standard way... if it is too slow,
    we profile it, pick what's taking the most time and try to make it run
    faster. When that's done, if it's still too slow, we loop."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: When did you hear for the first time about
    Python?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>NC</b>: I first heard about Python in 1995, but only used it a year
    later, when I was involved in a web agency and wanted a replacement
    for C-based CGIs to internationalize our web site. I had tried Perl a
    week before and never ever used it again :-)"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: On which Python projects are you working at the
    moment?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>NC</b>: Logilab published two new tools this week : pylint and
    aspects."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Pylint checks python code and syntax, looking for errors. I am a
    code quality maniac and try to get our whole development team to reach
    the highest possible quality. That means a lot of automated testing,
    the use of code metrics, setting up a strongly structured development
    environment, etc. A private company can not survive and publish a lot
    of free software without producing it very efficiently. Some people
    already know about our pyreverse tool. Both will probably end up being
    integrated into the same library. We also need to get closer to
    pychecker's authors, since pylint duplicates its functionnalities."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Aspects is a module to implement Aspect-Oriented Programming in
    Python. We also use it for Eiffel-style contracts."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Recently, we started merging some of our knowledge management
    software with Plone's Archetypes. My opinion is that Plone is the
    future of Zope and Archetypes the future of Plone ;-). We look forward
    to the next version of Plone that may include Archetypes, but we
    decided that the best parts of that KM software will stay proprietary
    for some time, though. Our next step will be the implementation of
    semantic web features with Plone, so stay tuned."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Why didn't you contacted the PyChecker authors
    before creating PyLint? This would perhaps avoided the work of
    creating a tool from the ground up?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>NC</b>: Well, we already had most of what we needed, thanks to the
    standard library and pyreverse. And in some cases, creating a tool
    from the ground up and merging later with similar projects is more
    efficient than trying to force cooperation from the very
    beginning. Pychecker works and has no reason to change radically just
    because some punk comes in with new ideas. If you prove your ideas are
    worth a change and cooperation would be fruitful, that's a different
    matter."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What is Aspect-Oriented Programming?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>NC</b>: Some would say "separating orthogonal concerns". I'd say
    "dynamic call-path rewriting". See http://www.aosd.net"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: So you use Eiffel, next to Python?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>NC</b>: No, we don't use Eiffel nor any of the twenty-or-so other
    languages we've looked at or played with. But that does not prevent us
    from trying to import great things that others have implemented. Like
    contracts in Eiffel."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Can you tell something more about contracts in
    Eiffel. What's nice about contracts?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>NC</b>: Some people complain that "python is not strongly typed", which
    is a bad way putting it. Python does not bother you with method
    signatures or type checking during the compilation stage. The direct
    benefit is that it does not get in your way by forcing you to think
    about such checks too early. In a sense, signature checking is like
    optimisation, you should not try to do it too early. But when the APIs
    stabilize and the time comes to make the code more robust, simply
    checking for the types of the arguments appears like very weak. What
    about intervals for integers, numbers of elements for lists, etc."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Contracts are meant to address three issues :<br/>
    - provide really strong checking<br/>
    - document interfaces' requirements and promises<br/>
    - get rid of defensive programming techniques<br/>
    """), style=styles["2cola"]))
    
    story.append(Paragraph(("""If your function takes an argument named num, contracts will let
    you make explicit that num is an integer above 0 and below 12 for
    example. When contracts are activated, if the function is called with
    13 as an argument, an exception ContractFailed will be raised."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Requirements on the arguments coming in and promises on the results
    going out are considered part of the documentation."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Since the contract requires the arguments to be correct, the body
    of the function can focus on processing the data and does not have to
    start by checking that it is correct.  This helps getting rid of
    "defensive programming" habits."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Contracts can be enabled or disabled at will. During development,
    you can enable contracts and have calls made with bad arguments raise
    an exception, then fix the problem.  When everything works, you may
    disable contracts to get more speed."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""If someone tells you that Python is missing so-called strong-typing,
    just answer that it provides better than that: contracts."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: How do you use Eiffel contracts in Python? Can you give our
    readers a quick start?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>NC</b>: Let's try a example of code. pre, post and inv stand for
    pre-conditions, post-conditions and invariants."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<pre>
    from logilab.aspects.weaver import weaver
    from logilab.aspects.lib.contracts import ContractAspect, ContractFailedError
    
    class Account :
        '''Account can not hold a negative amount.
    
        inv:
            self.amount > 0
        '''
    
        def __init__(self) :
            self.amount = 0
    
        def credit(self,amount) :
            '''can not credit a negative amount
           
            pre:
                amount > 0
            '''
            self.amount += amount
    
        def debit(self,amount) :
            '''can not debit a negative amount
    
            pre:
                amount > 0
            '''
            self.amount -= amount
    
    if __name__ == '__main__' :
        # enable contracts on Account class
        weaver.weave_methods(Account, ContractAspect)
    
        account = Account()
        account.credit(100)
        try:
            account.debit(-200)
        except ContractFailedError :
            print "Can't debit a negative amount"
        try:
            account.debit(200)
        except ContractFailedError :
            print "Can't debit more than what's available"
    
    
    </pre>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""For more information, download the package from logilab.org, read the
    documentation and look at the examples."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What did you decide that Plone is the future of
    Zope?  There are many other CMS in Zope. Are you following the Zope 3
    development?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>NC</b>: I don't think I can or should "decide" anything about Zope or
    Plone.  My personal opinion is that Plone provides what Zope/CMF was
    actually after. And since having "one obvious way to do it" would be
    nice...  I don't have time to follow Zope 3 development closely but
    wish the other Zope based CMSes and Plone could end up being a large
    set of components for Zope 3. Free software is also about reusability,
    isn't it?"""), style=styles["2cola"]))
    
    
    story.append(Paragraph(("""<b>EuroPython: What do you like best about Python?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>NC</b>: Two things."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""First is that Python does not enforce a design style. It's
    multiparadigm by nature. Want to write a ten lines script? Use
    Python. Need objects? Use Python. Want to mix objects and procedures?
    Use Python. Want part of your application in C and the rest in a
    high-level language that provides first class types like dates, lists
    and mappings? Use Python. Want more power, like introspection, very
    dynamic behavior and on-the-fly modification of the code? Use
    Python. Want contracts, aspects, logic programming?  Use Python
    libraries from Logilab ;-)"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Second is that if something you need is not in *the* standard
    library, it is probably in *a* standard library. Numeric, PIL,
    Reportlab, PyXML, twisted et al. make a real difference when compared
    to other languages. That makes me call Python an agile development
    framework rather than a programming language."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Oh, and I was about to forgot about the syntax, which is so clean
    that I have had a very hard time using anything else but Python to
    program for the past seven years."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What do you not like about Python?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>NC</b>: I'll like Python even better when the Python-in-Python
    interpreter will be available for everyone to toy with home-grown
    dialects and language changes or optimisations. Python needs to be
    stable and PyPy will be a great tool for experimentation. CPython is
    harder to tweak."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""I also wish we could include some of the Mozart/Oz interpreter and
    language features, namely concurrency and distribution, but I doubt
    Python will ever want to go down that road. A recent thread on the
    mozart list mentionned that the concept of future/promise could
    benefit Python. It first started on another list when discussing the
    implementation of logic variables with stackless. I just wish I had
    more time to spend on deeply understanding both Python's and Mozart's
    interpreters... and be able to explore the issue myself."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What is the next 'big' issue Python should have,
    according to you?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>NC</b>: I think that the standard libraries (Numeric, PIL, etc.) are the
    places where the real improvements should be expected. Python is
    already quite mature as it is. Moreover, Python 2.3 added what I
    missed most, the datetime module and the set type. That's why I
    think the next big move concerning Python itself will be
    Python-in-Python. Go sprinters!"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Are you interested in joining the sprint efforts of
    PyPy?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>NC</b>: Unfortunately, I currently do not have the time, nor the
    energy."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""As I said, we are already involved in and contributing to many different
    free software projects. Even if we're using agile methodologies like
    eXtreme Programming and a very structured development environment that allow
    us to produce software efficiently, we can not take part in all the
    projects we would like to see succeed."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Finally, what do you expect from EuroPython?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>NC</b>: A lot of goods talks, ideas and code to come back home with and
    think about or use. I'm confident this year's conference will
    fulfill my expectations, as did EuroPython 2002. See you there :-)"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""http://www.europython.org"""), style=styles["2colar"]))
    story.append(Paragraph(("""People who would like to find out more about the mentioned topics, have a look at:"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""http://www.logilab.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.python-in-business.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://aosd.net/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://codespeak.net/pypy/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://pychecker.sourceforge.net/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.mozart-oz.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.plone.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.python.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.zope.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://docs.eiffel.com/general/guided_tour/"""), style=styles["2cola"]))
          
    #end of interview
    
def getEverittInterview():
    ## Start of interview
    #Interview with Paul Everitt

    story.append(NextPageTemplate('TwoColPlus'))
    newPage()
    story.append(Paragraph('Interview of Paul Everitt<onDraw name="myOnDrawCB" label="Paul Everitt"/>', style=styles["h3"]))
    story.append(FrameBreak())
    story.append(NextPageTemplate('TwoCol'))
    
    story.append(Paragraph(("""Tom Deprez"""), style=styles["2colar"]))
    
    story.append(Paragraph(("""<b>EuroPython: I guess most people will know you from the previous
    edition, but for the new, I think an introduction would be in it's
    place. Can you introduce yourself Paul?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Paul: I'm Paul Everitt, director of the Zope Europe Association,
    co-founder of Zope Corporation.  I've been involved in Python a long
    time (I have the t-shirt from the first Python conference at NIST)."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""I program but I'm not a good programmer, so in Python and Zope, I'm
    always the dumbest guy in the room.  But in a way that's good, as
    there are things developers don't like to do for which I'm interested.
    Thus I've tried instead to participate through organization,
    communication, community-building, etc."""), style=styles["2cola"]))
    
    
    story.append(Paragraph(("""<b>EuroPython: Lot has changed in your life since last year, can you tell
    us about these changes?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Paul: It's been a lot of fun.  Last year at this time I was still at
    Zope Corp, living in Virginia with my family.  Now we're living in
    Rennes, to the west of Paris.  I'm no longer in management at a
    medium-sized company.  Instead, I travel around, working with small
    companies and focusing on the important things in the "commons" part
    of Zope."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What do you exactly mean with the "commons" part?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Paul: The part of the Zope world that is shared by all, both business
    and community.  Open source is about scratching your own itch.  But to
    make progress on growing Zope, there are common tasks (organizing,
    communication) that must be done.  Also known as "the dirty work". :^)"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: When did you hear about EuroPython?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Paul: Last year on the EuroZope mailing list."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What was your first thought concerning such an initiative?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Paul: I thought it was perfect timing.  The original IPC Python
    conferences were coming unravelled.  This conference proved the idea
    that a low-cost conference could deliver most of what Python folks
    wanted."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Last year, you organised the Zope Track together with Tom
    Deprez, this year together with Heimo Laukkanen. What can we expect
    from the track this year and what did you learn from last year?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Paul: Though my name is on the Zope Track this year, it's really Heimo
    that did all the work.  Heimo is awesome.  He has put together a
    really, really interesting track, with several topics that I'm very
    eager to learn more about.  And it is diverse, covering technology and
    business."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Last year's conference seemed like such a perfect event, with the
    conversations and real pleasure people had talking to each other, that
    I really hope this year can be the same.  I've learned a lot more,
    through organizing OSCOM (http://www.oscom.org/) events,
    about conferences since Charleroi last year.  The biggest lesson is to
    not over-organize.  Leave time and space for people to hang out and
    network."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: What made you decide in helping on the OSCOM conference and
    what was you involvement?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(('''Paul: I was invited to speak at the first OSCOM in March of last year.
    Wow, Zurich is a gorgeous place.  The conference had this neat vibe
    to it, and the part I enjoyed the most was hanging out with the
    "competitors".  During their talks I found myself thinking, "Hmm, they
    do such-and-such better than Zope does, we should learn from them."'''), style=styles["2cola"]))
    
    story.append(Paragraph(("""Afterwards I volunteered to help organize OSCOM 2 and thus wound up on
    the board of directors.  My biggest contribution is to write articles
    with Gregor that get posted on the front of Slashdot one week before
    the conferences. :^) This year I taught a half-day tutorial on
    "Building CMS Clients With Mozilla"."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: You are the founder of Zope Europe Association (ZEA). Was
    there a big need for such an organisation? How did you initiate this?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Paul: The ZEA's mission is to grow Zope business in Europe.  The story
    goes like this.  Do the Zope companies in Europe need Zope to grow?
    Yup.  Are they individually responsible for Zope's growth.  Nope, few
    are big enough to fully fund a marketing person or campaign.  But can
    we wait around and hope for the best?  Nope, that won't work either."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Thus, I'm like a shared marketing expense, trying to do the publicity
    and communications for the common Zope market.  And we know how much
    most developers enjoy working on marketing. :^) I'm not a competitor
    in the Zope market: Zope Europe is a non-profit and I'm not part of ZC
    (though I still have my stock)."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""At ZC, we talked a lot about the growth of Zope in Europe.  It wasn't
    really feasible, nor perhaps wise, to open an office in Europe.  They
    proposed instead a foundation involving several companies, where I'd
    move to Europe and be the director from outside of ZC.  I was also
    thinking the same thing, so we quickly worked out a deal.  Thus I'm
    now an American abroad."""), style=styles["2cola"]))
    
    
    story.append(Paragraph(("""<b>EuroPython: Did you know of the EuroZope Foundation and did you or
    they contact each other?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""The original plan was to try and work out of the existing EuroZope eV.
    However, its purpose, charter, legal form, etc. were all geared
    towards something quite different, more like a user group than an
    business with an employee.  After the Rotterdam sprint we found out
    from a German lawyer that it wasn't realistic."""), style=styles["2cola"]))
    
    
    story.append(Paragraph(("""<b>EuroPython: So basically only companies can become member of the ZEA?
    What do people have to do to become a member?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Paul: I'm a big believer in trying to focus your mission and not be
    all things to all people.  Conversely, this means being clear about
    what you *won't* do, which is where lots of people fail and thus
    dilute the message into corporatespeak."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Zope Europe is about growing the business of Zope.  It isn't a user
    group.  However, there is a huge need for user groups in each country,
    and it's part of my job to support these efforts.  Increasingly I
    think there's a role for a common, top-level user group that speaks
    for the Zope community.  If anybody's interested in such a thing,
    lemme know."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: How big is ZEA at the moment? I.e. how many employees does  
    the ZEA count at the moment?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Paul: It's the perfect size: small. :^) There are five sponsors (ZC,
    BlueDynamics in Austria, Euralba in France, ZopeMag in Germany, and
    Reflab in Italy) plus two members (Ingeniweb and Pilot Systems).
    These are the companies that have really supported the mission of
    growing Zope business in Europe.  Once I finish the first round of
    deliverables, and thus have proof of the value, I plan to ask more
    companies to participate."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""ZEA has one employee, me.  I don't expect ZEA to get big.  Rather, it
    is Zope Europe's job to make the participating companies big."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: How does Zope Corporation look at this association?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""ZC is the original ZEA sponsor, and along with BlueDynamics, they are
    the ones that got things in motion last summer.  ZC wants Zope to
    grow, but they shouldn't be burdened with 100% of that responsibility.
    It is important for the community to shoulder some of the effort for
    market growth.  Thus, Zope Europe isn't some permanent charity that ZC
    has to provide a blank check for."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""All in all, it's a healthy situation and evidence that Zope is growing
    into adulthood."""), style=styles["2cola"]))
    
    
    story.append(Paragraph(("""<b>EuroPython: If you look back, are you at the moment in the position
    you thought would have been?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Paul: On the big picture, wow, no way.  The world of Zope is an
    amazing privilege to be a part of.  Each time I see a Zope book in a
    bookstore, or see Zope mentioned in a mainstream magazine, or sit
    beside a stranger at a dinner in France and find out they've heard of
    Zope, I still get amazed.  5 years ago, Rob, Jim, Hadar, and I put
    together a company that started this ball rolling.  It's a gratifying
    feeling, but also a deep sense of dept and responsibility to the world
    of Zope."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""On the little picture, no, I think I'm 3-6 months behind where I
    thought I'd be on getting Zope Europe stuff done.  Matriculation in
    France was more work than I thought, plus I signed up for a horrific
    travel schedule.  For those that went to the Zope track last year, I'm
    50/50 on needing to wear the "I Suck" shirt. :^)"""), style=styles["2cola"]))
    
    
    story.append(Paragraph(("""<b>EuroPython: What's your relationship with Zope?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Paul: The Zope/Business/Europe trifecta is where I spend most of my
    time these days.  I also try to spend time on the community.  Probably
    my biggest responsibility is to attract attention, then point that
    attention at important and worthy companies and projects in the world
    of Zope."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""For instance, Giuseppe Masili is an organizer of Zope Italia, a Zope
    user group in Italy.  There are probably people in Italy that don't
    know about what he's doing.  But maybe they'll read this and contact
    him."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Thus, in Zope I try to connect things together.  There's so much cool
    stuff and excitement going on that, obviously, I have a very fun job."""), style=styles["2cola"]))
    
    
    story.append(Paragraph(("""<b>EuroPython: In which projects are you involved at the moment?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Paul: Organizing my mammoth pile of travel expenses. :^) I just
    finished a period with lots of travel, plus helping organize two
    conferences (OSCOM 3 and EPC2003) that were a month apart."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""I'm now getting my sights set on the real Zope Europe stuff I need to
    do: producing quality marketing material for decision makers,
    inceasing my speaking/interview schedule, and formally starting the
    discussion about a business network.  If anybody has suggestions about
    what should be on ZEA's radar, come grab me in Charleroi and let's
    talk."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""I'm also in deep with the Plone community, again, not in a code sense.
    Plone is to Zope what Zope is to Python.  The Plone folks really have
    gathered up a diverse, active community, one which is in some ways
    bigger than the Zope community."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Finally, I spend a lot of time working with OSCOM.  I'm trying my hand
    at a small software project under the OSCOM umbrella.  Since I
    complain a lot about developers, perhaps I'll gain a little sympathy
    and perspective. ;^)"""), style=styles["2cola"]))
    
    
    story.append(Paragraph(("""<b>EuroPython: What do you like best about Zope?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Paul: The people.  I don't think an outsider should judge an open
    source project based on the software, as software changes.  Instead,
    look at the people, how well they work with each other, how strong
    they feel about the project, the quality of their ideas, etc."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""The world of Zope is, like the world of Python from whence it came, a
    wonderful place to hang out.  Someone from the PostNuke community sent
    me a wild anecdote about how mean people can be in other communities."""), style=styles["2cola"]))
    
    
    story.append(Paragraph(("""<b>EuroPython: What do you not like about Zope?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Paul: I think Zope needs to try harder to play nice with others.
    Instead of inventing our own answers to some things, sort of a NIH, we
    should make it a higher priority to leverage existing work."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Through OSCOM I spend a lot of quality time listening to the non-Zope
    universe.  You can learn a lot by listening, about strategic things
    like barriers to adoption."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Some folks have heard me talk about "10x", the idea that Zope should
    grow market share tenfold in the next year.  It seems that, of that
    10x, the 9x ahead of us think differently than the 1x we already have.
    We have to work harder to care about 9x, if 10x is a high priority."""), style=styles["2cola"]))
    
    
    story.append(Paragraph(("""<b>EuroPython: Do you think Zope 3 will keep the Zope users of today? Or
    do you think some might search for another tool because of the changes
    in Zope 3?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Paul: Very good question.  Whenever you start over on a mature system,
    you introduce a disruption.  I think that Zope 3 is shaping up,
    though, to make component developers very very happy. What's critical
    is that something like Plone come along, atop Zope, that covers the
    "site developer" audience."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""If you're a Python guy and Zope 2 didn't float your boat, give the
    Zope 3 developer tutorial a try.  The tutorial shows that you can write all your
    Python code without introducing any Zope stuff into it.  The Zope
    stuff is asserted outside your code, in configuration and interface
    files."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Another important, but less obvious point, about Zope 3: it is
    developed by the community.  Two years ago, ZC was the only ones with
    CVS access.  Now, the majority of people and checkins are outside ZC.
    This will have both obvious and subtle impacts, and marks the growing
    maturity of the world of Zope."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Thus, Jim Fulton has pulled a double miracle on Zope 3: he conjured up
    a better component architecture, plus he birthed a new culture in the
    process.  Jim should be quite proud of both."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""At a minimum, Zope 3 will fix the mistake we made in Zope 2, where we
    aimed at too wide an audience."""), style=styles["2cola"]))
    
    
    story.append(Paragraph(("""<b>EuroPython: Have you ever looked at Twisted, Siena Web Services
    Architecture, CherryPy, ...? How do you compare all these web  
    application tools?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Paul: Unfortunately no, though since I'm not heavy into development, I
    wouldn't really appreciate what I was seeing. :^) However, I do talk
    to all of the people, and have a lot of appreciation for their
    efforts.  Especially Twisted.  It seems like Zope has done a dance
    with Twisted but still hasn't asked it out on a date."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""I'd like to see the various projects enjoy each other's success more.
    Some stuff has been written about Zope that, IMO, crossed the line
    between healthy criticism and mean-spirited rhetoric.  The latter
    doesn't help us learn from each other and doesn't really fit the Python culture.  Plus,
    ZC has done a lot for Python, with some aspects being quite
    measurable."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Actually, through OSCOM, I spend more time these days working with,
    and chatting with, content management system projects that aren't Zope
    or Python."""), style=styles["2cola"]))
    
    
    story.append(Paragraph(("""<b>EuroPython: Have you at OSCOM seen some applications from which Zope
      could learn a lot?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Paul: Very much so.  For instance, other projects are in their second
    generation of XML support in the core.  It doesn't appear that Zope 3
    will have XML in the core, except for ZCML.  I also think that the
    pipeline approach to template processing (Cocoon and AxKit, as well as
    Sean McGrath's Propylon) is a nice model."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""In reverse, I think that WebDAV is glue for content management, and
    I'm working with the other OSCOM "Band of Brothers" to improve DAV
    support.  Clearly Zope is far, far ahead on this front."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Does that gives you a feeling of uncertainty about Zope's
    future?  Will you be able to achieve your goal if it is missing some
    features which competitors do have?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Paul: I sort of see it in reverse: there's something that needs to get
    done and we need to find a way to do it.  Fortunately there are people
    interested in working on such a thing."""), style=styles["2cola"]))
    
    
    story.append(Paragraph(("""<b>EuroPython: Why the emphasis on OSCOM?  Isn't that helping the
    competitors?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Paul: Good question.  I don't think code alone is the primary factor
    for Zope's future growth.  In my opinion, there are factors at the
    macro-level: alliances, interoperability, and telling a story that the
    mainstream is ready to hear."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""I think that open source content management must work together to
    educate the market about the key selling points.  None of us can
    tackle that job alone.  We need to validate open source as a
    mainstream option in content management."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Finally, there are two practical points.  First, I think we owe it to
    our customers to make ourselves replaceable.  Open source should
    actively fight, not passively promote, vendor lockin.  Second, it's
    clear that OSCOM is a hot story that mainstream journalists want to
    write about, and goodness knows we aren't doing enough regular
    publicity."""), style=styles["2cola"]))
    
    story.append(Paragraph(("""<b>EuroPython: Finally, what do you expect from EuroPython?</b>"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""Paul: Chimay, the official beer of Python.  Also, I want to leave with
    feedback from Zope companies about what Zope Europe should do in 2003."""), style=styles["2cola"]))
    
    
    
    story.append(Paragraph(("""Interested readers can find more info at:"""), style=styles["2cola"]))
    
    
    
    story.append(Paragraph(("""http://www.zope-europe.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.europython.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.zope.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.python.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.twistedmatrix.com/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.siena-tech.com/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.cherrypy.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.bluedynamics.com/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.oscom.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://cocoon.apache.org/2.0/index.html"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://axkit.org/"""), style=styles["2cola"]))
    story.append(Paragraph(("""http://www.propylon.com/"""), style=styles["2cola"]))
    
    story.append(Paragraph(("""http://www.europython.org"""), style=styles["2colar"]))
    
    #end of interview

    
def getGeneralInfo():
    story.append(NextPageTemplate('AdPage'))
    #story.append(NextPageTemplate('OneCol'))

    newPage()
    story.append(Paragraph('General Information<onDraw name="myOnDrawCB" label="General Information"/>', style=styles["h2"]))

    story.append(Paragraph(("City Information Office"""), style=styles["h4b"]))

    story.append(Paragraph("""It's in front of the main railway station of Charleroi. They speak English
so don't be afraid. They will be pleased to help you, it's their job.""", style=styles["bbody"]))

    story.append(Spacer(height=6, width=10))

    story.append(Table(
        [
            [Paragraph(("<b>Telephone :</b>"), style=styles["bbody"]),
             Paragraph(("32(0)71 31.82.18"), style=styles["bbody"]),
             ""],
            [Paragraph(("<b>Open Time</b>"), style=styles["bbody"]),
             Paragraph(("9h until 17h"), style=styles["bbody"]),
             ""],
            ],
                 colWidths=[1*inch,1.5*inch,3.75*inch],
                 rowHeights=[12,12],
                 style=None,
                 splitByRow=1,
                 repeatRows=0,
                 repeatCols=0)
                 )
    story.append(Paragraph("""(If they don't answer, here is another phone number where you
    could get some info about the town:""", style=styles["bbody"]))

    story.append(Spacer(height=6, width=10))

    story.append(Table(
        [
            [Paragraph(("<b>Telephone :</b>"), style=styles["bbody"]),
             Paragraph(("32(0)71 86.61.52 "), style=styles["bbody"]),
             ""],
            ],
                 colWidths=[1*inch,1.5*inch,3.75*inch],
                 rowHeights=[12],
                 style=None,
                 splitByRow=1,
                 repeatRows=0,
                 repeatCols=0)
                 )

    story.append(Spacer(height=6, width=10))
    story.append(Paragraph(("Phone Numbers"""), style=styles["h4b"]))

    story.append(Paragraph("""If you use your own international phone you have to
    compose <b>32 71</b> in the beginning. If you use a public phone in Charleroi you
    have to start with <b>071</b>. """, style=styles["bbody"]))

    story.append(Spacer(height=6, width=10))
    story.append(Paragraph(("Important Phone Numbers"""), style=styles["h4b"]))

    story.append(Paragraph("""<b>European Emergency Help:</b> 112""", style=styles["bbody"]))

    story.append(Paragraph("""<b>Medical Emergency:</b> 100""", style=styles["bbody"]))

    story.append(Paragraph("""<b>FireDepartment:</b> 100""", style=styles["bbody"]))

def getContactsPage():
    story.append(NextPageTemplate('AdPage'))
    #story.append(NextPageTemplate('OneCol'))

    newPage()
    story.append(Paragraph('Contact Information<onDraw name="myOnDrawCB" label="Contact Information"/>', style=styles["h2"]))

    story.append(Paragraph(("""If you need to contact local people at Charleroi concerning
information about the conference, sponsership, hiring a booth, etc. :"""), style=styles["bNormal"]))

    story.append(Paragraph(("General means:"""), style=styles["h4b"]))

    story.append(Table(
        [
            [Paragraph(("Telephone :"), style=styles["bbody"]),
             Paragraph(("+32(0)71/27.03.11"), style=styles["bbody"]),
             ""],
            [Paragraph(("Fax :"), style=styles["bbody"]),
             Paragraph(("+32(0)71/31.67.35"), style=styles["bbody"]),
             ""],
            ],
                 colWidths=[1*inch,1.5*inch,3.75*inch],
                 rowHeights=[12,12],
                 style=None,
                 splitByRow=1,
                 repeatRows=0,
                 repeatCols=0)
                 )
    story.append(Paragraph("""(Let them know that you are calling
    concerning the EuroPython congress and they will know whom to
    contact)""", style=styles["bbody"]))

    story.append(Paragraph("Electronic Mail: ", style=styles["h4b"]))
    story.append(Paragraph("""You can contact the organizers directly
by mail. We will do our best to answer all your questions and if
possible pass you to the right person for your case. """, style=styles["bbody"]))

    story.append(Paragraph("You can reach the organisers through the following email address:", style=styles["bbody"]))
    story.append(Paragraph("europython@p3b.org", style=styles["bNormal"]))

    story.append(Paragraph("Address of the Organization Team", style=styles["bbody"]))
    story.append(Paragraph("EuroPython Conference Team", style=styles["bNormal"]))
    story.append(Paragraph("P3B c/o Aragne", style=styles["bNormal"]))
    story.append(Paragraph("Boulevard Gnral Michel 1E", style=styles["bNormal"]))
    story.append(Paragraph("B-6000 Charleroi.", style=styles["bNormal"]))
    story.append(NextPageTemplate('OneCol'))
    
def getCredits():
    story.append(NextPageTemplate('AdPage'))
    #story.append(NextPageTemplate('OneCol'))
    newPage()
    story.append(Paragraph('Credits<onDraw name="myOnDrawCB" label="Credits"/>', style=styles["h2"]))

    t = Table(
        [
            [Paragraph(("<b>Conference Chairman</b>"), style=styles["bbodyr"]),
             Paragraph(("Tom Deprez"), style=styles["bbody"])],

            [Paragraph(("<b>Local Chairman</b>"), style=styles["bbodyr"]),
             Paragraph(("Denis Frre"), style=styles["bbody"])],

            [Paragraph(("<b>Conference Compere</b>"), style=styles["bbodyr"]),
             Paragraph(("Tim Couper"), style=styles["bbody"])],

            [Paragraph(("<b>Track Chairs</b>"), style=styles["bbodyr"]),
             Paragraph(("""<i>Without them we wouldn't have over 70 talks and
             tutorials! "Thou mighty prophets; --Be nice to them during the
             conference</i>"""), style=styles["bbody"])],

            [Paragraph(("<i>Lightning Talks, Open Space, Bofs</i>"), style=styles["bbodyrs"]),
             Paragraph(("Anna Ravenscroft; Moshe Zadka"), style=styles["bbody"])],

            [Paragraph(("<i>Python Frameworks Track</i>"), style=styles["bbodyrs"]),
             Paragraph(("Martijn Faassen"), style=styles["bbody"])],

            [Paragraph(("<i>Python Language Track</i>"), style=styles["bbodyrs"]),
             Paragraph(("Michael Hudson"), style=styles["bbody"])],

            [Paragraph(("<i>Python in Business Track</i>"), style=styles["bbodyrs"]),
             Paragraph(("Tim Couper; Marc-Andre Lemburg"), style=styles["bbody"])],

            [Paragraph(("<i>Python in Science & Industry Track</i>"), style=styles["bbodyrs"]),
             Paragraph(("Nicolas Chauvat"), style=styles["bbody"])],

            [Paragraph(("<i>Zope Track</i>"), style=styles["bbodyrs"]),
             Paragraph(("Heimo Laukkanen; Paul Everitt"), style=styles["bbody"])],

            [Paragraph(("<b>Local Organisation Team</b>"), style=styles["bbodyr"]),
             Paragraph(("""<i>Without them, there wouldn't be a conference building,
             no food, no internet room, no t-shirts, ... and most important:
             are taking the financial risk!; --Make it easy for them at the
             conference</i>"""), style=styles["bbody"])],

            [Paragraph((""), style=styles["bbodyr"]),
             Paragraph(("""Denis Frre; Franois Gillet; Vincent Maton; Alain Meurant;
             the whole Aragne, P3B and OS3B team."""), style=styles["bbody"])],

            [Paragraph(("<b>Website</b>"), style=styles["bbodyr"]),
             Paragraph(("""<i>Keeping everybody up-to-date. Providing
             interesting info,...</i>"""), style=styles["bbody"])],

            [Paragraph(("<b></b>"), style=styles["bbodyr"]),
             Paragraph(("Tom Deprez; Vincent Maton; Jean-Franois Roche; Joachim Schmitz"), style=styles["bbody"])],

            [Paragraph(("<i>Registration and Database</i>"), style=styles["bbodyrs"]),
             Paragraph(("Aixtraware"), style=styles["bbody"])],

            [Paragraph(("<i>Hosting</i>"), style=styles["bbodyrs"]),
             Paragraph(("Amaze"), style=styles["bbody"])],

            [Paragraph(("<b>Graphics</b>"), style=styles["bbodyr"]),
             Paragraph(("<i>Who made everything pleasing for the eyes</i>"), style=styles["bbody"])],

            [Paragraph((""), style=styles["bbodyr"]),
             Paragraph(("Vincent Maton"), style=styles["bbody"])],

            [Paragraph(("<b>Daily IRC meetings club</b>"), style=styles["bbodyr"]),
             Paragraph(("""<i>Who offered big help and support, gave a lot of ideas,
             discussed a lot of issues, provided contact points, fixed very important
             things; In one sentence : "they worked their butts off, be them ever
             grateful"</i>"""), style=styles["bbody"])],

            [Paragraph((""), style=styles["bbodyr"]),
             Paragraph(("""Laura Creighton; Tom Deprez; Martijn Faassen;
             Michael Hudson; Anna Ravenscroft; Moshe Zadka"""), style=styles["bbody"])],

            [Paragraph(("<b>Proofreading</b>"), style=styles["bbodyr"]),
             Paragraph(("<i>Who made sure we didn't upload too many spelling mistakes</i>"), style=styles["bbody"])],

            [Paragraph((""), style=styles["bbodyr"]),
             Paragraph(("Paul Everitt; Anna Ravenscroft"), style=styles["bbody"])],

            [Paragraph(("<b>Brochure</b>"), style=styles["bbodyr"]),
             Paragraph(("""<i>Without them, we wouldn't have such a nice brochure as we are
                        now looking at.</i>"""), style=styles["bbody"])],

            [Paragraph((""), style=styles["bbodyr"]),
             Paragraph(("Reportlab"), style=styles["bbody"])]

            ],
                 colWidths=[2*inch,4.25*inch],
                 style=None,
                 splitByRow=1,
                 repeatRows=0,
                 repeatCols=0)
    t.setStyle(TableStyle([('ALIGN',(0,0),(-1,-1),'LEFT'),
                           ('VALIGN',(0,0),(-1,-1),'TOP'),]))
    story.append(t)

    newPage()
    story.append(Paragraph('Credits', style=styles["h2"]))

    t = Table(
        [

            [Paragraph(("<b>Speakers</b>"), style=styles["bbodyr"]),
             Paragraph(("""<i>Without them there wouldn't be much to see at
             EuroPython</i>"""), style=styles["bbody"])],

            [Paragraph((""), style=styles["bbody"]),
             Paragraph(("""Albertas Agejevas; Steve Alexander; Francesc Alted; Xavier Antoviaque;
             Harald Armin Massa; Ludovic Aubry; kit Blake; Brad Bollenbach; Robert Boulanger;
             Benjamin Bruheim; David Chan; Pravir Chawdhry; Francesco Ciriaci; Simon Eisenmann;
             Paul Everitt; Martijn Faassen; Stefane Fermier; Rasmus H. Fogh; Francesco Garelli;
             Marius Gedminas; Dinu Gherman; Francis Glassborow; Duncan Grisby; Jrgen Hermann;
             Bernhard Herzog; Stefan H. Holek; Dirk Holtwick; Michael Hudson; Juan David Ibez Palomar;
             Jack Jansen; Andreas Jung; Nik Klever; Wojciech Kosinski; Heimo Laukkanen;
             Marc-Andr Lemburg; Alexander Limi; Niels Mache; Alex Martelli; Klaus G. Muller;
             Federico Nati; Ronald Oussoren; Christophe Perrin; John Pinner; Anna Ravenscroft;
             Roeland Rengelink; Thomas Reulbach; Leonard Richardson; Armin Rigo; Andy Robinson;
             Maik Rder; Guido van Rossum; Just van Rossum; Benjamin Saller; Alan James Salmoni;
             Ulrich Schneider; Vincenzo Di Somma; Aroldo Souza-Leite; Andrew Smart; Jean-Paul Smets;
             Duncan Smith; Konstantin Teplinskiy; Phil Thompson; Darrin Tisdale; Chistian Tismer;
             Daniel Veillard; Joachim Werner; Torsten Will; John Wilson; Moshe Zadka; Christian Zagrodnick
"""), style=styles["bbody"])],

            [Paragraph(("<b>Visitors</b>"), style=styles["bbodyr"]),
             Paragraph(("""<i>Yes, without you, all our efforts would have been wasted and
             speakers would have a very silent audience... Thank you for supporting
             EuroPython 2003!</i>"""), style=styles["bbody"])],

            [Paragraph((""), style=styles["bbody"]),
             Paragraph(("""And finally, a big "thank you" to all of you who attended the
             conference!"""), style=styles["bbody"])]

            ],
                 colWidths=[2*inch,4.25*inch],
                 style=None,
                 splitByRow=1,
                 repeatRows=0,
                 repeatCols=0)
    t.setStyle(TableStyle([('ALIGN',(0,0),(-1,-1),'LEFT'),
                           ('VALIGN',(0,0),(-1,-1),'TOP'),]))
    story.append(t)

    story.append(Paragraph("How this brochure was made", style=styles["h3a"]))
    story.append(Paragraph("""
This brochure was made using ReportLab software - both the free open
source toolkit and the commercial products.
""", style=styles["bbody"]))
    story.append(Paragraph("""
The initial information came from a variety of filtered sources -
including a Zope database. The cover, template for the inside pages
and all the adverts were submitted as PDF files. The ReportLab product
PageCatcher was used to 'catch' these pages, allowing us to manipulate
and reuse them in the final brochure. All this was integrated into
this brochure using the open source ReportLab toolkit (and the Python
programming language).
""", style=styles["bbody"]))
    story.append(Paragraph("""
These techniques meant that the brochure could be regenerated when
needed, and kept constantly up-to-date despite the inevitable wording
and timetable changes.
""", style=styles["bbody"]))
    story.append(Paragraph("""
For more information about ReportLab, the reportlab open source PDF
toolkit and PageCatcher (and our other commercial products), go to
www.reportlab.com.
""", style=styles["bbody"]))


def getReportLabSpin():
    story.append(Paragraph("How this brochure was made", style=styles["h3a"]))
    story.append(Paragraph("""
This brochure was made using ReportLab software - both the free open
source toolkit and the commercial products.
""", style=styles["bbody"]))
    story.append(Paragraph("""
The initial information came from a variety of filtered sources -
including a Zope database. The cover, template for the inside pages
and all the adverts were submitted as PDF files. The ReportLab product
PageCatcher was used to 'catch' these pages, allowing us to manipulate
and reuse them in the final brochure. All this was integrated into
this brochure using the open source ReportLab toolkit (and the Python
programming language).
""", style=styles["bbody"]))
    story.append(Paragraph("""
These techniques meant that the brochure could be regenerated when
needed, and kept constantly up-to-date despite the inevitable wording
and timetable changes.
""", style=styles["bbody"]))
    story.append(Paragraph("""
For more information about ReportLab, the reportlab open source PDF
toolkit and PageCatcher (and our other commercial products), go to
www.reportlab.com.
""", style=styles["bbody"]))

def blankPage(c):
    newPage()

def newPage():
    story.append(PageBreak())

def myOnDrawCB(canv,kind,label):
    l2s = ["Paul F. Dubois",
           "Eric Raymond",
           "Itamar Shtull-Trauring",
           "Paul Everitt",
           "Steve Alexander & Stephan Richter",
           "Alex Martelli",
           "Jim Fulton",
           "Juergen Hermann",
           "Laura Creighton",
           "Moshe Zadka",
           "Michael Hudson",
           "Marc-Andre Lemburg",
           "Martijn Faassen",
           "Tim Couper",
           "Phil Thompson",
           "Duncan Grisby",
           "Nicolas Chauvat",
           "Paul Everitt",
           "Day 1 - AM",
           "Day 1 - PM",
           "Day 2 - AM",
           "Day 2 - PM",
           "Day 3 - AM",
           "Day 3 - PM"]
    if label in l2s:
        level=2
    else:
        level=1
    #print 'myOnDrawCB(%s)'%kind, 'Page number=', canv.getPageNumber(), 'label value=', label
    canv.bookmarkPage(label)
    canv.addOutlineEntry(label, label, level, None)

class MyDocTemplate(BaseDocTemplate):
    "The document template used for all PDF documents."

    _invalidInitArgs = ('pageTemplates',)

    def __init__(self, filename, **kw):
        frame1 = Frame(2.5*cm, 2.5*cm, 15*cm, 25*cm, id='F1')
        self.allowSplitting = 1
        apply(BaseDocTemplate.__init__, (self, filename), kw)
        #template = PageTemplate('normal', [frame1], myMainPageFrame)
        #self.addPageTemplates(template)

    def afterFlowable(self, flowable):
        "Registers TOC entries."
        if flowable.__class__.__name__ == 'Paragraph':
            styleName = flowable.style.name
            if styleName[:7] == 'Heading':
                # Register TOC entries.
                try:
                    level = int(styleName[7:])
                    text = flowable.getPlainText()
                    pageNum = self.page
                    if level<4:
                        #don't need greater detail on something this short
                        self.notify('TOCEntry', (level, text, pageNum))
                except:
                    pass #we occasionally use 'heading3a'

def makeTocHeaderStyle(level, delta, epsilon, fontName='Helvetica'):
    "Make a header style for different levels in the Table of Contents."

    PS = ParagraphStyle
    size = 12
    style = PS(name = 'Heading' + str(level),
               fontName = fontName,
               fontSize = size,
               leading = size*1.2,
               spaceBefore = size/4.0,
               spaceAfter = size/8.0,
               firstLineIndent = -epsilon,
               leftIndent = level*delta + epsilon)

    return style

def makeDoc(tracks=None,interviews=None, genBy=None, genFor=None, fn='brochure.pdf'):
    """function to actually construct the document out of the
    individual sections."""

    global story

    if verbose: log("\t started function 'makeDoc'")

    def myCanvasMaker(fn,**kw):
        from reportlab.pdfgen.canvas import Canvas
        k2={"pageCompression":1,"pagesize":A4}
        kw.update(k2)
        canv = apply(Canvas,(fn,),kw)
        # attach our callback to the canvas
        canv.myOnDrawCB = myOnDrawCB
        return canv

    doc = MyDocTemplate(fn,showBoundary=0)

    # special - for cover page only
    frameS = Frame(35, 230, 510, 165, id='personalize')

    #normal frame - single column
    frameT = Frame(doc.leftMargin, doc.bottomMargin, doc.width, doc.height, id='normal')

    #Two Columns
    frame1 = Frame(doc.leftMargin, doc.bottomMargin, doc.width/2-6, doc.height, id='col1')
    frame2 = Frame(doc.leftMargin+doc.width/2+6, doc.bottomMargin, doc.width/2-6,
                   doc.height, id='col2')

    #Two Columns plus header frame
    framehead = Frame(doc.leftMargin, (doc.bottomMargin+doc.height)-28, doc.width, 32, id='header', showBoundary=0)
    frame1h = Frame(doc.leftMargin, doc.bottomMargin, doc.width/2-6, doc.height-32, id='col1h', showBoundary=0)
    frame2h = Frame(doc.leftMargin+doc.width/2+6, doc.bottomMargin, doc.width/2-6,
                   doc.height-32, id='col2h', showBoundary=0)

    doc.addPageTemplates([PageTemplate(id='First',frames=frameS, onPage=myTitlePage),
                          PageTemplate(id='OneCol',frames=frameT, onPage=myLaterPages),
                          PageTemplate(id='TwoCol',frames=[frame1,frame2], onPage=myLaterPages),
                          PageTemplate(id='TwoColPlus',frames=[framehead,frame1h,frame2h], onPage=myLaterPages),
                          PageTemplate(id='AdPage',frames=frameT, onPage=myAdsPage),
                          PageTemplate(id='FloorPlanPage',frames=frameT, onPage=myFloorPlanPage),
                          ])

    story = []

    getCoverPage(genBy, genFor)
    getTOC()
    getWelcomeLetter()
    #getListOfOrganizers()
    getTimetable()
    getTalkDetails(tracks)
    getInterviews(interviews)
    getFloorPlan()
    getGeneralInfo()
    getTravelInfo()
    getTransportationInfo()
    getPlacesToMeet()
    getContactsPage()
    getCredits()

    if verbose: log("\tstarting 'multibuild'")
    doc.multiBuild(story,canvasmaker=myCanvasMaker)
    if verbose: log("\t'multibuild' complete\n")
    if verbose: log("\tfunction 'makeDoc' complete\n")

    if verbose: print "wrote %s" % fn


if __name__ == '__main__':
    if len(sys.argv)>1:
        sys.argv = sys.argv[1:]
    if '-verbose' in sys.argv or '-v' in sys.argv:
        verbose=1
    elif verbose:
        verbose=1 # if it's already 1 from rl_config
    else:
        verbose=0
    if '-minimal' in sys.argv or '-m' in sys.argv:
        MINIMAL = 1
    else:
        MINIMAL = 0
    if '-pause' in sys.argv or '-p' in sys.argv:
        PAUSE = 1
    else:
        PAUSE = 0

    if MINIMAL:
        makeDoc(tracks=[],interviews=[], fn='brochure_minimal.pdf')
    else:
        makeDoc()
    if verbose:
        print "\nTotal time taken: %2.3f seconds" % elapsed
